/* Programme pour tester le bon fonctionnement des modules de pilotage des moteurs.
   Les moteurs font un tour dans un sens et un tour dans l'autre sans pause entre les deux.
   En plaçant un B.P. entre A0 et GND avec une 1kohm reliée au +5V on peut déclencher
   une pause en mode VEILLE. Le B.P. détecté provoque la veille et temporise durant deux secondes
   pour permettre de relâcher le bouton poussoir. On cale alors manuellement le moteur "au zéro".
   Puis, pour relancer le processus de rotations alternées on clique une deuxième fois sur le B.P.
   À tout moment un appel avec le B.P. permet de vérifier la non perte de pas, le moteur
   passant en veille après un tour complet doit se trouver sur sa référence initiale.
   Le moteur sur X et le moteur sur Y sont en mode "Direct" car le déplacement élémentaire sera
   de 0,2mm si la poulie fait 13 mm de diamètre moyen.
   SW1 SW2 SW2 EFFET
   ON  ON  OFF Pas à pas direct : 200 pas par tour.
    0   0   1  Pas de division.
   
DIFFERENTS BRANCHEMENTS :
 D5 -> Moteurs sous énergie.
 D6 -> Direction sur X.
 D7 -> Impulsion sur X.
 D8 -> Direction sur Y.
 D9 -> Impulsion sur Y.
       
                                               PGM                   RAM dynamique
 PGM de génération PWM.                    1402 Octets (4%)      11 Octets (0%)    */


//--------------------- Déclarations pour le matériel -----------------------

//--------------------- Définition des broches d'E/S ------------------------
#define Moteurs_actifs 5 // D5 valide ou place au repos les deux moteurs.
#define Sens_moteur_X 6  // D6 impose le sens de rotation au moteur X'X.
#define Un_pas_sur_X  7  // D7 génère les mouvements sur X'X.
#define Sens_moteur_Y 8  // D8 impose le sens de rotation au moteur Y'Y.
#define Un_pas_sur_Y  9  // D96 génère les mouvements sur Y'Y.
#define BP_VEILLE  14    // A0 permet de forcer le mode veille pour caler le moteur à "zéro".


//------------------------ Constantes du programme --------------------------

//--------------------------- Variables du programme ---------------------------

void setup() {
  pinMode(Moteurs_actifs, OUTPUT);
  pinMode(Sens_moteur_X, OUTPUT); digitalWrite(Sens_moteur_X, LOW);
  pinMode(Un_pas_sur_X, OUTPUT); digitalWrite(Un_pas_sur_X, LOW);
  pinMode(Sens_moteur_Y, OUTPUT); digitalWrite(Sens_moteur_Y, LOW);
  pinMode(Un_pas_sur_Y, OUTPUT); digitalWrite(Un_pas_sur_Y, LOW);
  VEILLE(false);}

void loop() {
  for(int Nb=0; Nb<200; Nb++) {X_plus(); Y_plus();}
  Tester_le_BP_A0();
  for(int Nb=0; Nb<2 00; Nb++) {X_moins(); Y_moins();}
  Tester_le_BP_A0();}

void X_plus() {
  digitalWrite(Sens_moteur_X, HIGH); 
  digitalWrite(Un_pas_sur_X, HIGH); delay(1);
  digitalWrite(Un_pas_sur_X, LOW);}

void X_moins() {
  digitalWrite(Sens_moteur_X, LOW); 
  digitalWrite(Un_pas_sur_X, HIGH); delay(1);
  digitalWrite(Un_pas_sur_X, LOW);}

void Y_plus() {
  digitalWrite(Sens_moteur_Y, HIGH); 
  digitalWrite(Un_pas_sur_Y, HIGH); delay(1);
  digitalWrite(Un_pas_sur_Y, LOW);}

void Y_moins() {
  digitalWrite(Sens_moteur_Y, LOW); 
  digitalWrite(Un_pas_sur_Y, HIGH); delay(1);
  digitalWrite(Un_pas_sur_Y, LOW);}
  
void VEILLE(boolean REPOS) {digitalWrite(Moteurs_actifs, REPOS);}

void Tester_le_BP_A0() {
  if (analogRead(0) < 710) {
       VEILLE(true); delay(2000);
       while (analogRead(0) > 710) delay(10); VEILLE(false);}}
  
