/* Programme qui liste le contenu de la carte SD sur OLED comme pour P05.
   Intègre le codeur rotatif et ajoute l'indexation des dix fichiers potentiels.
   Si Clic central sur fichier "---" BIP d'alerte.
   Si Clic central sur fichier "oui" pas d'action.
   >>>> Commentaires sur ce programme en fin de listage.

Branchements à effectuer :
--------------------------
 A2 : Bouton central du codeur rotatif.
 A3 : LED verte du clavier.
 A4 : SDA OLED.
 A5 : SCL OLED.
 Codeur rotatif B : Broche D2. >>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
 Codeur rotatif A : Broche D3. >>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
                               >>>>>>>>>>>>>>>>>> obtenir un sens de variation correct.
 D4 : BUZZER actif.
 D10 : CS   
 D11 : MOSI
 D12 : MISO
 D13 : SCK
                                                                    PGM               RAM dynamique
Intégration du listage de la carte sur OLED.                11348 Octets (36%)     342 Octets (16%)
Codeur rotatif + BUZZER + LED verte + BP central :          12508 Octets (40%)     357 Octets (17%) */

//--------------------- Déclarations pour le matériel -----------------------
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)
#include "PetitFS.h"
FATFS fs;     /* File system object */
//------------- Définition des broches d'entrées analogiques ----------------
#define BP_central 16 // A2 pour lire le bouton central.

//------------- Définition des broches de sorties analogiques ---------------
#define LED_Clavier 17 // A3 pour allumer la LED verte du clavier.

//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_A 3    // D2 : DT  >>>>>>>>>>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
#define Codeur_B 2    // D3 : CLK >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
#define BUZZER 4      // D4 : BUZZER actif.
#define chipSelect 10
//----------------------- Déclaration des variables -------------------------
boolean BP_central_a_traiter;
boolean Lecteur_OK;
boolean Existe[10];
char Indice[10] = {'0','1','2','3','4','5','6','7','8','9'};
char Nom_Fichier[9] = "IMGN.GCO";

boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean Sens_positif; // Sens de rotation.
volatile boolean Rotation;  // Indique une activation du codeur rotatif.

byte Num_Img_indexee;
//---------------------------------------------------------------------------

void setup() {
  //---------------------- Définition des vecteurs d'interruption ----------------------
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  // Broche de l'interruption 1 (E/S D3) :
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(chipSelect, OUTPUT); digitalWrite(chipSelect,HIGH);
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW);
  pinMode(11,INPUT); pinMode(12,INPUT); pinMode(13,INPUT);
  pinMode(LED_Clavier, OUTPUT); digitalWrite(LED_Clavier, LOW);
  BP_central_a_traiter = false; digitalWrite(BP_central, HIGH); // Valide R PUL-UP sur A2.
  Front_sur_A(); Front_sur_B(); Num_Img_indexee = 9;
  Lecture_contenu_SD(); // Vérifie présence carte et initialise le tableau des images présentes.
  Liste_contenu_SD();}

void loop() {
  //======================== Traite le B.P. central ======================
  Teste_activation_BP_central(); if (BP_central_a_traiter) {BP_central_a_traiter = false; Traite_BP_Central();}
  //--------------------- Traite la rotation du codeur incrémental ---------------------
  if (Rotation) TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();}

  //-------------------- Routines de servitude pour traiter le capteur incrémental -------------------
  void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 100 expérimentale.
  
  void Front_sur_A(){ // Interruption quand A change d'état avant B.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
    if (digitalRead(Codeur_A) != A_set ) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();} } }
  
  void Front_sur_B(){ // Interruption quand B change d'état avant A.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
    if( digitalRead(Codeur_B) != B_set ) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();} } }
  //--------------------------------------------------------------------------------------------------

void BIP() {digitalWrite(BUZZER, HIGH); delay(100); digitalWrite(BUZZER, LOW);}

void Debut_Nom(byte X, byte Y, byte N) {u8g.setPrintPos(X, Y); u8g.print("Img"); u8g.print(N);}
void Affiche_OUI() {u8g.print(" oui");}
void Affiche_non() {u8g.print(" ---");}

void Lecture_contenu_SD() {
  Lecteur_OK = true; if (pf_mount(&fs))Lecteur_OK = false;
  if (!Lecteur_OK) {// Teste la présence carte et message d'erreur sur boucle infinie si non OK :
     u8g.firstPage(); do {u8g.setPrintPos(15, 40); u8g.print("Pb sur lecteur SD.");} while(u8g.nextPage());
     BIP(); INFINI: goto INFINI;}
  else // Teste la présence éventuelle des dix fichiers possibles et en fait la liste :
     for (byte I=0; I<10; I++) {Nom_Fichier[3] = Indice[I]; Existe[I] = !pf_open(Nom_Fichier);}}

void Liste_contenu_SD() {// Ne teste plus la présence carte et liste son "contenu images" en aveugle
                         // en utilisant le tableau de booléens Existe[10] initialisé sur RESET.
  byte X,Y;
  // Afficher la liste :
  u8g.firstPage(); do {
     Y = 10; X = 0;
     for (byte I=0; I<10; I++) {
          if (I > 4) X = 66; if (I == 5) Y = 10;
          Debut_Nom(X,Y,I); if (Existe[I]) Affiche_OUI(); else Affiche_non();  Y = Y + 13;
          if (I == Num_Img_indexee) u8g.print(" <");}} 
  while(u8g.nextPage());}
  
void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  INT_active = true; // Armer un nouveau comptage.
  Rotation = false; // Prise en compte du traitement.
  if (Sens_positif) {Num_Img_indexee++; if (Num_Img_indexee == 10) Num_Img_indexee = 0;}
  else {if (Num_Img_indexee > 0) Num_Img_indexee--; else Num_Img_indexee = 9;}
  Liste_contenu_SD();}

void Valide_le_fichier_indexe() {} // provisoirement ne fait rien.

void Traite_BP_Central() {// B.P. central activé dans la boucle de base.
     if (Existe[Num_Img_indexee]) Valide_le_fichier_indexe(); else BIP();}

void Teste_activation_BP_central() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     // ========== Attendre le relacher du B.P. central. ==========
     digitalWrite(LED_Clavier, HIGH); // Allume la LED.
     while (analogRead(BP_central) < 512); // Attendre le relâché.
     digitalWrite(LED_Clavier, LOW);
     // ===========================================================
     BP_central_a_traiter = true;}}

/* Commentaires sur ce programme.
   ==============================

Problèmes liés au lecteur de carte SD RAM :
-------------------------------------------
  Si la lecture est correcte, la liste des fichiers valides de Img0.gco à Img9.gco est établies jusqu'au
  prochain changement de carte qui imposera d'effectuer un RESET. C'est pour des raisons d'optimisation du
  logiciel que ces seules dix références ont été envisagées. Si la lecture ne se fait pas correctement,
  l'écran OLED affiche un texte d'alerte et le programme boucle sur INFINI jusqu'au prochain RESET.
  REMARQUE : La lecture de la carte SDRAM est relativement longue, ne l'effectuer qu'une seule fois va
             donc dans le sens de la convivialité d'utilisation opérationnelle

Problèmes liés au codeur rotatif :
----------------------------------
  Ce type de codeur engendre une rotation fantôme positive et négative à sa première utilisation.
  Pour parer cette difficulté, les instructions suivantes sont ajoutées dans les initialisations :
  Front_sur_A(); Front_sur_B(); Num_Img_indexee = 9;
  Cette ligne de code effectue une "rotation positive artificielle". Du coup Num_Img_indexee prend
  la valeur 0 en tête de listage et le fonctionnement du codeur rotatif est correct. */
  


