/* Directement issu de P08 ce programme ajoute la gestion d'un clavier à 5 touches.
   On retrouve tous les branchements et les fonctions de P08 et on complète avec A0 et A1.
   Le programme se comporte comme P08. Si un B.P. est cliqué durant "la boucle de base",
   sa référence est affichée sur OLED jusqu'à une autre action. Le B.P. n'interromp pas un
   listage de fichier GCO en cours sur OLED.
   NOTE : Dans ce démonstrateur les lignes de remarque en code GCO sont ignorées.
   >>> La broche A3 est rendue disponible et la LED Clavier se branche sur la sortie S0
 
Branchements à effectuer :
--------------------------
 A0 : Ligne clavier de 3 touches.
 A1 : Ligne clavier de 2 touches.
 A2 : Bouton central du codeur rotatif.
 A4 : SDA OLED et Multiplexeur.
 A5 : SCL OLED et Multiplexeur.
 D2 : Codeur rotatif B >>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
 D3 : Codeur rotatif A >>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
                       >>>>>>>>>>>>>>>>>>>>>>> obtenir un sens de variation correct.
 D4 :  BUZZER actif.
 D10 : CS   
 D11 : MOSI
 D12 : MISO
 D13 : SCK
 
 Multiplexeur S0 : Branchement le la LED bleue du CLAVIER.
 Multiplexeur S15 : Sortie de pilotage PWM du LASER.
 (L'affectation des sorties est issue du dernier démonstrateur et construite pour séparer les
  picots utilisés et ainsi faciliter les branchements.)
 
                                                                    PGM               RAM dynamique
Programme initial clone de p08.                             16422 Octets (53%)     600 Octets (29%)
Ajout des routines du clavier en affichage du n° de B.P :   16602 Octets (54%)     603 Octets (29%)
LED clavier sur S0 et attente d'un BP passé sur clavier :   16664 Octets (54%)     603 Octets (29%)
 */

//--------------------- Déclarations pour le matériel -----------------------
#include <Wire.h> // Gère les protocole I2C sur A4 et A5 pour le Multiplexeur.
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)
#include "PetitFS.h"
FATFS fs;     // Zone de travail pour le volume SD : Objet système.
#include <Adafruit_PWMServoDriver.h> // déclaration du Multiplexeur.
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

//------------- Définition des broches d'entrées analogiques ----------------
#define Ligne_trois_touches 14 // A0 pour lire le clavier à 3 BP. 
#define Ligne_deux_touches 15 // A1 pour lire le clavier à 2 BP. 
#define BP_central 16 // A2 pour lire le bouton central.
//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_A 3
#define Codeur_B 2
#define BUZZER 4
#define chipSelect 10
//------------------------ Constantes du programme --------------------------
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.

//----------------------- Déclaration des variables -------------------------
boolean BP_central_a_traiter;
boolean Lecteur_OK;
boolean Existe[10];
char Indice[10] = {'0','1','2','3','4','5','6','7','8','9'};
char Nom_Fichier[9] = "IMGN.GCO";

boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean Sens_positif; // Sens de rotation.
volatile boolean Rotation;  // Indique une activation du codeur rotatif.

byte Num_BP; // Désigne le bouton poussoir activé sur le clavier.
byte Num_Img_indexee;

int Numerisation_BP; // Valeur d'une touche clavier retournée par la CAN.
int Pulse = 5; // Rapport cyclique du signal de pilotage du LASER.

//---------------------------------------------------------------------------

void setup() {
  //---------------------- Définition des vecteurs d'interruption ----------------------
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  // Broche de l'interruption 1 (E/S D3) :
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pwm.begin(); // Initialise le module multiplexeur.
  pwm.setPWMFreq(1000); // Fréquence particulière 1000Hz pour piloter le module LASER.
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(chipSelect, OUTPUT); digitalWrite(chipSelect,HIGH);
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW);
  pinMode(11,INPUT); pinMode(12,INPUT); pinMode(13,INPUT);
  BP_central_a_traiter = false; digitalWrite(BP_central, HIGH); // Valide R PUL-UP sur A2.
  Front_sur_A(); Front_sur_B(); Num_Img_indexee = 9;
  Bilan_contenu_SD();} // Vérifie présence carte et initialise le tableau des images présentes.
  // Pas besoin d'afficher le contenu car la rotation "artificielle" du B.R. provoque l'affichage. 

void loop() {
  //======================== Traite le B.P. central ======================
  Teste_activation_BP_central(); if (BP_central_a_traiter) {BP_central_a_traiter = false; Traite_BP_Central();}
  //--------------------- Traite la rotation du codeur incrémental ---------------------
  if (Rotation) TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();
  //---------------------- Traiter la requête de l'un des cinq B.P. --------------------
  Tester_le_BP_A0();
  if (Num_BP > 0) {
      ATTENDRE_le_relacher();
      u8g.firstPage(); do {u8g.setPrintPos(40, 40); u8g.print(Num_BP);} while(u8g.nextPage());
      Num_BP = 0;} // Action sur le B.P. prise en compte.
  //----------------------- Sortie 15 : PWM de rapport 0% à 50% ------------------------
  Pulse++; if (Pulse == 2000) Pulse = 0; pwm.setPWM(15, 0, Pulse); delay(5);}

  //-------------------- Routines de servitude pour traiter le capteur incrémental -------------------
  void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 100 expérimentale.
  
  void Front_sur_A(){ // Interruption quand A change d'état avant B.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
    if (digitalRead(Codeur_A) != A_set ) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();} } }
  
  void Front_sur_B(){ // Interruption quand B change d'état avant A.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
    if( digitalRead(Codeur_B) != B_set ) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();} } }
  //--------------------------------------------------------------------------------------------------

void BIP() {digitalWrite(BUZZER, HIGH); delay(100); digitalWrite(BUZZER, LOW);}

void Affiche_ERR_et_attente_BP(char* msg) {
  u8g.firstPage(); do {u8g.setPrintPos(15, 40); u8g.print(msg);} while(u8g.nextPage()); BIP();
  ATTENDRE_un_BP();}

void Construire_Nom_Fichier() {Nom_Fichier[3] = Indice[Num_Img_indexee];}
void Debut_Nom(byte X, byte Y, byte N) {u8g.setPrintPos(X, Y); u8g.print("Img"); u8g.print(N);}
void Affiche_OUI() {u8g.print(" oui");}
void Affiche_non() {u8g.print(" ---");}

void Bilan_contenu_SD() {
  Lecteur_OK = true; if (pf_mount(&fs))Lecteur_OK = false;
  if (!Lecteur_OK) // Teste la présence carte et message d'erreur sur boucle infinie si non OK :
     Affiche_ERR_et_attente_BP("Pb sur lecteur SD.");
  else // Construire la table de présence des dix fichiers possibles :
     for (byte I=0; I<10; I++) {Nom_Fichier[3] = Indice[I]; Existe[I] = !pf_open(Nom_Fichier);}}

void Liste_contenu_SD() {// Ne teste plus la présence carte et liste son "contenu images" en aveugle
  byte X,Y;              // en utilisant le tableau de booléens Existe[10] initialisé sur RESET.
  // Afficher la liste de l'état des fichiers sur OLED :
  u8g.firstPage(); do {
     Y = 10; X = 0;
     for (byte I=0; I<10; I++) {
          if (I > 4) X = 66; if (I == 5) Y = 10;
          Debut_Nom(X,Y,I); if (Existe[I]) Affiche_OUI(); else Affiche_non();  Y = Y + 13;
          if (I == Num_Img_indexee) u8g.print(" <");}} 
  while(u8g.nextPage());}
  
void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  INT_active = true; // Armer un nouveau comptage.
  Rotation = false; // Prise en compte du traitement.
  if (Sens_positif) {Num_Img_indexee++; if (Num_Img_indexee == 10) Num_Img_indexee = 0;}
  else {if (Num_Img_indexee > 0) Num_Img_indexee--; else Num_Img_indexee = 9;}
  Liste_contenu_SD();}

void Traite_la_ligne_du_fichier(char Tampon[40]) {
  u8g.firstPage(); do {u8g.setPrintPos(0, 40); u8g.print(Tampon);} while(u8g.nextPage());}

void Valide_le_fichier_indexe() { // Traite le contenu du fichier ligne à ligne.
  uint8_t buf[40]; // Tampon de travail.     >>>>>>>>>>>>>> Doivent avoir la taille de la plus grande ligne
  char Tampon[40]; // Tampon de traitement.  >>>>>>>>>>>>>> pour ne pas décaleur l'affichage sur le moniteur.
  unsigned int pos = 0; // Position courante dans le fichier.
  Construire_Nom_Fichier();
  if (pf_open(Nom_Fichier)) Affiche_ERR_et_attente_BP("Erreur_ouverture");
  while (1) {// Lecture ligne à ligne du fichier.
    UINT br; // Pointeur de lecture dans le fichier ouvert.
    //On lit le fichier octet par octet en mettant 1 en deuxième paramètre :
    if (pf_read(buf, 1, &br)) Affiche_ERR_et_attente_BP("Erreur_lecture");
    if (br == 0) break;
    char carac = (char)buf[0]; //Pour la lecture, on stocke dans la variable caractere.
    //Si on est sur une fin d'une ligne, affiche la chaîne construite :
    if(carac=='\n' ){
       //Il faut rajouter le caractère de fin " \0 " pour marquer la fin de chaîne.
       Tampon[pos] = '\0';
       pos = 0; //On reinitialise la position à 0 pour une nouvelle lecture recommencer à zéro
       if (Tampon[0] != ';') Traite_la_ligne_du_fichier(Tampon);} // Ignorer les remarques.
    else{ //Si ce n'est pas la fin de ligne, on continue à remplir le Tampon à chaque position :
      Tampon[pos] = (char)buf[0]; pos++;}}
  Liste_contenu_SD();}

void Traite_BP_Central() {// B.P. central activé dans la boucle de base.
  if (Existe[Num_Img_indexee]) Valide_le_fichier_indexe(); else BIP();}

void Teste_activation_BP_central() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     // ========== Attendre le relacher du B.P. central. ==========
     pwm.setPWM(0, 0, 4095); // Allume la LED.
     while (analogRead(BP_central) < 512); // Attendre le relâché.
     pwm.setPWM(0, 0, 0);
     // ===========================================================
     BP_central_a_traiter = true;}}

void Tester_le_Clavier() {// Les valeurs tiennent compte de VCC = 4V car fourni par la carte NANO.
  Num_BP = 0; // Signale aucun B.P. utilisé.
  Numerisation_BP = analogRead(Ligne_trois_touches);
  if (Numerisation_BP < 758) {Num_BP = 1;
      if (Numerisation_BP < 508)  Num_BP = 2; if (Numerisation_BP < 230)  Num_BP = 3;} 
  Numerisation_BP = analogRead(Ligne_deux_touches);
  if (Numerisation_BP < 709) {Num_BP = 5; if (Numerisation_BP < 300)  Num_BP = 4;}}

void ATTENDRE_le_relacher() {
  byte SAV_Num_BP = Num_BP;
  byte COMPTEUR = 0; delay(100); // Attendre_la_stabilisation du travail.
  pwm.setPWM(0, 0, 4095);
  while (COMPTEUR < NB_tests_antirebonds) {// Attendre_la_stabilisation du repos :
         COMPTEUR++; Tester_le_BP_A0(); if (Num_BP > 0) COMPTEUR = 0;}
  pwm.setPWM(0, 0, 0); Num_BP = SAV_Num_BP;}

void ATTENDRE_un_BP() {
  byte Allume; Num_BP = 0;
  while (Num_BP == 0) {
         if (Allume == 0) Allume = 4095; else Allume = 0; pwm.setPWM(0, 0, Allume);
         Tester_le_BP_A0(); delay(100);}
  ATTENDRE_le_relacher();}

