/* Directement issu de P09 ce programme ajoute l'ANALYSEUR SYNTAXIQUE.
   On retrouve tous les branchements et les fonctions de P09.
   La gestion de D5 est ajoutée, sortie forcée à "1" sur RESET.
   Le témoin bleu du mode VEILLE est géré.
   L'exploration du fichier affiche les données ligne à ligne et le titre.
   La taille des chaînes de caractère passe de 40 à 30.
   
UTILISATION DU PROGRAMME :
==========================
   Sur un RESET il y a lecture de la carte SD et affichage d'un texte d'alerte si le lecteur n'est pas "décodé".
   Si lecteur SD et contenu de la carte sorrect, il y a affichage du bilan des fichiers "ImgN.cgo".
   C'est l'état @ du "menu de base".
   À ce stade @ toute action sur une touche du clavier affiche le n° de son identification.
   À ce stade @ la rotation du codeur affiche le contenu SD et déplace l'index face à chaque "ImgN" "oui" ou "---".
   Le B.P. central du codeur rotatif "valide le fichier" pour l'analyse.
        Valider sur un "---" génère un BIP d'erreur et attend une action.
        valider sur un "oui" provoque le texte "Affichage ?" et attend une touche clavier.
                Cliquer sur PB4 déclenche l'analyse du fichier et affichage de l'état pour chaque ligne. (1)
                Cliquer sur les autres boutons déclenche l'analyse sur un écran noir. (2)
                Durant l'analyse :
                ------------------
                Tourner le codeur rotatif stoppe l'analyse et affiche "FIN de la gravure ?"
                Le BP4 provoque le retour en attente initiale @.
                Le BP2 affiche l'état de la ligne qui vient d'être analysée. Il y a attente d'un BP clavier.
                       Tout BP relance l'analyse avec écran noir ou affichage fonction de l'option choisie en (1) ou (2). 
   
   >>>> Commentaires sur ce programme en fin de listage.
   >>>> Extraire le titre et l'afficher coute  +70 octets de programme et +20 octets en mémoire dynamique.
 
Branchements à effectuer :
--------------------------
 A0 : Ligne clavier de 3 touches.
 A1 : Ligne clavier de 2 touches.
 A2 : Bouton central du codeur rotatif.
 A4 : SDA OLED et Multiplexeur.
 A5 : SCL OLED et Multiplexeur.
 D2 : Codeur rotatif B >>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
 D3 : Codeur rotatif A >>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
                       >>>>>>>>>>>>>>>>>>>>>>> obtenir un sens de variation correct.
 D4 :  BUZZER actif.
 D5 :  Pilote le mode VEILLE du LASER.
 D10 : CS   
 D11 : MOSI
 D12 : MISO
 D13 : SCK
 
 Multiplexeur  S0 : Branchement le la LED verte du CLAVIER.
 Multiplexeur S15 : Branchement d'une LED bleue pour visualiser PWM LASER.
 
                                                                    PGM               RAM dynamique
Programme initial clone de p09.                             16670 Octets (54%)     603 Octets (29%)
Sans l'affichage du titre :                                 18942 Octets (61%)     685 Octets (33%)
Avec l'affichage du titre :   (+70 et +40)                  19012 Octets (61%)     725 Octets (35%)
Avec l'analyseur syntaxique complet et alerte sur ERR :     19130 Octets (62%)     716 Octets (34%)
Interruption du listage par le codeur rotatif :             19518 Octets (63%)     740 Octets (34%)
Traite affichage, IRQ et touche noire + Num ligne :         19710 Octets (64%)     761 Octets (37%)
 */

//--------------------- Déclarations pour le matériel -----------------------
#include <Wire.h> // Gère les protocole I2C sur A4 et A5 pour le Multiplexeur.
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)
#include "PetitFS.h"
FATFS fs;     // Zone de travail pour le volume SD : Objet système.
#include <Adafruit_PWMServoDriver.h> // déclaration du Multiplexeur.
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

//------------- Définition des broches d'entrées analogiques ----------------
#define Ligne_trois_touches 14 // A0 pour lire le clavier à 3 BP. 
#define Ligne_deux_touches 15 // A1 pour lire le clavier à 2 BP. 
#define BP_central 16 // A2 pour lire le bouton central.
//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_A 3
#define Codeur_B 2
#define BUZZER 4
#define VEILLE 5 // Broche qui à l'état haut fait passer le moteur en mode "non alimenté".
#define chipSelect 10
//------------------------ Constantes du programme --------------------------
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.
#define Taille_tampon 30 // Taille des tampons pour les chaînes de caractères.
//----------------------- Déclaration des variables -------------------------
boolean Afficher_les_lignes;
boolean BP_central_a_traiter;
boolean Lecteur_OK;
boolean Syntaxe_correcte;
boolean Existe[10];

char Indice[10] = {'0','1','2','3','4','5','6','7','8','9'};
char Nom_Fichier[9] = "IMGN.GCO";
char Tampon[Taille_tampon];
char Titre[Taille_tampon];

boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean Sens_positif; // Sens de rotation.
volatile boolean Rotation;  // Indique une activation du codeur rotatif.

byte Code_instruction; // Code en sortie de l'analyse syntaxique.
byte Longueur_ligne_de_code; 
byte Num_BP; // Désigne le bouton poussoir activé sur le clavier.
byte Num_Img_indexee;
byte Pourcentage; // Pourcentage de puissance du LASER.

int Numerisation_BP; // Valeur d'une touche clavier retournée par la CAN.
int Pulse = 5; // Rapport cyclique du signal de pilotage du LASER.
unsigned int Num_Ligne;
float X, Y;

//---------------------------------------------------------------------------

void setup() {
  //---------------------- Définition des vecteurs d'interruption ----------------------
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  // Broche de l'interruption 1 (E/S D3) :
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pwm.begin(); // Initialise le module multiplexeur.
  pwm.setPWMFreq(1000); // Fréquence particulière 1000Hz pour piloter le module LASER.
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(chipSelect, OUTPUT); digitalWrite(chipSelect,HIGH);
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW);
  pinMode(VEILLE, OUTPUT); Passer_en_VEILLE(); // Passe les moteurs en Veille et allume la LED bleue.
  Pourcentage = 0; // Puissance minimale sur RESET.
  pinMode(11,INPUT); pinMode(12,INPUT); pinMode(13,INPUT);
  BP_central_a_traiter = false; digitalWrite(BP_central, HIGH); // Valide R PUL-UP sur A2.
  Front_sur_A(); Front_sur_B(); Num_Img_indexee = 9;
  Bilan_contenu_SD();} // Vérifie présence carte et initialise le tableau des images présentes.
  // Pas besoin d'afficher le contenu car la rotation "artificielle" du B.R. provoque l'affichage. 

void loop() {
  //======================== Traite le B.P. central ======================
  Teste_activation_BP_central(); if (BP_central_a_traiter) {BP_central_a_traiter = false; Traite_BP_Central();}
  //--------------------- Traite la rotation du codeur incrémental ---------------------
  if (Rotation) TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();
  //---------------------- Traiter la requête de l'un des cinq B.P. --------------------
  Tester_le_BP_A0();
  if (Num_BP > 0) {
      ATTENDRE_le_relacher();
      u8g.firstPage(); do {u8g.setPrintPos(40, 40); u8g.print(Num_BP);} while(u8g.nextPage());
      Num_BP = 0;}} // Action sur le B.P. prise en compte.

void Teste_activation_BP_central() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     // ========== Attendre le relacher du B.P. central. ==========
     pwm.setPWM(0, 0, 4095); // Allume la LED.
     while (analogRead(BP_central) < 512); // Attendre le relâché.
     pwm.setPWM(0, 0, 0);
     // ===========================================================
     BP_central_a_traiter = true;}}
     
//-------------------- Routines de servitude pour traiter le capteur incrémental -------------------
void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 100 expérimentale.
  
void Front_sur_A(){ // Interruption quand A change d'état avant B.
  if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
  if (digitalRead(Codeur_A) != A_set) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();}}}
  
void Front_sur_B(){ // Interruption quand B change d'état avant A.
  if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
  if (digitalRead(Codeur_B) != B_set) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();}}}
//--------------------------------------------------------------------------------------------------

void BIP() {digitalWrite(BUZZER, HIGH); delay(100); digitalWrite(BUZZER, LOW);}

void ATTENDRE_le_relacher() {
  byte SAV_Num_BP = Num_BP;
  byte COMPTEUR = 0; delay(100); // Attendre_la_stabilisation du travail.
  pwm.setPWM(0, 0, 4095);
  while (COMPTEUR < NB_tests_antirebonds) {// Attendre_la_stabilisation du repos :
         COMPTEUR++; Tester_le_BP_A0(); if (Num_BP > 0) COMPTEUR = 0;}
  pwm.setPWM(0, 0, 0); Num_BP = SAV_Num_BP;}

void Tester_le_BP_A0() {
  Num_BP = 0; // Signale aucun B.P. utilisé.
  Numerisation_BP = analogRead(Ligne_trois_touches);
  if (Numerisation_BP < 742) {Num_BP = 1;
      if (Numerisation_BP < 469)  Num_BP = 2; if (Numerisation_BP < 170)  Num_BP = 3;} 
  Numerisation_BP = analogRead(Ligne_deux_touches);
  if (Numerisation_BP < 695) {Num_BP = 5; if (Numerisation_BP < 263)  Num_BP = 4;}}

void ATTENDRE_un_BP() {
  byte Lumiere; Num_BP = 0;
  while (Num_BP == 0) {
         if (Lumiere == 0) Lumiere = 4095; else Lumiere = 0; pwm.setPWM(0, 0, Lumiere);
         Tester_le_BP_A0(); delay(100);}
  ATTENDRE_le_relacher();}

void Affiche_ERR_et_attente_BP(char* msg) {
  u8g.firstPage(); do {u8g.setPrintPos(5, 35); u8g.print(msg);} while(u8g.nextPage());
  BIP(); ATTENDRE_un_BP();}

void Construire_Nom_Fichier() {Nom_Fichier[3] = Indice[Num_Img_indexee];}
void Debut_Nom(byte X, byte Y, byte N) {u8g.setPrintPos(X, Y); u8g.print("Img"); u8g.print(N);}
void Affiche_OUI() {u8g.print(" oui");}
void Affiche_non() {u8g.print(" ---");}

void Bilan_contenu_SD() {
  Lecteur_OK = true; if (pf_mount(&fs))Lecteur_OK = false;
  if (!Lecteur_OK) // Teste la présence carte et message d'erreur sur boucle infinie si non OK :
     Affiche_ERR_et_attente_BP("Pb sur lecteur SD.");
  else // Construire la table de présence des dix fichiers possibles :
     for (byte I=0; I<10; I++) {Nom_Fichier[3] = Indice[I]; Existe[I] = !pf_open(Nom_Fichier);}}

void Liste_contenu_SD() {// Ne teste plus la présence carte et liste son "contenu images" en aveugle
  byte X,Y;              // en utilisant le tableau de booléens Existe[10] initialisé sur RESET.
  // Afficher la liste de l'état des fichiers sur OLED :
  u8g.firstPage(); do {
     Y = 10; X = 0;
     for (byte I=0; I<10; I++) {
          if (I > 4) X = 66; if (I == 5) Y = 10;
          Debut_Nom(X,Y,I); if (Existe[I]) Affiche_OUI(); else Affiche_non();  Y = Y + 13;
          if (I == Num_Img_indexee) u8g.print(" <");}} 
  while(u8g.nextPage());}
  
void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  INT_active = true; // Armer un nouveau comptage.
  Rotation = false; // Prise en compte du traitement.
  if (Sens_positif) {Num_Img_indexee++; if (Num_Img_indexee == 10) Num_Img_indexee = 0;}
  else {if (Num_Img_indexee > 0) Num_Img_indexee--; else Num_Img_indexee = 9;}
  Liste_contenu_SD();}
  
void Decale_la_chaine() {
  for (byte I = 0; I < Taille_tampon; I++) Tampon[I] = Tampon[I+1];}

void Passe_item_suivant() {while (Tampon[0] != ' ') Decale_la_chaine(); Decale_la_chaine();}
  
void Traite_X() {
  Decale_la_chaine(); X = atof(Tampon);
  while (Tampon[0] != 'S') Decale_la_chaine(); Decale_la_chaine();
  Pourcentage =  atoi(Tampon);}

void Traite_G() {
  boolean G1;
  Decale_la_chaine(); G1 = Tampon[0] == '1'; for (byte I=0; I<3; I++) Decale_la_chaine();
  X = atof(Tampon); Passe_item_suivant; // Pointe 'Y' ou 'S'.
  if (G1) {Decale_la_chaine(); Pourcentage =  atoi(Tampon);}
  else {Decale_la_chaine(); Y = atof(Tampon);Decale_la_chaine(); Decale_la_chaine();
        if (Tampon[0] == 'S') {Decale_la_chaine(); Pourcentage =  atoi(Tampon);}}}

void Moduler_le_LASER() {// Place sur S15 la valeur correspondant à Pourcentage.
  int Pulse = map(Pourcentage, 0, 100, 0, 4050); pwm.setPWM(15, 0, Pulse);}
  
void Passer_en_VEILLE() {digitalWrite(VEILLE, HIGH); pwm.setPWM(1, 0, 4095);};

void Activer_les_moteurs() {digitalWrite(VEILLE, LOW); pwm.setPWM(1, 0, 0); delay(50);}
     // Ci-dessus delai 50mS pour stabilisation des moteurs.
  
void Analyseur_Syntaxique() {
  switch (Tampon[0]) {
    case 'G' : {Traite_G(); break;}
    case 'M' : {Pourcentage = 0;
                if (Tampon[1] == '3') Activer_les_moteurs(); else {Passer_en_VEILLE(); break;}}
    case 'S' : {Pourcentage = 0; break;}
    case 'X' : {Traite_X(); break;} // Le break est impératif pour sauter "defalt" si OK.
    default : {Affiche_ERR_et_attente_BP("Code illegal."); Syntaxe_correcte = false;}}}

void AFFICHE_etat_machine(){
  u8g.firstPage(); do {
  u8g.setPrintPos(0, 10); u8g.print(Titre);
  u8g.setPrintPos(0, 23); u8g.print("Ligne "); u8g.print(Num_Ligne);
  u8g.setPrintPos(0, 36); u8g.print("P = "); u8g.print(Pourcentage);
  u8g.setPrintPos(0, 49); u8g.print("X = "); u8g.print(X,1);
  u8g.setPrintPos(0, 62); u8g.print("Y = "); u8g.print(Y,1);}
  while(u8g.nextPage());}

void Eteindre_Ecran(){u8g.firstPage(); do {} while(u8g.nextPage());}

void Execute_la_ligne_du_fichier() {
  if (Rotation) {// Traitement si on veut interrompre une pyrogravure.
     Rotation = false; // Prise en compte du traitement.
     INT_active = true; // Armer un nouveau comptage.
     Affiche_ERR_et_attente_BP("FIN de la gravure ?");
     if (Num_BP == 4) {Passer_en_VEILLE(); Pourcentage = 0; Moduler_le_LASER(); Syntaxe_correcte = false;}
     if (Num_BP == 2) {AFFICHE_etat_machine(); ATTENDRE_un_BP();} Eteindre_Ecran();}
  if (Afficher_les_lignes) AFFICHE_etat_machine();}

void Traite_le_fichier_indexe() { // Traite le contenu du fichier ligne à ligne.
  boolean Premiere_ligne = true;
  boolean Ligne_a_ignorer;
  Syntaxe_correcte = true; Num_Ligne = 0;
  uint8_t buf[Taille_tampon]; // Tampon de travail.
  unsigned int pos = 0; // Position courante dans le fichier.
  Construire_Nom_Fichier();
  if (pf_open(Nom_Fichier)) Affiche_ERR_et_attente_BP("Erreur_ouverture");
  Affiche_ERR_et_attente_BP("Affichage ?"); Afficher_les_lignes = (Num_BP == 4); Eteindre_Ecran(); 
  while (1) {// Lecture ligne à ligne du fichier.
    UINT br; // Pointeur de lecture dans le fichier ouvert.
    //On lit le fichier octet par octet en mettant 1 en deuxième paramètre :
    if (pf_read(buf, 1, &br)) Affiche_ERR_et_attente_BP("Erreur_lecture");
    if ((br == 0) || (!Syntaxe_correcte)) break;
    char carac = (char)buf[0]; //Pour la lecture, on stocke dans la variable caractere.
    //Si on est sur une fin d'une ligne, affiche la chaîne construite :
    //============================ Traiter la ligne construite ===========================
    if(carac=='\n' ){
       //Il faut rajouter le caractère de fin " \0 " pour marquer la fin de chaîne.
       Tampon[pos] = '\0'; Longueur_ligne_de_code = --pos; Num_Ligne++;
       pos = 0; //On réinitialise la position à 0 pour une nouvelle lecture recommencer à zéro.
       // Extraire la première ligne dans TITRE :
       if (Premiere_ligne) {
           for (byte I = 0; I<Taille_tampon; I++) Titre[I] = Tampon[I]; Premiere_ligne = false;}
       Ligne_a_ignorer = false; if (Longueur_ligne_de_code == 0) Ligne_a_ignorer = true;
       if (Tampon[0] == ';') Ligne_a_ignorer = true; // Ignorer les remarques.
       if (Tampon[0] == 'F') Ligne_a_ignorer = true; // Ignorer "F1000".
       if ((Tampon[0] == 'G') && (Tampon[1] == '9')) Ligne_a_ignorer = true; // Ignorer G90.
       if (!Ligne_a_ignorer) {Analyseur_Syntaxique(); if (Syntaxe_correcte) Execute_la_ligne_du_fichier();}}
    //====================================================================================   
    else{//Si ce n'est pas la fin de ligne, on continue à remplir le Tampon à chaque position :
         Tampon[pos] = (char)buf[0]; pos++;}}
    Liste_contenu_SD();}

void Traite_BP_Central() {// B.P. central activé dans la boucle de base.
  if (Existe[Num_Img_indexee]) Traite_le_fichier_indexe(); else BIP();}

/* Commentaires sur ce programme relatif à l'analyseur syntaxique.
   ===============================================================
   Par l'utilisation de LASER GRBL le fichier image ne contient que deux fois le code 'M' :
   M3 S0 en début de listage et M5 à la fin. Par programme, lorsque l'on valide un fichier
   le LASER est en veille. Donc M3 ne servira qu'à forcer 0% pour la puissance et sortir du mode veille.
   M5 forcera à 0% la puissance et passera le LASER en mode VEILLE.
   
   L'analyseur syntaxique sur un gros fichier de 25000 lignes exige 565 secondes environ.
   Il faut en moyenne 23mS pour extraire une ligne du fichier et en effectuer l'analyse.    */

