/* Directement issu de P11 ce programme ajoute la gestion de la motorisation en intégrant les
   procédures établies avec P12 et P14.
   
   UTILISATION DU PROGRAMME :
   ==========================
   Sur un RESET il y a lecture de la carte SD et affichage d'un texte d'alerte si le lecteur
   n'est pas "décodé". Si lecteur SD et contenu de la carte sorrect, il y a affichage du bilan des fichiers
   "ImgN.cgo". C'est l'état @ du "menu de base". Le BP central du codeur rotatif affiche les informations
   sur le fichier pointé si l'image est présente sur la carte SD. La rotation du codeur incrémental
   permet de pointer le fichier à traiter s'il est présent sur la carte SD.
   À ce stade @ toute action sur une touche du clavier provoque l'effet suivant :
     BP1 : Par défaut effectue la capture de l'origine machine.
           Dans la boucle de base, le [switch(Num_BP) case 1 ] comporte deux remarques qui permettent
           deux options de plus : Soit déplacer le LASER à l'origine du fichier Image,
                                  Soit déplacer le LASER au centre du plateau de la machine.
     BP2 : Bascule qui force à "NOIR" OLED ou qui rétablit l'affichage. (Économiseur d'écran.)
     BP3 : Bascule qui valide ou annule le "Mode PAS a PAS". L'état est affiché pendant une demi seconde.
     BP4 : Passe en PYROGRAVURE. Durant la pyrogravure le codeur rotatif permet de sortir à convenance
           ou si la réponse est négative d'afficher l'état actuel du processus puis de reprendre la
           gravure. Si la sortie anticipée est validée, enchaîne sur un retout en configuration
           "plateau dégagé" puis un BIP signale la fin de la gravure. En pyrogravure le mode pas à pas et
           l'affichage de l'état machine sont inhibés. Durant ce mode le B.P. du codeur rotatif est ignoré.
     BP5 : Passe en mode analyse du fichier. Durant l'analyse le codeur rotatif permet de sortir à
           convenance si la réponse est affirmative. Si le mode pas à pas est validé, il faut cliquer
           sur un BP du clavier à chaque ligne. Le B.P. noir est une bascule qui fait passer en analyse
           rapide, c'est à dire écran OLED noir et mode PAS a PAS suspendu. Rétablir l'affichage valide
           également la valeur initiale du booléen "mode PAS a PAS". En entrée il y a visualisation ligne
           à ligne. En mode ANALYSE du FICHIER le B.P. central du codeur rotatif permet d'afficher les
           caractéristiques de l'image.

   >>>> Par défaut le programme commence par vérifier le bon fonctionnement des capteurs Origine machine
        ce qui suppose que dans la pratique la machine sera en configuration dégagée. Si un mauvais
        fonctionnement est détecté le programme demande de résoudre le problème puis oblige à faire un
        RESET. Si les essais sont réalisés et que les capteurs ne sont pas encore opérationnels, il suffit
        de passer en remarque la ligne de void setup() qui invoque Tester_les_capteurs_Origine() cette
        ligne étant repérée par // @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

  Branchements à effectuer :
  --------------------------
 A0 : Ligne clavier de 3 touches.
 A1 : Ligne clavier de 2 touches.
 A2 : Bouton central du codeur rotatif.
 A4 : SDA OLED et Multiplexeur.
 A5 : SCL OLED et Multiplexeur.
 D2 : Codeur rotatif B >>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
 D3 : Codeur rotatif A >>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
                       >>>>>>>>>>>>>>>>>>>>>>> obtenir un sens de variation correct.
 D4 :  BUZZER actif.
 D5 :  Pilote le mode VEILLE des moteurs.
 D6 :  Pilotage du sens de rotation sur X.
 D7 :  Pilotage "faire un pas" sur X.
 D8 :  Pilotage du sens de rotation sur Y.
 D9 :  Pilotage "faire un pas" sur Y. 
 D10 : CS   
 D11 : MOSI
 D12 : MISO
 D13 : SCK
 
 L'affectation des sorties est issue du dernier démonstrateur P19 et construite pour espacer les
 picots utilisés et ainsi faciliter les branchements.
 Multiplexeur  S0 : Branchement de la LED verte du CLAVIER.
 Multiplexeur  S3 : Branchement de la LED bleue du mode VEILLE sur les moteurs.
 Multiplexeur S12 : Branchement d'une LED rouge pour visualiser en "répétiteur" le pourcentage LASER.
 Multiplexeur S15 : Sortie de pilotage PWM LASER.
                                                                    PGM               RAM dynamique
Programme initial clone de p11.                             19656 Octets (63%)     628 Octets (30%)
Mise en place des procédures de gestion des mouvements :    23452 Octets (76%)     860 Octets (41%)
Ci-avant : Intègre la gestion du LASER, la prise en compte des coordonnées négatives et des débordements
           de limites mécaniques sur la machine. Teste également des caractères de commande non valides.
Mise en place de la vérification des fichiers :             23610 Octets (76%)     846 Octets (41%)
Codage pour la bascule Ecran NOIR : Ecran affiché :         23632 Octets (76%)     847 Octets (41%)
Implantation d'un menu minimal et capture des Origines :    24230 Octets (78%)     860 Octets (41%)
Lecture caractéristiques fichier et saisie origine image :  24942 Octets (81%)     875 Octets (42%)
Options de déplacement sur Origines et centre du plateau :  25212 Octets (82%)     858 Octets (41%)
Booleen "Rapide" pour G0 et origines sans délai / pas :     25394 Octets (82%)     860 Octets (41%)
>>>> Ci-dessus l'occupation mémoire est effectuée en validant sur  l'un des deux case : 1 forcé en case 9.

Mesure sur les capteurs Origines et PAGE1 : +298 octets (1%) et 6 octets de mémoire dynamique.
Mise en place du test des capteurs d'Origines : +92 octets et rien en mémoire dynamique.              */

//--------------------- Déclarations pour le matériel -----------------------
#include <Wire.h> // Gère les protocole I2C sur A4 et A5 pour le Multiplexeur.
#include <EEPROM.h>
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)
#include "PetitFS.h"
FATFS fs;     // Zone de travail pour le volume SD : Objet système.
#include <Adafruit_PWMServoDriver.h> // déclaration du Multiplexeur.
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

//------------- Définition des broches d'entrées analogiques ----------------
#define Ligne_trois_touches 14 // A0 pour lire le clavier à 3 BP. 
#define Ligne_deux_touches 15 // A1 pour lire le clavier à 2 BP. 
#define BP_central 16 // A2 pour lire le bouton central.
#define Pilotage_capteurs_Origines 17 // A3 active les modules uniquement aux moments pertinents.
#define Capteur_Origine_X  20 // A6 pour lire le capteur sur butée X.
#define Capteur_Origine_Y  21 // A7 pour lire le capteur sur butée X.
//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_B 2
#define Codeur_A 3
#define BUZZER 4
#define Moteurs_actifs 5 // Broche qui à l'état haut fait passer le moteur en mode "non alimenté".
#define Sens_sur_X 6 // Pilotage du sens de rotation sur X.
#define Un_pas_sur_X 7 // Pilotage "faire un pas" sur X.
#define Sens_sur_Y 8 // Pilotage du sens de rotation sur Y.
#define Un_pas_sur_Y 9 // Pilotage "faire un pas" sur Y.
#define chipSelect 10
//------------------- Déclaration des constante du programme ----------------
// #define Coef_poulie 0.9804 // Correction du diamètre moyen de la poulie motrice. @@@@@@@@@@@@@@@@@@@@@@@@
#define Coef_poulie 1 // Ci-dessous valeur sur X élevée car orifice du "filtre" mal centré. @@@@@@@@@@@@@@@@
#define Seuil_Origine_X 900 // Tension moyenne entre Actif et neutre sur X. @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Seuil_Origine_Y 553 // Tension moyenne entre Actif et neutre sur Y. @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Course_MAX_sur_X 225.0 // Déplacement maximal possible sur la machine @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Course_MAX_sur_Y 225.0 // Déplacement maximal possible sur la machine @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Origine_machine_sur_Y 2250 // Dix fois la valeur de Course_MAX_sur_Y @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.
#define Taille_tampon 30 // Taille des tampons pour les chaînes de caractères.
//----------------------- Déclaration des variables -------------------------
boolean Afficher_en_gravant; // Sera une option éventuelle pour le programme d'exploitation.
boolean BP_central_a_traiter;
boolean Demande_FIN_active; // Bascule pour ne pas avoir un doublon si fin demandée lors d'un PAS moteur.
boolean DX_Positif, DY_Positif; //, DX_sup_a_DY;
boolean Ecran_Noir;
boolean Fin_demandee;
boolean Lecteur_OK;
boolean Ligne_G90_trouvee;
boolean Ligne_M3_trouvee;
boolean Mode_pas_a_pas;
boolean Mode_Verification_Programme;
boolean Mouvements_Rapide; // Ne pas ralentir les moteurs pour capter les origines.
boolean OrigineX_effective, OrigineY_effective;
boolean Saisie_Informations_Fichier;
boolean STOP; // Permet de sortir d'une gravure si fin demandée.
boolean Test_Capteurs_OK;

boolean Existe[10];

char Indice[10] = {'0','1','2','3','4','5','6','7','8','9'};
char Nom_Fichier[9] = "IMGN.GCO";
char Tampon[Taille_tampon];
char Titre[Taille_tampon];
String Contenu_ligne[Taille_tampon];

boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean Sens_positif; // Sens de rotation.
volatile boolean Rotation;  // Indique une activation du codeur rotatif.

byte Code_instruction; // Code en sortie de l'analyse syntaxique.
byte Longueur_ligne_de_code; 
byte Num_BP; // Désigne le bouton poussoir activé sur le clavier.
byte Menu_en_cours; // Page-écran à rétablir suite à "Ecran NOIR".
byte Num_Img_indexee;
byte Pourcentage; // Consigne de Pourcentage de puissance du LASER.
byte Puissance_MAX_sur_le_LASER; // Puissance qui sera appliquée quand Pourcentage = 100;
byte SAV_Pourcentage;
byte Type_Erreur;

int Ancien_X, Ancien_Y; // Positions exprimées en dixièmes de mm.
int Delai_par_pas_elementaire;
int Numerisation_BP; // Valeur d'une touche clavier retournée par la CAN.
int Pulse; // Rapport cyclique du signal de pilotage du LASER.
int U_sur_X, U_sur_Y; // tensions sur les capteurs d'origine machine.
unsigned long Num_Ligne, Taille;

float Position_X, Position_Y, Origine_Image_X, Origine_Image_Y;
//======================================================================================

void setup() {
  //---------------------- Définition des vecteurs d'interruption ----------------------
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  // Broche de l'interruption 1 (E/S D3) :
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pwm.begin(); // Initialise le module multiplexeur.
  pwm.setPWMFreq(1000); // Fréquence particulière 1000Hz pour piloter le module LASER.
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(chipSelect, OUTPUT); digitalWrite(chipSelect,HIGH);
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW);
  pinMode(Pilotage_capteurs_Origines,OUTPUT); digitalWrite(Pilotage_capteurs_Origines, LOW);
  pinMode(Moteurs_actifs, OUTPUT); Passer_en_VEILLE(); // Passe les moteurs en veille et allume la LED bleue.
  pinMode(Sens_sur_X, OUTPUT); pinMode(Un_pas_sur_X, OUTPUT); digitalWrite(Un_pas_sur_X, LOW);
  pinMode(Sens_sur_Y, OUTPUT); pinMode(Un_pas_sur_Y, OUTPUT); digitalWrite(Un_pas_sur_Y, LOW);
  // Ci-dessous Puissance 30% pour distinguer sur les deux LED LASER la différence.
  Puissance_MAX_sur_le_LASER = 20; // @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  Couper_le_LASER(); // Puissance minimale sur RESET.
  pinMode(11,INPUT); pinMode(12,INPUT); pinMode(13,INPUT);
  BP_central_a_traiter = false; digitalWrite(BP_central, HIGH); // Valide R PUL-UP sur A2.
  Front_sur_A(); Front_sur_B(); // Routines d'IRQ pour le codeur rotatif.
  Mode_pas_a_pas = false; Mode_Verification_Programme = false; Ecran_Noir = false; Num_BP = 0;
  Delai_par_pas_elementaire = 10; // Prendre 10 par défaut @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  Num_Img_indexee = 9; Eteindre_LED_clavier(); Saisie_Informations_Fichier = false;
  Tester_les_capteurs_Origine();
  if (!Test_Capteurs_OK) {Type_Erreur = 5; Probleme_de_lecture();} // @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  // Ci-dessous : Vérifie présence carte et initialise le tableau des images présentes :
  Bilan_contenu_SD(); Liste_contenu_SD(); Afficher_en_gravant = false;}
  // Pas besoin d'afficher le contenu car la rotation "fictive sur RESET" du B.R. provoque l'affichage. 
  
void loop() {
  //======================== Traite le B.P. central ======================
  Teste_activation_BP_codeur_incremental(); // Si B.P. activé saisir les caractéristiques de l'image.
  if (BP_central_a_traiter) {BP_central_a_traiter = false;
     if (Existe[Num_Img_indexee]) {Initialiser_les_caracteristiques_du_fichier();
         Liste_contenu_Fichier(); ATTENDRE_un_BP(); Liste_contenu_SD();}
     else BIP();}
  //--------------------- Traite la rotation du codeur incrémental ---------------------
  if (Rotation) TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();
  //---------------------- Traiter la requête de l'un des cinq B.P. --------------------
  Tester_le_Clavier(); // Menu minimal pour effectuer les divers tests.
  if (Num_BP !=0) {ATTENDRE_le_relacher();
     switch(Num_BP) {
       case 1 : {Capturer_Origine_machine(); Passer_en_VEILLE(); Liste_contenu_SD(); break;}
 //    case 1 : {Positionner_LASER_sur_Origine_Image(false); break;} // Place sur l'origine de l'image.
 //    case 1 : {Positionner_LASER_sur_Origine_Image(true); break;} // Centre sur le plateau de la machine.
       case 2 : {Inverser_bascule_Ecran_Noir(); break;}
       case 3 : {Mode_pas_a_pas = !Mode_pas_a_pas; Aff_Item_demi_seconde(398, 14);
                 Ecran_Noir = false; Liste_contenu_SD(); break;}
       case 4 : {Aff_Item_demi_seconde(149, 10); PYROGRAVER(); break;}
       case 5 : {Aff_Item_demi_seconde(368, 18); VERIFIER_LE_FICHIER();}}
    Num_BP = 0;}} // Action sur le B.P. prise en compte.

void Allume_LED_clavier() {pwm.setPWM(0, 0, 4095);}
void Eteindre_LED_clavier() {pwm.setPWM(0, 0, 0);}

void Teste_activation_BP_codeur_incremental() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     // ========== Attendre le relacher du B.P. central. ==========
     Allume_LED_clavier();
     while (analogRead(BP_central) < 512);  // Boucler sans rien faire pour attendre le relâché.
     Eteindre_LED_clavier();
     // ===========================================================
     BP_central_a_traiter = true;}}
     
//------------------------------ Différents écrans affichés sur OLED -------------------------------
void Aff_TEXTE_EEPROM(int PTR, byte Longueur) {// Routines pour les affichages textes EEPROM.
    for (byte I=0; I<Longueur; I++) {u8g.print(char(EEPROM.read(PTR))); PTR++;}; }

void Aff_Item_demi_seconde(int PTR, byte LGR) {
  u8g.firstPage(); do {u8g.setPrintPos(0, 30); Aff_TEXTE_EEPROM(PTR, LGR);
  if (Num_BP == 3) Affiche_Deux_points_espace_OUI_NON(Mode_pas_a_pas);}
  while(u8g.nextPage()); delay (500); Eteindre_Ecran();}

void Eteindre_Ecran(){u8g.firstPage(); do {} while(u8g.nextPage());}
 
void Liste_contenu_SD() {// Ne teste plus la présence carte et liste son "contenu images" en aveugle
  byte X,Y;              // en utilisant le tableau de booléens Existe[10] initialisé sur RESET.
  // Afficher la liste de l'état des fichiers sur OLED :
  u8g.firstPage(); do {
     Y = 10; X = 0;
     for (byte I=0; I<10; I++) {
          if (I > 4) X = 66; if (I == 5) Y = 10;
          Debut_Nom(X,Y,I); if (Existe[I]) Affiche_OUI(); else Aff_TEXTE_EEPROM(7,4);  Y = Y + 13;
          if (I == Num_Img_indexee) Aff_TEXTE_EEPROM(50,2);}} 
  while(u8g.nextPage()); Menu_en_cours = 6;}

void AFFICHE_etat_machine(){
  if ((!Ecran_Noir) && (!Mouvements_Rapide)) {// En gravure, ne pas ralentir si mouvement rapide.
      u8g.firstPage(); do {
      u8g.setPrintPos(0, 12); u8g.print(Titre);
      u8g.setPrintPos(0, 24); Aff_TEXTE_EEPROM(45,6); u8g.print(Num_Ligne);
      u8g.setPrintPos(0, 37); for(byte I=0; I<Taille_tampon; I++) u8g.print(Contenu_ligne[I]);
      u8g.setPrintPos(0, 50); Aff_TEXTE_EEPROM(52,4); u8g.print(Pourcentage); Aff_TEXTE_EEPROM(326,10);
      u8g.setPrintPos(0, 63); Aff_TEXTE_EEPROM(56,4); u8g.print(Position_X,1);
      u8g.setPrintPos(65, 663); Aff_TEXTE_EEPROM(60,4); u8g.print(Position_Y,1);}
      while(u8g.nextPage());}
  Menu_en_cours = 7;
  if ((Mode_pas_a_pas) && (!Ecran_Noir)) {
       ATTENDRE_un_BP(); if (Num_BP == 2) Inverser_bascule_Ecran_Noir();}
  Tester_le_Clavier(); if (Num_BP == 2) Inverser_bascule_Ecran_Noir();}

void Liste_contenu_Fichier() {
 u8g.firstPage(); do {
 u8g.setPrintPos(0, 12); u8g.print(Titre);
 u8g.setPrintPos(0, 24); Aff_TEXTE_EEPROM(415,7); u8g.print(Taille); Aff_TEXTE_EEPROM(421,7);
 u8g.setPrintPos(0, 36); Aff_TEXTE_EEPROM(186,12); u8g.print(Origine_Image_X); 
 u8g.setPrintPos(0, 48); Aff_TEXTE_EEPROM(200,12); u8g.print(Origine_Image_Y);  
 u8g.setPrintPos(0, 60); Aff_TEXTE_EEPROM(428,12); u8g.print(Taille / 1800); Aff_TEXTE_EEPROM(439,4);}
while(u8g.nextPage());}

void Affiche_Deux_points_espace_OUI_NON(boolean OUI) {
  if (OUI) Aff_TEXTE_EEPROM(173,5); else Aff_TEXTE_EEPROM(179,5);}

//-------------------- Routines de servitude pour traiter le capteur incrémental -------------------
void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 100 expérimentale.
  
void Front_sur_A(){ // Interruption quand A change d'état avant B.
  if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
  if (digitalRead(Codeur_A) != A_set) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();}}}
  
void Front_sur_B(){ // Interruption quand B change d'état avant A.
  if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
  if (digitalRead(Codeur_B) != B_set) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();}}}
//-------------------------------------------------------------------------------------------------
void Delay_1mS() {delay(1);}
void Delay_30mS() {delay(30);}
void Delay_100mS() {delay(100);}

void BIP() {digitalWrite(BUZZER, HIGH); Delay_100mS(); digitalWrite(BUZZER, LOW);}

void Tester_le_Clavier() {// Les valeurs tiennent compte de VCC = 4V car fourni par la carte NANO.
  Num_BP = 0; // Signale aucun B.P. utilisé.
  Numerisation_BP = analogRead(Ligne_trois_touches);
  if (Numerisation_BP < 758) {Num_BP = 1;
      if (Numerisation_BP < 508)  Num_BP = 2; if (Numerisation_BP < 230)  Num_BP = 3;} 
  Numerisation_BP = analogRead(Ligne_deux_touches);
  if (Numerisation_BP < 709) {Num_BP = 5; if (Numerisation_BP < 300)  Num_BP = 4;}}

void ATTENDRE_le_relacher() {
  byte SAV_Num_BP = Num_BP;
  byte COMPTEUR = 0; Delay_30mS(); // Attendre_la_stabilisation du travail.
  Allume_LED_clavier();
  while (COMPTEUR < NB_tests_antirebonds) {// Attendre_la_stabilisation du repos :
         COMPTEUR++; Tester_le_Clavier(); if (Num_BP > 0) COMPTEUR = 0;}
  Eteindre_LED_clavier(); Num_BP = SAV_Num_BP;}

void ATTENDRE_un_BP() {
  boolean Allumer;
  Num_BP = 0;
  while (Num_BP == 0) {Tester_le_Clavier(); Delay_100mS();
         Allumer =!Allumer; if (Allumer) Allume_LED_clavier(); else Eteindre_LED_clavier();}
  ATTENDRE_le_relacher();}

void Retablir_Ecran_OLED() {
  if (Menu_en_cours == 6)  Liste_contenu_SD(); if (Menu_en_cours == 7)  AFFICHE_etat_machine();}

void Inverser_bascule_Ecran_Noir() {// Bascule pour éteindre ou activer l'écran OLED.
  if (Num_BP == 2) {Ecran_Noir = !Ecran_Noir; ATTENDRE_le_relacher();
      if (Ecran_Noir) Eteindre_Ecran(); else Retablir_Ecran_OLED();}}

void Affiche_ERR_et_attente_BP(int PTR, byte LGR) { Couper_le_LASER(); // Simple précaution.
  u8g.firstPage(); do {u8g.setPrintPos(5, 35); Aff_TEXTE_EEPROM(PTR,LGR);} while(u8g.nextPage());
  BIP(); ATTENDRE_un_BP();}

void Debut_Nom(byte X, byte Y, byte N) {u8g.setPrintPos(X, Y); Aff_TEXTE_EEPROM(0,3); u8g.print(N);}
void Affiche_OUI() {Aff_TEXTE_EEPROM(3,4);}

void Probleme_de_lecture() {// Impose de faire un RESET.
  Couper_le_LASER(); Passer_en_VEILLE();
  INFINI: 
  if (Type_Erreur == 4) Affiche_ERR_et_attente_BP(11,21); // Affiche "Pb sur le lecteur SD."
  else if (Type_Erreur == 5) Affiche_ERR_et_attente_BP(267,19); // Affiche "Pb Capteur ORIGINE !"
       else Affiche_ERR_et_attente_BP(113,17); // Affiche "Erreur en lecture"
  u8g.firstPage(); // Affiche "Resoudre le probleme puis RESET !"
  do {u8g.setPrintPos(0, 22); Aff_TEXTE_EEPROM(212,20);
      u8g.setPrintPos(15, 42); Aff_TEXTE_EEPROM(232,16);}
       while(u8g.nextPage());
  ATTENDRE_un_BP(); goto INFINI;}

void Bilan_contenu_SD() { // Teste la présence carte et message d'erreur sur boucle infinie si non OK :
  Lecteur_OK = true; if (pf_mount(&fs)) Lecteur_OK = false; 
  if (!Lecteur_OK) // Teste la présence carte et message d'erreur sur boucle infinie si non OK :
     {Type_Erreur = 4; Probleme_de_lecture();}
  else // Construire la table de présence des dix fichiers possibles :
     for (byte I=0; I<10; I++) {Nom_Fichier[3] = Indice[I]; Existe[I] = !pf_open(Nom_Fichier);}}
  
void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  INT_active = true; // Armer un nouveau comptage.
  Rotation = false; // Prise en compte du traitement.
  if (Sens_positif) {Num_Img_indexee++; if (Num_Img_indexee == 10) Num_Img_indexee = 0;}
  else {if (Num_Img_indexee > 0) Num_Img_indexee--; else Num_Img_indexee = 9;}
  Liste_contenu_SD();}

void Moduler_le_LASER() {// Place sur S15 la valeur correspondant à Pourcentage.
  int Pulse;
  if (!Mode_Verification_Programme) // Moduler uniquement si GRAVURE et non en vérification de fichier. 
     {Pulse = map(Pourcentage, 0, 100, 0, (Puissance_MAX_sur_le_LASER * 4095)); pwm.setPWM(15, 0, Pulse);}
  Pulse = map(Pourcentage, 0, 100, 0, 4095); pwm.setPWM(12, 0, Pulse);} // Allume le répétiteur LED.

void Couper_le_LASER() {SAV_Pourcentage = Pourcentage; Pourcentage = 0; Moduler_le_LASER();}

void Retablir_le_LASER() {Pourcentage = SAV_Pourcentage; Moduler_le_LASER();}

void Passer_en_VEILLE() {digitalWrite(Moteurs_actifs, HIGH); pwm.setPWM(3, 0, 4095);}

void Activer_les_moteurs() {digitalWrite(Moteurs_actifs, LOW); pwm.setPWM(3, 0, 0); Delay_30mS();}
     // Ci-dessus delai 30mS pour stabilisation des moteurs.

void Prepare_degagement() {Position_X = 10.0, Position_Y = 215.0; Mouvements_Rapide = true;}

void Position_degagee() {
  if (!Mode_Verification_Programme) {// Ne dégager que si mode pyrogravure.
      Couper_le_LASER(); Activer_les_moteurs(); Prepare_degagement();
      STOP = false; Effectuer_un_deplacement_gco(); Passer_en_VEILLE();}}

void Capturer_Origine_machine() {// Déplacer à gauche et vers l'arrière de la machine.
  // Tourner dans le sens "négatif" pour X et positif pour Y :
  STOP = false; Mouvements_Rapide = true; Activer_les_moteurs(); DX_Positif = false; DY_Positif = true;
  Aff_Item_demi_seconde(248, 19); // Affiche "> Capture ORIGINE <
  // Positionner OrigineX_effective et OrigineY_effective avant d'effectuer la prise d'origine :
  Mesurer_U_sur_capteurs_Origines(); // Positionne OrigineX_effective et OrigineY_effective actuelles.
  while (!OrigineX_effective) {Faire_un_pas_sur_X(); Mesurer_U_sur_capteurs_Origines();}
  while (!OrigineY_effective) {Faire_un_pas_sur_Y(); Mesurer_U_sur_capteurs_Origines();}
  Ancien_X = 0; Ancien_Y = 2250;} // Attentions : Position en dixièmes de mm.
  // Ci-dessus on suppose que capteur sur Y est ajusté mécaniquement à 225mm de l'origine Image.

void Degager_le_plateau() {// Procédure utilisées dans les options.
  Capturer_Origine_machine(); // Force STOP à false et Activer_les_moteurs(). 
  Position_degagee();}
  
void Decale_la_chaine() {
  for (byte I = 0; I < Taille_tampon; I++) Tampon[I] = Tampon[I+1];}

void Item_suivant(char Caractere) {while (Tampon[0] != Caractere) Decale_la_chaine(); Decale_la_chaine();}
  
void Traite_X() {
  Decale_la_chaine(); Position_X = atof(Tampon); Item_suivant('S'); Pourcentage =  atoi(Tampon);}

void Traite_G() {
  boolean G1;
  Decale_la_chaine(); G1 = Tampon[0] == '1'; Item_suivant('X'); Position_X = atof(Tampon);
  // Ci-dessous : G1 est de la forme : G1 Xn Sn
  if (G1) {Item_suivant('S'); Pourcentage =  atoi(Tampon); Mouvements_Rapide = false;}
  // Ci-dessous, on traite G0 qui est de la forme : G0 Xn Yn Sn ou G0 Xn Yn. 
  else {Mouvements_Rapide = true; // G0 est par définition du gcode un déplacement rapide LASER 0%.
        // Traite la ligne G0 X0 Y0 finale en adoptant la configuration machine dégagée.
        if (Longueur_ligne_de_code == 8) Prepare_degagement(); 
        else {Item_suivant('Y'); Position_Y = atof(Tampon);
              if (Ligne_M3_trouvee) Item_suivant('S'); Pourcentage = atoi(Tampon);}}}

void Demande_FIN_de_procedure() {
  boolean SAV_Rapide = Mouvements_Rapide; 
  boolean SAV_Ecran_noir = Ecran_Noir;
  // Touche OUI cliquée : Couper les moteurs, éteindre le LASER et sortir du fichier.
  Couper_le_LASER(); // mémorise la valeur actuelle de Pourcentage pour Retablir_le_LASER().
  Affiche_ERR_et_attente_BP(64, 18); // Affiche le texte "Fin de procedure ?".
  Fin_demandee = (Num_BP == 4);
  if (Fin_demandee) {STOP = true; Couper_le_LASER(); SAV_Pourcentage = 0;
                     Mode_pas_a_pas = false; Demande_FIN_active = false;}
  else {Mouvements_Rapide = false; Ecran_Noir = false; AFFICHE_etat_machine(); ATTENDRE_un_BP();
        Ecran_Noir = SAV_Ecran_noir; Mouvements_Rapide = SAV_Rapide;}
  if (!Mode_Verification_Programme) Retablir_le_LASER(); Eteindre_Ecran();}

void Teste_demande_sortie() {
  if (Rotation) {// Traitement si on veut interrompre une pyrogravure.
     Rotation = false; // Prise en compte du traitement.
     INT_active = true; // Armer un nouveau comptage.
     Demande_FIN_de_procedure();}}
  
void Analyseur_Syntaxique() {
  Type_Erreur = 0; // Avant d'analyser la ligne il n'y a pas d'erreur.
  //-------------- Teste le codeur rotatif avant de traiter la ligne. -------------
  if (Demande_FIN_active) Teste_demande_sortie(); // Ne sera pas demander si annulé durant un PAS.
  //------------------ Gère le clavier pour passer en écran noir. -----------------
  if ((Mode_Verification_Programme) && (!Fin_demandee)) {
      Tester_le_Clavier(); if (Num_BP == 2) {Inverser_bascule_Ecran_Noir(); Num_BP == 0;}}
  //------------------------------------------------------------------------------- 
  switch (Tampon[0]) {  // NOTE : M3 : LASER ON,   M5 : LASER OFF.
     case 'G' : {Traite_G(); break;}
     case 'M' : {Ligne_M3_trouvee = true; Couper_le_LASER(); // M3 et M5 forcent le LASER à puissance nulle.
                 if (Tampon[1] == '3') Activer_les_moteurs(); break;}
     case 'S' : {Couper_le_LASER(); break;}
     case 'X' : {Traite_X(); break;} // Le break est impératif pour sauter "defalt" si OK.
     default  : Type_Erreur = 3;}
  //----------------------- Gère une coordonnée négative. -------------------------
  if ((Position_X < 0) || (Position_Y < 0)) Type_Erreur = 1;
  //-------------------- Gère une coordonnée > MAX possible -----------------------
  if ((Position_X > Course_MAX_sur_X) || (Position_Y > Course_MAX_sur_X)) Type_Erreur = 2;
  //-------------------------------------------------------------------------------
  if (Type_Erreur == 0) {
      if ((Mode_Verification_Programme) || (Afficher_en_gravant)) AFFICHE_etat_machine();
      // En vérification de programme la machine est inerte :
      Moduler_le_LASER(); // Seul la LED rouge sera allumée si mode vérification defichier.
      // L'appel à Moduler_le_LASER() se fait systématiquement pour traiter la LED rouge du répétiteur,
      // y compris en mode vérification de programme pour visualiser l'état sur la LED.
      if (!Mode_Verification_Programme) Effectuer_un_deplacement_gco();}}
      
void Traite_ligne_a_ligne_le_fichier() {
  boolean Ligne_a_ignorer; uint8_t buf[Taille_tampon]; // Tampon de travail.
  Ligne_G90_trouvee = false; Ligne_M3_trouvee = false; Type_Erreur = 0; Num_Ligne = 1;
  Fin_demandee = false; Demande_FIN_active = true;
  unsigned long pos = 0; // Position courante dans le fichier.
  //++++++++++++++++++++++++++++++ // Lecture ligne à ligne du fichier. +++++++++++++++++++++++++++++++++ 
  while ((Type_Erreur == 0) && (!Fin_demandee)) {// Sera réalisé tant que "break" non rencontré.
    // Ci-avant le test sur Type_Erreur sert à sortir si FIN gravure demandée.
    UINT PLFO; // Pointeur de Lecture dans le Fichier Ouvert.
    //On lit le fichier octet par octet en mettant 1 en deuxième paramètre :
    if (pf_read(buf, 1, &PLFO)) Probleme_de_lecture();
    if (PLFO == 0) break; // Sortie si fin de fichier.
    char carac = (char)buf[0]; //Pour la lecture, on stocke dans la variable caractere.
    //============================ Traiter la ligne construite ===========================
    if(carac=='\n' ){
       //Il faut rajouter le caractère de fin " \0 " pour marquer la fin de chaîne.
       Tampon[pos] = '\0'; Longueur_ligne_de_code = --pos;
       pos = 0; //On réinitialise la position à 0 pour une nouvelle lecture recommencer à zéro.
       //-------------------------- Traiter Contenu_ligne[I] -------------------------
       for (byte I = 0; I<Taille_tampon; I++) Contenu_ligne[I] = String(' ');
       for (byte I = 0; I<Longueur_ligne_de_code; I++) Contenu_ligne[I] = String(Tampon[I]);
       //------------------------- Traiter la première ligne -------------------------
       if (Num_Ligne == 1) { // Ci-deoous : Extraire la première ligne dans TITRE.
           for (byte I = 0; I<Taille_tampon; I++) Titre[I] = Tampon[I];}
       //------------------------- Traiter la deuxième ligne -------------------------
       // Sortie anticipée dès les données acquises si on désire juste "les informations fichier".
       if (Saisie_Informations_Fichier) {
          if (Num_Ligne == 2) {// Ci-dessous : Extraire le nombre de lignes.
          Decale_la_chaine(); Taille = atol(Tampon); Tampon[0] = ';';}
          //--------------------- Traiter la ligne Origine Image ---------------------
          if (Ligne_G90_trouvee) {Ligne_G90_trouvee = false; Traite_G();
              Origine_Image_X = Position_X; Origine_Image_Y = Position_Y; break;}}
              // Doit être placée avant le traitement de la ligne G90.          
       //------------------------ Repérer les lignes à ignorer -----------------------
       Ligne_a_ignorer = false;
       if (Longueur_ligne_de_code == 0) Ligne_a_ignorer = true;
       if (Tampon[0] == ';') Ligne_a_ignorer = true; // Ignorer les remarques.
       if (Tampon[0] == 'F') Ligne_a_ignorer = true; // Ignorer "F1000".
       if ((Tampon[0] == 'G') && (Tampon[1] == '9'))
          {Ligne_a_ignorer = true; Ligne_G90_trouvee = true;} // Ignorer G90.
          // Le test ci-avant est important car il pare la première ligne avec des coordonnées
          // différentes de zéro et pas de paramètre pour le pourcentage du LASER. 
       //----------------------- Traiter les lignes pertinentes ----------------------
       // Si "le contenu de la ligne" est incorrect l'analyseur syntaxique force Fin_demandee à true.
       if ((!Ligne_a_ignorer) && (!Saisie_Informations_Fichier)) Analyseur_Syntaxique();
       if (Type_Erreur == 0) Num_Ligne++;} // Ne pas incrémenter si une erreur a été détectée.
    //====================================================================================   
    else{//Si ce n'est pas la fin de ligne, on continue à remplir le Tampon à chaque position :
         Tampon[pos] = (char)buf[0]; pos++;}}
   //++++++++++++++++++++++++++++ // Lecture ligne à ligne du fichier. +++++++++++++++++++++++++++++++++
   // Ci-dessous : Impératif si une erreur se produit en analyse de fichier ou en pyrogravure.
   Couper_le_LASER(); // Impératif si une erreur se produit en analyse de fichier ou en pyrogravure.
   // En sortie d'exploration complète afficher les problèmes de syntaxe y compris si le fichier est correct
   // Car sur non erreur précise que le fichier est cohérent.
   BIP();} // Alerte de la fin du traitement.

//========================== Procédures pour gérer les mouvements ==========================
void Mesurer_U_Capteur(boolean Sur_X) {
  #define Nb_Iterations 20 // Mesure avec Nb itérations pour l'intégration "d'antiparasitage".
  // La valeur de 20 est choisie expérimentalement et semble un bon compromis entre des
  // mesures fiables et une rapidité suffisante de rotation des moteurs.
  long Valeurs_lues;
  byte Compteur = Nb_Iterations;
  Valeurs_lues = 0;
  for (int I = 0; I < Nb_Iterations; I++) // Cumuler N mesures. 
      {if (Sur_X) Valeurs_lues = Valeurs_lues + analogRead(Capteur_Origine_X);
       else Valeurs_lues = Valeurs_lues + analogRead(Capteur_Origine_Y);}
  if (Sur_X) U_sur_X = int(Valeurs_lues / Nb_Iterations); else U_sur_Y = int(Valeurs_lues / Nb_Iterations);}
  
void Traiter_Origines_sans_allumer() {
  Mesurer_U_Capteur(true); // Donc mesure avec anti-rebonds sur le capteur X.
  Mesurer_U_Capteur(false); // Donc mesure avec anti-rebonds sur le capteur Y.
  OrigineX_effective = (U_sur_X > Seuil_Origine_X); OrigineY_effective = (U_sur_Y > Seuil_Origine_Y);}

void Mesurer_U_sur_capteurs_Origines() {
  digitalWrite(Pilotage_capteurs_Origines, HIGH); Delay_1mS(); Traiter_Origines_sans_allumer();}

void Tester_les_capteurs_Origine() {
  Traiter_Origines_sans_allumer(); // Ci-dessous les deux doivent avoir "Origine_effective".
  Test_Capteurs_OK = ((OrigineX_effective) && (OrigineY_effective));
  Mesurer_U_sur_capteurs_Origines(); // Ci-dessous les deux doivent avoir NON "Origine_effective".
  if ((OrigineX_effective) || (OrigineY_effective)) Test_Capteurs_OK = false;
  digitalWrite(Pilotage_capteurs_Origines, LOW);}  

void Faire_un_pas_sur_X() {// Ancien X incrémenté ou décrémenté de 2 car chaque pas fait 0,2mm.
  if (!STOP) {
    if (DX_Positif) {Ancien_X = Ancien_X + 2; digitalWrite(Sens_sur_X, LOW);}
    else {Ancien_X = Ancien_X - 2; digitalWrite(Sens_sur_X, HIGH);}
    digitalWrite(Un_pas_sur_X, HIGH); Delay_1mS(); digitalWrite(Un_pas_sur_X, LOW);
    if (Mouvements_Rapide)  delay(10); else delay(Delai_par_pas_elementaire);
    Teste_demande_sortie();}}

void Faire_un_pas_sur_Y() {// Ancien Y incrémenté ou décrémenté de 2 car chaque pas fait 0,2mm.
  if (!STOP) {
    if (DY_Positif) {Ancien_Y = Ancien_Y + 2; digitalWrite(Sens_sur_Y, LOW);}
    else {Ancien_Y = Ancien_Y - 2; digitalWrite(Sens_sur_Y, HIGH);}
    digitalWrite(Un_pas_sur_Y, HIGH); Delay_1mS();
    digitalWrite(Un_pas_sur_Y, LOW);
    if (Mouvements_Rapide)  delay(10); else delay(Delai_par_pas_elementaire);
    Teste_demande_sortie();}}

void Effectuer_un_mouvement_vectoriel(int DeltaX, int DeltaY, boolean DX_sup_a_DY) {
  boolean Fini; int NbPX, NbPY; float Cumul, Decimale;
  int Compteur; int Rapport; // Impérativement un int car peut aller jusqu'à 2250.
  // Attention ci-dessous cumul est utilisé pour un calcul préalable du rapport décimal.
  if (DX_sup_a_DY) Cumul = float(DeltaX) / float(DeltaY); else Cumul = float(DeltaY) / float(DeltaX);
  Decimale = Cumul - ((int) Cumul); Rapport = Cumul - Decimale ;
  Cumul = 0.0; NbPX=0; NbPY=0; Fini = false;
  if (DX_sup_a_DY) {
      while (!Fini) {
        Compteur = 0;
        while (Compteur < Rapport) {
           if (NbPX < DeltaX) {Faire_un_pas_sur_X(); NbPX++; Compteur++; Cumul = Cumul + Decimale;}
           else Compteur = Rapport;} // Forcer la sortie si NbPX = DeltaX.
        Faire_un_pas_sur_Y(); NbPY++; 
        if (int(Cumul + 0.01) > 0) {// Donc Cumul vaut au moins 1.
        if (NbPX < DeltaX) 
           {Faire_un_pas_sur_X(); NbPX++; Cumul = abs(Cumul - 1); if (Cumul > 1) Cumul = abs(Cumul - 1);}}
        Fini = NbPX == DeltaX;}
      while (NbPY < DeltaY) {Faire_un_pas_sur_Y(); NbPY++;}}
  else {// Si DeltaY > DeltaX il faut inverses les rôles joués par X et par Y.
      while (!Fini) {
        Compteur = 0;
        while (Compteur < Rapport) {// Si DeltaX inférieur à DeltaY il faux échanger les rôles de X et de Y.
           if (NbPY < DeltaY) {Faire_un_pas_sur_Y(); NbPY++; Compteur++; Cumul = Cumul + Decimale;}
           else Compteur = Rapport;} // Forcer la sortie si NbPY = DeltaY.
        Faire_un_pas_sur_X(); NbPX++; 
        if (int(Cumul + 0.01) > 0) {// Donc Cumul vaut au moins 1.
        if (NbPY < DeltaY) 
           {Faire_un_pas_sur_Y(); NbPY++; Cumul = abs(Cumul - 1); if (Cumul > 1) Cumul = abs(Cumul - 1);}}
        Fini = NbPY == DeltaY;}
      while (NbPX < DeltaX) {Faire_un_pas_sur_X(); NbPX++;}}}

void Traiter_les_mouvements_du_LASER(int DeltaX, int DeltaY, boolean DX_sup_a_DY) {
  int NbPX = 0;   int NbPY = 0;
  if ((DeltaX > 0) || (DeltaY > 0)) {// Ne rien faire si les deux déplacements sont nuls.
     if (DeltaX == 0) while (NbPY < DeltaY) {Faire_un_pas_sur_Y(); NbPY++;}
     if (DeltaY == 0) while (NbPX < DeltaX) {Faire_un_pas_sur_X(); NbPX++;}
     if ((DeltaX != 0) && (DeltaY != 0)) Effectuer_un_mouvement_vectoriel(DeltaX, DeltaY, DX_sup_a_DY);}}

void Effectuer_un_deplacement_gco() {
  // Ancien_X et Ancien_Y sont mis à jour par les routines qui font faire un pas aux moteurs.
  boolean DX_sup_a_DY;
  int DeltaX, DeltaY; // Nombre de pas à effectuer sur les moteurs.
  int X,Y; // Les coordonnées à atteindre exprimées en dixième de mm :
  X = int(Position_X * 10 * Coef_poulie); // Abscisse à atteindre en cinquièmes de mm.
  Y = int(Position_Y * 10 * Coef_poulie); // Ordonnée à atteindre en cinquièmes de mm.
  DX_Positif = (X >= Ancien_X); DY_Positif = (Y >= Ancien_Y); // Détermine les sens de déplacement.
  if (DX_Positif) DeltaX = X - Ancien_X; else DeltaX = Ancien_X - X;
  if (DY_Positif) DeltaY = Y - Ancien_Y; else DeltaY = Ancien_Y - Y;
  // Ci-dessous division par 2 car un PAS moteur engendre un déplacement de 0.2mm
  DeltaX = DeltaX / 2;  DeltaY = DeltaY / 2; DX_sup_a_DY = (DeltaX >= DeltaY);
  Traiter_les_mouvements_du_LASER(DeltaX, DeltaY, DX_sup_a_DY);}

void Capture_Informations_Fichier() {
  Nom_Fichier[3] = Indice[Num_Img_indexee]; // Construire le Nom du Fichier indexé.
  if (pf_open(Nom_Fichier)) Probleme_de_lecture();
  boolean Ligne_a_ignorer; Ligne_G90_trouvee = false;
  Num_Ligne = 1; uint8_t buf[Taille_tampon]; // Tampon de travail.
  unsigned long pos = 0; // Position courante dans le fichier.
  //++++++++++++++++++++++++++++++ // Lecture ligne à ligne du fichier. +++++++++++++++++++++++++++++++++ 
  while (1) {// Sera réalisé tant que "break" non rencontré.
    UINT PLFO; // Pointeur de Lecture dans le Fichier Ouvert.
    //On lit le fichier octet par octet en mettant 1 en deuxième paramètre :
    if (pf_read(buf, 1, &PLFO)) Probleme_de_lecture();
    char carac = (char)buf[0]; //Pour la lecture, on stocke dans la variable caractere.
    //============================ Traiter la ligne construite ===========================
    if(carac=='\n' ){
       //Il faut rajouter le caractère de fin " \0 " pour marquer la fin de chaîne.
       Tampon[pos] = '\0'; Longueur_ligne_de_code = --pos;
       pos = 0; //On réinitialise la position à 0 pour une nouvelle lecture recommencer à zéro.
       //-------------------------- Traiter Contenu_ligne[I] -------------------------
       for (byte I = 0; I<Taille_tampon; I++) Contenu_ligne[I] = String(' ');
       for (byte I = 0; I<Longueur_ligne_de_code; I++) Contenu_ligne[I] = String(Tampon[I]);
       //------------------------- Traiter la première ligne -------------------------
       if (Num_Ligne == 1) { // Ci-deoous : Extraire la première ligne dans TITRE.
           for (byte I = 0; I<Taille_tampon; I++) Titre[I] = Tampon[I];}
       //------------------------- Traiter la deuxième ligne -------------------------
       if (Num_Ligne == 2) {// Ci-dessous : Extraire le nombre de lignes.
       Decale_la_chaine(); Taille = atol(Tampon); Tampon[0] = ';';}
       //---------------------- Traiter la ligne Origine Image -----------------------
       if (Ligne_G90_trouvee) {Ligne_G90_trouvee = false; Traite_G();
           Origine_Image_X = Position_X; Origine_Image_Y = Position_Y; break;}
           // Doit être placée avant le traitement de la ligne G90.
       //------------------------ Repérer les lignes à ignorer -----------------------
       Ligne_a_ignorer = false;
       if (Longueur_ligne_de_code == 0) Ligne_a_ignorer = true;
       if (Tampon[0] == ';') Ligne_a_ignorer = true; // Ignorer les remarques.
       if ((Tampon[0] == 'G') && (Tampon[1] == '9'))
          {Ligne_a_ignorer = true; Ligne_G90_trouvee = true;} // Ignorer G90.
          // Le test ci-avant est important car il rare la première ligne avec des coordonnées
          // différentes de zéro et pas de paramètre pour le pourcentage du LASER.
       Num_Ligne++;}
    else{Tampon[pos] = (char)buf[0]; pos++;}}} //Si ce n'est pas la fin de ligne, on continue.

void Ouvrir_le_fichier() { // Si erreur détectée en ouverture : Impose un RESET.
  Nom_Fichier[3] = Indice[Num_Img_indexee]; // Construire le Nom du Fichier indexé.
  if (pf_open(Nom_Fichier)) Probleme_de_lecture();} // C'est Probleme_de_lecture() qui impose un RESET.

void Initialiser_les_caracteristiques_du_fichier() {Saisie_Informations_Fichier = true;
         Ouvrir_le_fichier(); // Si problème d'ouverture bouclera sur une demande de RESET;
         Traite_ligne_a_ligne_le_fichier(); Saisie_Informations_Fichier = false;}

void Positionner_LASER_sur_Origine_Image(boolean Centrage_sur_le_plateau) {
  if (Existe[Num_Img_indexee]) {STOP = false;
      if (Centrage_sur_le_plateau) {Origine_Image_X = 112.5; Origine_Image_Y = 112.5;}
      else Initialiser_les_caracteristiques_du_fichier();
      Capturer_Origine_machine();  // Commence par activer les moteurs passe (et laisse) en "Rapide".
      Position_X = Origine_Image_X; Position_Y = Origine_Image_Y;
      Effectuer_un_deplacement_gco(); Passer_en_VEILLE();}}    

void Traiter_PB_Syntaxe() {
  Ecran_Noir = false; // Nécessaire pour afficher l'état machine.
  if (Type_Erreur > 0) {
     switch (Type_Erreur) { // Ci-dessous : Degager_le_plateau() coupe aussi le LASER.
       case 1 : {Affiche_ERR_et_attente_BP(286, 20); break;} // Ne dégage le plateau
       case 2 : {Affiche_ERR_et_attente_BP(306, 20); break;} // que si mode gravure.
       case 3 : Affiche_ERR_et_attente_BP(32,13);}
     AFFICHE_etat_machine(); ATTENDRE_un_BP();} // Affiche l'instruction qui a généré le problème.
  else if (Mode_Verification_Programme) {u8g.firstPage(); do { // Ci-dessous : Affiche syntaxe correcte.
           u8g.setPrintPos(0, 25); Aff_TEXTE_EEPROM(336,8); Aff_TEXTE_EEPROM(346,9);}
           while(u8g.nextPage()); ATTENDRE_un_BP();}}

void VERIFIER_LE_FICHIER() { // Si PB d'ouverture bouclera en imposant un RESET.
  if (Existe[Num_Img_indexee]) {Ouvrir_le_fichier(); Mode_Verification_Programme = true;
      Ecran_Noir = false; Num_BP = 0;  Mouvements_Rapide = false; Traite_ligne_a_ligne_le_fichier();
      if (!Saisie_Informations_Fichier) Traiter_PB_Syntaxe();} // Rend compte du résultat de l'analyse.
  else BIP(); // Faire un BIP si fichier non valide sur la carte SD.
  Liste_contenu_SD();}
  
void PYROGRAVER() {
  if (Existe[Num_Img_indexee]) {// Il ne faut capturer l'origine que si le fichier s'ouvre.
      Ouvrir_le_fichier(); Mode_Verification_Programme = false; STOP = false; // Pas de sortie demandée.
      Eteindre_Ecran(); Mode_pas_a_pas = false; Ecran_Noir = true; Num_BP = 0;
      Capturer_Origine_machine(); Traite_ligne_a_ligne_le_fichier();
      if (Type_Erreur > 0) {Traiter_PB_Syntaxe(); Position_degagee();} // Affiche l'erreur puis dégage.
      else Position_degagee(); // X = 10 mm et Y = 115 mm. (On dégage le plateau dans tous les cas.)
      Passer_en_VEILLE();}
  BIP(); // Faire un BIP si fichier non valide ou FIN de GRAVURE.
  Liste_contenu_SD();}
  
/* NOTE : dans la procédure PYROGRAVER() on constate que Position_degagee() est effectuée qu'il y a une
   erreur trouvée ou pas. Il serait logique de n'invoquer Position_degagee() qu'une seule fois avant le
   test sur Type_Erreur. Cette modification conduit à la même taille de code objet; Donc, les deux appels
   sont conservés car en mode PYROGRAVURE il vaut mieux informer l'opérateur de l'erreur trouvée, puis
   effectuer le dégagement du plateau. Ainsi l'utilisateur comprend le pourquoi de ce mouvement prématuré.
   Noter également que la sortie de Traite_ligne_a_ligne_le_fichier() coupe systématiquement le LASER
  
   REMARQUE : Dans ce démonstrateur il n'y a appel qu'à seulement deux "Menu_en_cours".
              Hors ils sont numérotés 6 et 7. C'est que leur ordre de repérage anticipe
              celui qu'ils auront dans le programme d'exploitation futur. */

