
/*   ********************************************************************************************
     *  Que ce soit pour des raisons mécaniques, électronique ou informatique, l'arrêt des      *
     *  mouvements alors que le LASER est à pleine puissance n'est jamais totalement exclu.     *
     *  C'est la raison pour laquelle ON NE LAISSE JAMAIS UNE PYROGRAVEUSE EN SERVICE SANS LA   *
     *  SURVEILLER. Toute personne dans le local doit porter des lunettes de protection         *
     *  spéciales adaptées à la couleur du LASER utilisé pour éviter tout incident potentiel    *
     *  généré par une déviation imprévue du faisceau du LASER qui équipe la machine. Si on va  *
     *  dans une pièce voisine, un détecteur d'incendie est OBLIGATOIRE !                       *
     ********************************************************************************************

/* Programme pour tester le bon fonctionnement du module LASER.
   Se contente d'envoyer sur toutes les sorties du multiplexeur une PWM à 1000Hz dont le rapport
   cyclique varie entre 0% et 100% en fonction d'une consigne donnée sur la ligne série de
   dialogue avec le Moniteur de l'IDE. Un filtrage des valeurs est effectué, et les décimales
   sont enlevées, sauf si on en donne trop. Si on donne des valeurs négatives ou supérieures
   à 100% un BIP() d'alerte est généré. La valeur du pourcentage retenu est affichée sur
   l'afficheur OLED de taille 3,5 pouces et sur le Moniteur de l'IDE. La valeur pour générer
   l'impulsion qui contrôle le module LASER est précisée sur l'écran OLED.
   
   UTILISATION DU PROGRAMME :
   --------------------------
   Dans une première étape, tester diverses valeurs sans brancher le LASER. Les LEDs actuellement
   branchées sur le multiplexeur s'illuminent toutes en fonction du pourcentage saisi.
   
   Quand tout semble correct, brancher le module LASER et le piloter avec s15 du multiplexeur.
   >>>>>>>>>> AVANT DE METTRE SOUR TENSION LE +12V DE PUISSANCE, PRENDRE LES PRÉCAUTIONS
   >>>>>>>>>> INDIQUÉES DANS L'ENCADRÉ CI-DESSUS. Et particulièrement le port des lunettes spéciales.
   Diriger le module LASER vers une plaque métallique absorbante, ou vers un mur éloigné de plusieurs
   mètres. La divergence du faisceau le rend inopérent à longue distance. Vérifier son comportement
   et la consommation sur le +12V. (Avec une alimentation pouvant fournir largement 4A à 5A.
   
   DIFFERENTS BRANCHEMENTS : Rien à modifier par rapport à P17, Sauf le LASER sur la sortie 15
                             du multiplexeur et uniquement au moment d'effectuer les tests en réel.
       
                                               PGM                   RAM dynamique
 PGM de génération PWM.                    14316 Octets (46%)      725 Octets (35%)    */

//--------------------- Déclarations pour le matériel -----------------------
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();
//--------------------- Définition des broches d'E/S ------------------------
#define BUZZER 4
//------------------------ Constantes du programme --------------------------
#define NB_MAX  16 // Nombre de sorties utilisées.
#define TailleTexte 10 // Taille réservée pour LGR_MAX plus le délimiteur.
//--------------------------- Variables du programme ---------------------------
byte Num_Sortie;
int Pourcentage, Pulse;
char TEXTE[TailleTexte]; // Place pour LGR_MAX caractères plus le délimiteur.
byte I = 0; // Pointeur dans la chaine TEXTE.
int ARGUMENT;
boolean Valeur_correcte;

void setup() {
  u8g.setFont(u8g_font_6x10); // Police de matrices de 6x10 pixels lettres MJA/min comprises.
  pwm.begin(); // Initialise le module multiplexeur 
  pwm.setPWMFreq(1000);  // 1000Hz pour piloter le module LASER
  Pourcentage = 0;
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW);
  Serial.begin(115200);  I=0; // Au départ la chaine est vide.
  while(Serial.available() > 0) {Serial.read(); delay(5);}}  // Purger entièrement le FIFO :
  
void loop() {
  // Placer les valeurs en affichage et sur les sorties du multiplexeur :
  Num_Sortie = 0; Pulse = map(Pourcentage, 0, 100, 0, 4095);
  while (Num_Sortie < NB_MAX) {pwm.setPWM(Num_Sortie, 0, Pulse); Num_Sortie ++;}
  PAGE1();
  // ++++++++++++++++++++++++++ Saisir le pourcentage désiré ++++++++++++++++++++++
  Valeur_correcte = false;
  while (!Valeur_correcte) {
      Serial.println(); Serial.print("Indiquer la valeur voulue :");
      while (I==0) Traiter_la_ligne_USB_du_Moniteur(); Pourcentage = ARGUMENT; I=0;
      Valeur_correcte = (Pourcentage >= 0) && (Pourcentage <= 100);
      if (Valeur_correcte) {Serial.print(" >>> Pourcentage = "); Serial.println(Pourcentage);}
      else BIP();}}
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
void BIP() {digitalWrite(BUZZER, HIGH); delay(100); digitalWrite(BUZZER, LOW);}

void Traiter_la_ligne_USB_du_Moniteur() {
  while((Serial.available() > 0) && (I < (TailleTexte-1))) // Si FIFO non vide lire les caractères.
       {TEXTE[I] = Serial.read(); I++; delay(5);} // Délai impératif.
  TEXTE[I] = '\0'; // Placer le délimiteur de fin de texte.
  // Purger entièrement le FIFO si plus de LGR_MAX caractères envoyés.
  ARGUMENT = atof(TEXTE);}

void CADRES() {u8g.drawFrame(0,0,128,15); u8g.drawFrame(0,17,128,47);} // Cadre TITRE et cadre données.

void PAGE1() {
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //================ Définition des éléments à afficher ================
  CADRES();
  u8g.setPrintPos(15, 10); u8g.print("Rapport cyclique.");
  u8g.setPrintPos(20, 54); u8g.print(" Pulse = "); u8g.print(Pulse);
  u8g.setScale2x2(); // Affiche le % en format double.
  u8g.setPrintPos(21, 21); u8g.print(Pourcentage); u8g.print(" %");
  u8g.undoScale();} // Revient au format simple.
  //====================================================================
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.




