/* Programme indépendant pour simuler la gestion vectorielle des moteurs.

Ce programme affiche sur la ligne série USB du moniteur l'évolution des paramètres critiques lorsque
la procédure "Effectuer_un_mouvement_vectoriel()" est activée.

UTILISATION DU PROGRAMME :
==========================
Saisir à l'invitation la valeur de Delta X comprise entre 0 et 2250. (Valeurs maximales sur la machine.)
Saisir à la demande la valeur de Delta Y comprise entre 0 et 2250. (Valeurs maximales sur la machine.)
Si l'une des deux valeurs est hors fourchette les deux sont redemandées.
>>>>>> ATTENTION : Indiquer Delta X > Delta Y ou le programme passe en boucle infinie !

REMARQUE : Pour les valeurs affichées nous avons respectivement :
           X : Nombre d'incréments effectués sur X.
           Y : Nombre d'incréments effectués sur Y.
           C : Compteur de déplacement pour respecter la valeur de Rapport.

Branchements à effectuer : Aucun changement.

>>>>>> Dans ce démonstrateur les variables telles que Compteur, Cumul, Decimale, Rapport sont déclarées
       en variables globale dans le but de pouvoir en afficher les valeurs sur le moniteur de l'IDE.
       Dans le programme d'exploitation de la machine il faudra les intégrer dans la procédure qui
       gère les mouvements vectorisés.
>>>>>> La procédure devra parer la "division par zéro" si le plus petit des deux Delta X ou Y est nul.

                                                                    PGM               RAM dynamique
Programme avec gestion de la ligne série du moniteur :         5676ctets (18%)     359 Octets (17%) */

#define TailleTexte 10 // Taille réservée pour LGR_MAX plus le délimiteur.

//----------------------- Déclaration des variables -------------------------
char TEXTE[TailleTexte]; // Place pour LGR_MAX caractères plus le délimiteur.
byte I = 0; // Pointeur dans la chaine TEXTE.
int ARGUMENT;
boolean Valeurs_correctes;

boolean Fini;
int Compteur;
int Rapport; // Impérativement un int car peut aller jusqu'à 2250.
float Cumul, Decimale;
int DeltaX, DeltaY;
int NbPX, NbPY; // Nombres de pas réalisés. (Peuvent aller jusqu'à 2250.)

void setup() {
  Serial.begin(115200);   I=0; // Au départ la chaine est vide.
  while(Serial.available() > 0) {Serial.read(); delay(5);}}  // Purger entièrement le FIFO.

void loop() {
  // ++++++++++++++++++++ Saisir les valeurs DeltaX et pour DeltaY ++++++++++++++++
  Valeurs_correctes = false;
  while (!Valeurs_correctes) {
      Serial.println(); Serial.print("Indiquer DeltaX.");
      while (I==0) Traiter_la_ligne_USB_du_Moniteur(); DeltaX = ARGUMENT; I=0;
      Serial.print(" >>> DeltaX = "); Serial.println(DeltaX);
      Serial.print("Indiquer DeltaY.");
      while (I==0) Traiter_la_ligne_USB_du_Moniteur(); DeltaY = ARGUMENT; I=0;
      Serial.print(" >>> DeltaY = "); Serial.println(DeltaY);
      Valeurs_correctes = (DeltaX >= 0) && (DeltaX < 2251) && (DeltaY >= 0) && (DeltaY < 2251);}
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Traiter_les_mouvements_du_LASER(); Serial.println();}

void Traiter_la_ligne_USB_du_Moniteur() {
  while((Serial.available() > 0) && (I < (TailleTexte-1))) // Si FIFO non vide lire les caractères.
       {TEXTE[I] = Serial.read(); I++; delay(5);} // Délai impératif.
  TEXTE[I] = '\0'; // Placer le délimiteur de fin de texte.
  // Purger entièrement le FIFO si plus de LGR_MAX caractères envoyés.
  ARGUMENT = atoi(TEXTE);}

void Affiche_Position() {if (NbPX<10) Serial.print('0'); Serial.print(NbPX);
     Serial.print("   "); if (NbPY<10) Serial.print('0'); Serial.print(NbPY);
     Serial.print("   "); Serial.print(Compteur);
     Serial.print("   "); Serial.println(Cumul,4);}

void Affiche_la_premiere_ligne() {Serial.println(); Serial.println("X    Y    C   Cml.");}

void Effectuer_un_mouvement_vectoriel() {
  // Attention ci-dessous cumul est utilisé pour un calcul préalable.
  Cumul = float(DeltaX) / float(DeltaY); // Toujours diviser le plus grand par le plus petit.
  Decimale = Cumul - ((int) Cumul); Rapport = Cumul - Decimale ;
  // Afficher les valeurs critiques calculées ci-avant :
  Serial.println(); Serial.print("DeltaX / DeltaY = "); Serial.println(Cumul,3);
  Serial.println(); Serial.print("Rapport = "); Serial.println(Rapport);
  Serial.println(); Serial.print("Decimale = "); Serial.println(Decimale,3);
  Affiche_la_premiere_ligne();
  Cumul = 0.0; NbPX=0; NbPY=0; Fini = false;
  while (!Fini) {
    Compteur = 0;
    while (Compteur < Rapport) {
       if (NbPX < DeltaX) {NbPX++; Compteur++; Cumul = Cumul + Decimale; Affiche_Position();}
       else Compteur = Rapport;} // Forcer la sortie si NbPX = DeltaX.
    NbPY++; Affiche_Position();
    if (int(Cumul + 0.01) > 0) {
        if (NbPX < DeltaX) 
           {NbPX++; Cumul = abs(Cumul - 1); if (Cumul > 1) Cumul = abs(Cumul - 1); Affiche_Position();}}
    Fini = NbPX == DeltaX;}
  while (NbPY < DeltaY) {NbPY++; Affiche_Position();}}

void Traiter_les_mouvements_du_LASER() {
  NbPX = 0; NbPY = 0;
  if ((DeltaX > 0) || (DeltaY > 0)) {// Ne rien faire si les deux déplacements sont nuls.
     // Il est préférable de tester le cas où on retrouve sur plusieurs lignes des coordonnées
     // identiques. Ce n'est pas le cas pour les fichiers de LASER GRBL mais le fichier
     //  pourrait provenir d'un autre programme ou avoir été modifié manuellement.
     if (DeltaX == 0) {Affiche_la_premiere_ligne(); while (NbPY < DeltaY) {NbPY++; Affiche_Position();}}
     if (DeltaY == 0) {Affiche_la_premiere_ligne(); while (NbPX < DeltaX) {NbPX++; Affiche_Position();}}
     if ((DeltaX != 0) && (DeltaY != 0)) Effectuer_un_mouvement_vectoriel();}}
