/* Reconstruction complète pour intégration de toutes les procédures extraites des démonstrateurs.
   Cette version optimise les divers modules et ajoutes les compléments nécessaires.
   
   >>>>>>>>>> Version du 7 Juin 2020. >>> Donnée inscrite en EEPROM.
   
   UTILISATION DU PROGRAMME :   Les diverses fonctions et options sont décrites dans le manuel d'utilisation
   ==========================   ainsi que dans les fiches "opérationnelles".
                     
 Branchements à effectuer :
 --------------------------
 Multiplexeur  S0 : Branchement de la LED Verte du CLAVIER.
 Multiplexeur  S1 : Vers LED verte pulses moteur X.
 Multiplexeur  S2 : Vers LED bleue pulses moteur Y.
 Multiplexeur  S3 : Branchement de la LED "triple" Bleue du mode VEILLE sur les moteurs.
 Multiplexeur  S4 : Branchement de la LED "Répériteur LASER".
 Multiplexeur  S6 : LED "triple" Rouge Origine machine non initialisée.
 Multiplexeur  S9 : LED "triple" Verte : Origine machine capturée.
 Multiplexeur S12 : Vers LED rouge de Débordement limites Machine.
 Multiplexeur S13 : Pilotage du Relais d'alimentation du LASER.
 Multiplexeur S15 : Sortie de pilotage PWM LASER.
                                                                PGM                 RAM dynamique
Noyau de base avec MENU de BASE :                         12228 Octets (39%)       464 Octets (22%)
Mise en place du Menu FICHIER :                           23952 Octets (77%)       851 Octets (41%)
Mise en place du Menu GRAVER sans "la motorisation" :     24660 Octets (80%)       859 Octets (41%)
Ajout des procédures pour la motorisation" :              26106 Octets (84%)       861 Octets (42%)
Menu Mode MANUEL mis en place :                           27232 Octets (88%)       866 Octets (42%)
Programme complet avec toutes les modifications :         29950 Octets (97%)       865 Octets (42%)
Ajout option systeme "Recoipie Répétiteur LASER" :        30252 Octets (98%)       914 Octets (44%)
Corrections et compléments divers :                       30364 Octets (99%)       850 Octets (41%)
Supprimer l'option de mesure sur le clavier est impératif pour pouvoir ajouter :
Nombreuses modifications et Mode LASER continu :          30636 Octets (99%)       852 Octets (41%)
Codes gco de complément et diverses améliorations :       30574 Octets (99%)       852 Octets (41%)
Optimisations et Indiquer la durée pour la gravure :      30578 Octets (99%)       852 Octets (41%)
Divers correctifs et touche X+ acceptée en menu de base : 30616 Octets (99%)       853 Octets (41%)
Commentaires en analyse de fichier sur M0 et M1 :         30688 Octets (99%)       855 Octets (41%)
Regroupé les u8g.setPrintPos suivi de Aff_TEXTE_EEPROM    30452 Octets (99%)       855 Octets (41%)
Introduit "Ignorer Capture Origines Machine:              30488 Octets (99%)       856 Octets (41%)
Divers correctifs et simplifications :                    30576 Octets (99%)       857 Octets (41%)
Gestion d'un relais de coupure automatique du LASER :     30590 Octets (99%)       857 Octets (41%)
Introduction du "sens gyratoire" en listage Images :      30634 Octets (99%)       857 Octets (41%)
X+ valide l'ITEM indexé si menu SYSTEME :                 30652 Octets (99%)       857 Octets (41%)
Modif. pour "LED Moteurs" à trois ciuleurs.               30690 Octets (99%)       857 Octets (41%)

Affiche PILE - TAS = 1054 donc pas de problème de collision de PILE à redouter.         */

//--------------------- Déclarations pour le matériel -----------------------
#include <Wire.h> // Gère les protocole I2C sur A4 et A5 pour le Multiplexeur.
#include <EEPROM.h>
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)
#include "PetitFS.h"
FATFS fs;     // Zone de travail pour le volume SD : Objet système.
#include <Adafruit_PWMServoDriver.h> // déclaration du Multiplexeur.
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

//------------- Définition des broches d'entrées analogiques ----------------
#define Ligne_trois_touches 14 // A0 pour lire le clavier à 3 BP. 
#define Ligne_deux_touches 15 // A1 pour lire le clavier à 2 BP. 
#define BP_central 16 // A2 pour lire le bouton central.
#define Pilotage_capteurs_Origines 17 // A3 active les modules uniquement aux moments pertinents.
#define Capteur_Origine_X  20 // A6 pour lire le capteur sur butée X.
#define Capteur_Origine_Y  21 // A7 pour lire le capteur sur butée X.
//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_B 2
#define Codeur_A 3
#define BUZZER 4
#define Moteurs_actifs 5 // Broche qui à l'état haut fait passer le moteur en mode "non alimenté".
#define Sens_sur_X 6 // Pilotage du sens de rotation sur X.
#define Un_pas_sur_X 7 // Pilotage "Faire un pas" sur X.
#define Sens_sur_Y 8 // Pilotage du sens de rotation sur Y.
#define Un_pas_sur_Y 9 // Pilotage "Faire un pas" sur Y.
#define chipSelect 10
//------------------- Déclaration des constante du programme ----------------
#define Seuil_Origine_X 511 // Tension moyenne entre Actif et neutre sur X. @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Seuil_Origine_Y 513 // Tension moyenne entre Actif et neutre sur Y. @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Course_MAX_sur_X 228.0 // Déplacement maximal possible sur la machine @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Course_MAX_sur_Y 202.0 // Déplacement maximal possible sur la machine @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// ATTENTION : Si Course_MAX_sur_X modifiée reprendre la valeur dans void Faire_un_pas_sur_X.
// ATTENTION : Si Course_MAX_sur_Y modifiée reprendre les deux valeurs dans void Capturer_Origine_machine.
//             Si Course_MAX_sur_Y modifiée reprendre la valeur dans void Faire_un_pas_sur_X.
#define Tmp_moyen_pour_une_ligne 2.17 // Nb moyen estimé de lignes gravées par seconde @@@@@@@@@@@@@@@@@@@@@

#define NB_tests_antirebonds 30 // Nombre de tests de stabilisation B.P.
#define Taille_tampon 28 // Taille des tampons pour les chaînes de caractères pour aller jusqu'à 25 caractères.
//----------------------- Déclaration des variables -------------------------
boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean Mdf_Pmax_en_gravant;
boolean BP_central_a_traiter;
boolean DX_Positif, DY_Positif; //, DX_sup_a_DY;
boolean Ecran_Noir;
boolean Fin_demandee;
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean LASER_actif;
boolean LASER_permanent;
boolean Lecteur_OK;
boolean Ligne_M0_trouvee;
boolean Ligne_M1_trouvee;
boolean Ligne_M2_trouvee;
boolean Ligne_M5_trouvee;
boolean Ligne_Premier_G0; // Permet de mémoriser les coordonnées Image.
boolean Limites_franchies;
boolean Mode_pas_a_pas;
boolean Mode_VEILLE;
boolean Mode_Verification_Programme;
boolean Origine_machine_effective;
boolean Origine_machine_ignoree;
boolean OrigineX_effective, OrigineY_effective;
boolean Pyrogravure_en_cours;
boolean Saisie_Informations_Fichier;
boolean Sens_positif; // Sens de rotation.
boolean STOP; // Permet de sortir d'une gravure si fin demandée.
boolean Temps_moyen;
boolean Test_Capteurs_OK;
volatile boolean Rotation;  // Indique une activation du codeur rotatif.

boolean Existe[10];
char Indice[10] = {'0','1','2','3','4','5','6','7','8','9'};
char Nom_Fichier[9] = "IMGN.GCO";
char Tampon[Taille_tampon];
char Titre[Taille_tampon];
String Contenu_ligne[Taille_tampon];

byte Curseur; // Définit la ligne de texte écran où doit se trouver le curseur.
byte Increment_Puissance; // Incrément / Décrément en mode LASER permanent.
byte Longueur_ligne_de_code; 
byte Menu_en_cours; // Page-écran à rétablir suite à "Ecran NOIR".
byte Num_BP; // Désigne le bouton poussoir activé sur le clavier.
byte Num_Img_indexee;
byte Pourcentage; // Consigne de Pourcentage de puissance du LASER.
byte Puissance_MAX_sur_le_LASER; // Puissance qui sera appliquée quand Pourcentage = 100;
byte SAV_Pourcentage;
byte Type_Erreur;

int Ancien_X, Ancien_Y; // Positions exprimées en dixièmes de mm.
int Delai_par_pas_elementaire;
int U_sur_X, U_sur_Y; // Tensions sur les capteurs d'origine machine.

unsigned long Num_Ligne, Taille;

float Position_X, Position_Y, Origine_Image_X, Origine_Image_Y, Avance_par_pas;
//======================================================================================

void setup() {
  //---------------------- Définition des vecteurs d'interruption ----------------------
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  // Broche de l'interruption 1 (E/S D3) :
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pwm.begin(); // Initialise le module multiplexeur.
  pwm.setPWMFreq(1000); // Fréquence particulière 1000Hz pour piloter le module LASER.
  Couper_le_Relais_LASER();
  Couper_le_LASER(); // impératif car RESET peut être déclenché durant la pyrogravure.
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(chipSelect, OUTPUT); digitalWrite(chipSelect,HIGH);
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW); pinMode(Pilotage_capteurs_Origines,OUTPUT);
  pinMode(Moteurs_actifs, OUTPUT); Passer_en_VEILLE(); // Passe les moteurs en veille et allume la LED bleue.
  pinMode(Sens_sur_X, OUTPUT); pinMode(Un_pas_sur_X, OUTPUT); digitalWrite(Un_pas_sur_X, LOW);
  pinMode(Sens_sur_Y, OUTPUT); pinMode(Un_pas_sur_Y, OUTPUT); digitalWrite(Un_pas_sur_Y, LOW);
  pinMode(11,INPUT); pinMode(12,INPUT); pinMode(13,INPUT);
  Forcer_a_zero_une_sortie_multiplexeur(1);  // Éteindre LED pulse moteurs X.
  Forcer_a_zero_une_sortie_multiplexeur(2);  // Éteindre LED pulse moteurs Y.
  Forcer_a_zero_une_sortie_multiplexeur(12); // Éteindre LED Débordement.
  digitalWrite(BP_central, HIGH); // Valide R PUL-UP sur A2.
  Front_sur_A(); Front_sur_B(); // Routines d'IRQ pour le codeur rotatif.
  //---------------------- Initialisation des diverses variables -----------------------
  Num_Img_indexee = 0; BP_central_a_traiter = false; Mode_pas_a_pas = false; LASER_permanent = false;
  Curseur = 2; // Valeur 2 pour pointer l'item PYROGRAVURE car sur RESET une rotation "négative" est détectée.
  Saisie_Informations_Fichier = false; Mdf_Pmax_en_gravant = false; Temps_moyen = false;
  Delai_par_pas_elementaire = 10; // Mouvements manuels rapides par défaut soit 10mS.
  Puissance_MAX_sur_le_LASER = 30; Forcer_a_zero_une_sortie_multiplexeur(4);
  Limites_franchies = false; Pyrogravure_en_cours = false;
  //--------------------------------- Actions initiales /-------------------------------
  Allume_LED_Bleue(); Tester_les_capteurs_Origine(); if (!Test_Capteurs_OK) Allume_LED_Rouge();
  // LED "violette si l'un des capteurs est masqué pour prévenir qu'il faut dégager le plateau.
  MENU_de_base(); Origine_machine_ignoree = false; Tester_le_Clavier();
  if (Num_BP ==4) {Origine_machine_ignoree = true; Allume_LED_Rouge(); Allume_LED_Verte();}
  Relecture_carte_SD();
  // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //@@@@@@@@@@@@@ Ci-dessous code ajouté pour afficher la place disponible. @@@@@@@@@@@
  // u8g.firstPage(); do {u8g.setPrintPos(22, 35); u8g.print("PILE - TAS = ");
  // u8g.print(SRAM_LIBRE());} while(u8g.nextPage()); ATTENDRE_un_BP();
  // Ci dessus : Tant que USB non activée : Ne rien faire. (Attendre la frappe d'un caractère au clavier.
  //@@@@@@@@@@@@@ ******************************************************** @@@@@@@@@@@@
  // Pour que le test puisse être réalisé il faut transformer la procédure
  // void Afficher_Ecran_PYROGRAVURE() {} en procédure vire.
  // Passer son corps en remarque par encadrement avec /* et */.
  }
  
  //@@@@@@@@@@@@@@ Ci-dessous fonction pour afficher la place disponible @@@@@@@@@@@@@@
  //int SRAM_LIBRE() { // Fonction qui retourne la taille de SRAM disponible.
  //extern int __heap_start, *__brkval; // Déclaration des deux pointeurs dédiés.
  //byte BIDON; // Dernière variable allouée, donc occupe le "haut" du TAS.
  //if (__brkval == 0) {return (int) &BIDON -(int) &__heap_start;}
  //else {return (int) &BIDON -(int) __brkval;}}
  //@@@@@@@@@@@@@@ ***************************************************** @@@@@@@@@@@@@@}

void loop() { // Dans la boucle de base le clavier ne gère que l'économiseur d'écran.
  MENU_de_base(); // L'écran est rafraichi en permanence mais ce n'est pas pénalisant.
  //--------------------------------- Traite le B.P. central ---------------------------
  Teste_activation_BP_codeur_incremental();
  if (BP_central_a_traiter) {BP_central_a_traiter = false; Activation_item_menu_de_base();}
  //--------------------- Traite la rotation du codeur incrémental ---------------------
  if (Rotation) TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();
  //---------------------- Traiter la requête de l'un des cinq B.P. --------------------
  Tester_le_Clavier();
  if (Num_BP !=0) {
      switch(Num_BP) {
        case 2 : {Inverser_bascule_Ecran_Noir(); if (!Ecran_Noir) Retablir_Ecran_OLED(); break;}
        case 3 : {Activation_item_menu_de_base(); break;}
      default : BIP();} Num_BP =0;}}
     
void Activation_item_menu_de_base() {
      BP_central_a_traiter = false;
      switch(Curseur) {
        case 1 : {Menu_en_cours = 1; Menu_PYROGRAVER(); break;}
        case 2 : {Menu_en_cours = 2 ; Menu_FICHIER(); break;}
        case 3 : {Menu_Mode_MANUEL(); break;}
        case 4 : {Menu_OPTIONS(); break;}
        case 5 : Menu_SYSTEME();}}

//====================================== Routines de servitude =====================================

void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 100 expérimentale.

void Front_sur_A(){ // Interruption quand A change d'état avant B.
  if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
  if (digitalRead(Codeur_A) != A_set) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();}}}
  
void Front_sur_B(){ // Interruption quand B change d'état avant A.
  if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
  if (digitalRead(Codeur_B) != B_set) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();}}}

void Delay_1mS() {delay(1);}
void Delay_30mS() {delay(30);}
void Delay_100mS() {delay(100);}

void BIP() {digitalWrite(BUZZER, HIGH); Delay_100mS(); digitalWrite(BUZZER, LOW);}

void Teste_activation_BP_codeur_incremental() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     //----------- Attendre le relacher du B.P. central. ----------
     Allume_LED_clavier();
     while (analogRead(BP_central) < 512); // Boucler sans rien faire pour attendre le relâché.
     Eteindre_LED_clavier();
     //------------------------------------------------------------
     BP_central_a_traiter = true;}}

void Tester_le_Clavier() {// Les valeurs tiennent compte de VCC = 4V car fourni par la carte NANO.
  Num_BP = 0; // Signale aucun B.P. utilisé.
  U_sur_X = analogRead(Ligne_trois_touches);
  if (U_sur_X < 744) {Num_BP = 1;
      if (U_sur_X < 466)  Num_BP = 2; if (U_sur_X < 162)  Num_BP = 3;}
  U_sur_Y = analogRead(Ligne_deux_touches);
  if (U_sur_Y < 696) {Num_BP = 5; if (U_sur_Y < 257)  Num_BP = 4;}
  if (Num_BP != 0) ATTENDRE_le_relacher();}

void Forcer_a_zero_une_sortie_multiplexeur(byte Sortie) {pwm.setPWM(Sortie, 0, 0);}

void ATTENDRE_le_relacher() {
  byte SAV_Num_BP = Num_BP;
  byte COMPTEUR = 0; Delay_30mS(); // Attendre_la_stabilisation du travail.
  Allume_LED_clavier();
  while (COMPTEUR < NB_tests_antirebonds) {// Attendre_la_stabilisation du repos :
         COMPTEUR++; Tester_le_Clavier(); if (Num_BP > 0) COMPTEUR = 0;}
  Eteindre_LED_clavier(); Num_BP = SAV_Num_BP;}

void Sortie_Multiplexeur_4095(byte Sortie) {pwm.setPWM(Sortie, 0, 4095);}

void Allume_LED_clavier() {Sortie_Multiplexeur_4095(0);}
void Eteindre_LED_clavier() {Forcer_a_zero_une_sortie_multiplexeur(0);}

void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  INT_active = true; // Armer un nouveau comptage.
  Rotation = false; // Prise en compte du traitement.
  //-------------------- Ajuster la valeur du curseur -------------------
  if ((Menu_en_cours == 1) || (Menu_en_cours == 2)) {// Indexer l'un des 10 fichiers.
     if (Sens_positif) {
        if ((Num_Img_indexee < 5) && (Num_Img_indexee != 0)) Num_Img_indexee--;
        else {if (Num_Img_indexee == 0) Num_Img_indexee = 5;
              else {Num_Img_indexee++; if (Num_Img_indexee == 10) Num_Img_indexee = 4;}}}
     else{if (Num_Img_indexee == 5) Num_Img_indexee = 0;
          else {if (Num_Img_indexee > 5) Num_Img_indexee--;     
                else {Num_Img_indexee++; if (Num_Img_indexee == 5) Num_Img_indexee = 9;}}}
     Liste_contenu_SD();} // Affiche la liste de l'état des fichiers et positionne '<' sur le fichier pointé.
  else {if (Sens_positif) {if (Curseur > 1) Curseur--; else Curseur = 5;}
        else {Curseur++; if (Curseur == 6) Curseur = 1;}}}

void Teste_SORTIR_du_MENU() {
  Teste_activation_BP_codeur_incremental(); // Si B.P. activé saisir les caractéristiques de l'image.
  if (BP_central_a_traiter) {Fin_demandee = true; BP_central_a_traiter = false;}}
 
void ATTENDRE_un_BP() {
  boolean Allumer;
  Num_BP = 0;
  while (Num_BP == 0) {Tester_le_Clavier(); Delay_100mS();
         Allumer =!Allumer; if (Allumer) Allume_LED_clavier(); else Eteindre_LED_clavier();}
  ATTENDRE_le_relacher();}

//====================== Routines pour traiter les capteurs d'origines machine =====================
void Mesurer_U_Capteur(boolean Sur_X) {
  #define Nb_Iterations 20 // Mesure avec Nb itérations pour l'intégration "d'antiparasitage".
  // La valeur de 20 est choisie expérimentalement et semble un bon compromis entre des
  // mesures fiables et une rapidité suffisante de rotation des moteurs.
  long Valeurs_lues;
  byte Compteur = Nb_Iterations;
  Valeurs_lues = 0;
  for (int I = 0; I < Nb_Iterations; I++) // Cumuler N mesures. 
      {if (Sur_X) Valeurs_lues = Valeurs_lues + analogRead(Capteur_Origine_X);
       else Valeurs_lues = Valeurs_lues + analogRead(Capteur_Origine_Y);}
  if (Sur_X) U_sur_X = int(Valeurs_lues / Nb_Iterations); else U_sur_Y = int(Valeurs_lues / Nb_Iterations);}
  
void Traiter_Origines_sans_allumer() {
  Mesurer_U_Capteur(true); // Donc mesure avec anti-rebonds sur le capteur X.
  Mesurer_U_Capteur(false); // Donc mesure avec anti-rebonds sur le capteur Y.
  OrigineX_effective = (U_sur_X > Seuil_Origine_X); OrigineY_effective = (U_sur_Y > Seuil_Origine_Y); 
  digitalWrite(Pilotage_capteurs_Origines, LOW);}

void Mesurer_U_sur_capteurs_Origines() {
  digitalWrite(Pilotage_capteurs_Origines, HIGH); Delay_1mS(); Traiter_Origines_sans_allumer();}

void Affiche_en_X_15(byte Y, int PTR, byte LGR)   {u8g.setPrintPos(15, Y); Aff_TEXTE_EEPROM(PTR,LGR);}

void Mesurer_sur_les_capteurs_origine() {// Clavier et codeur rotatifs ignorés.
  while (!Fin_demandee) {
    Teste_SORTIR_du_MENU();
    Mesurer_U_sur_capteurs_Origines();
    u8g.firstPage(); do {
    Affiche_en_X_15(15,184,14); u8g.print(U_sur_X);
    Affiche_en_X_15(30,198,14); u8g.print(U_sur_Y);
    Affiche_en_X_15(45,186,10);
    Affiche_Deux_points_espace_OUI_NON(OrigineX_effective);
    Affiche_en_X_15(60,200,10); 
    Affiche_Deux_points_espace_OUI_NON(OrigineY_effective);}
  while(u8g.nextPage());}}

void Tester_les_capteurs_Origine() {
  Traiter_Origines_sans_allumer(); // Ci-dessous les deux doivent avoir "Origine_effective".
  Test_Capteurs_OK = ((OrigineX_effective) && (OrigineY_effective));
  Mesurer_U_sur_capteurs_Origines(); // Ci-dessous les deux doivent avoir NON "Origine_effective".
  if ((OrigineX_effective) || (OrigineY_effective)) Test_Capteurs_OK = false;
  digitalWrite(Pilotage_capteurs_Origines, LOW);}  
    
//==================================== Procédures pour le LASER ====================================

void Activer_le_Relais_LASER() {Sortie_Multiplexeur_4095(13); Delay_100mS();}

void Couper_le_Relais_LASER() {Forcer_a_zero_une_sortie_multiplexeur(13);}

void Moduler_le_LASER() {// Place sur S15 la valeur correspondant à Pourcentage.
  int Pulse;
  if (!Mode_Verification_Programme){
     // Moduler uniquement si GRAVURE et non en vérification de fichier. 
     Pulse = map(Pourcentage, 0, 100, 0, (Puissance_MAX_sur_le_LASER * 40.95)); pwm.setPWM(15, 0, Pulse);}
     // Ci-dessus : 40.95 pour que multiplié par 100 on obtienne 4095 les 100% de rapport cyclique.
  Pulse = map(Pourcentage, 0, 100, 0, 1000); pwm.setPWM(4, 0, Pulse); // Allume le répétiteur LED.
  // Ci-dessus valeur 1000 pour diminuer la luminosité maximale sur la LED du répétiteur LASER.
  pwm.setPWM(4, 0, Pulse);}

void Couper_le_LASER() {
  Forcer_a_zero_une_sortie_multiplexeur(15); Forcer_a_zero_une_sortie_multiplexeur(4);}

//================================= Procédures pour la motorisation ================================

// Ci dessous : La boucle consomme 22 octets de moins que trois fois pwm.setPWM(N, 0, 0).
void Eteindre_les_trois_couleurs() {for (byte I=3; I< 10; I = I+3)  Forcer_a_zero_une_sortie_multiplexeur(I);}

// Ci-dessous : La procédure commune Allume_LED_Tricolore fait économiser 18 octets.
void Allume_LED_Tricolore(byte Sortie_multiplexeur) {Sortie_Multiplexeur_4095(Sortie_multiplexeur);}
void Allume_LED_Bleue() {Allume_LED_Tricolore(3);}
void Allume_LED_Rouge() {Allume_LED_Tricolore(6);}
void Allume_LED_Verte() {Allume_LED_Tricolore(9);}

void Traiter_la_LED_triple() {
   Eteindre_les_trois_couleurs();
   if (Mode_VEILLE) Allume_LED_Bleue(); // Si VEILLE allumer le Bleu prioritaire.
   // Note : Si mode VEILLE l'origine déjà capturée n'est pas fiable car peut être perturbée manuellement.
   else if (Origine_machine_effective) Allume_LED_Verte();
        else Allume_LED_Rouge();}

void Activer_les_moteurs() { // Ci-dessous delai 30mS pour stabilisation des moteurs.
  digitalWrite(Moteurs_actifs, LOW);  Mode_VEILLE = false;
  if (!Mode_Verification_Programme) Traiter_la_LED_triple(); Delay_30mS();}

void Passer_en_VEILLE() { digitalWrite(Moteurs_actifs, HIGH);
  Mode_VEILLE = true; Origine_machine_effective = false; Traiter_la_LED_triple();}

void Capturer_Origine_machine() {// Déplacer à gauche et vers l'arrière de la machine.
  if (!Origine_machine_ignoree) {
    Ancien_Y = 0; Ancien_X = 2500; // Valeurs arbitraires mais > aux limites machine.
    // Tourner dans le sens "négatif" pour X et positif pour Y :
    STOP = false; Pourcentage = 0; Activer_les_moteurs(); DX_Positif = false; DY_Positif = true;
    Aff_Item_une_seconde(248, 19); // Affiche "> Capture ORIGINE <
    // Vérifier OrigineX_effective et OrigineY_effective avant d'effectuer les mouvements idoines :
    Mesurer_U_sur_capteurs_Origines(); // Positionne OrigineX_effective et OrigineY_effective actuelles.
    while (!OrigineX_effective) {Faire_un_pas_sur_X(); Mesurer_U_sur_capteurs_Origines();}
    while (!OrigineY_effective) {Faire_un_pas_sur_Y(); Mesurer_U_sur_capteurs_Origines();}
    Origine_machine_effective = true; // Car le programme arrive ici que si les deux sont capturées.
    Traiter_la_LED_triple();}
  Ancien_X = 0; Ancien_Y = 2020; // Attentions : Ancien_X,Y en dixièmes de mm.
  Position_X = 0; Position_Y = 202.0;}

void Traiter_la_rotation_durant_la_gravure() {
  if (Sens_positif) {Puissance_MAX_sur_le_LASER = Puissance_MAX_sur_le_LASER + Increment_Puissance;
                     if (Puissance_MAX_sur_le_LASER > 100) {BIP(); Puissance_MAX_sur_le_LASER = 100;}}
  else {Puissance_MAX_sur_le_LASER = Puissance_MAX_sur_le_LASER - Increment_Puissance;
        if (Puissance_MAX_sur_le_LASER > 100) {BIP(); Puissance_MAX_sur_le_LASER = 0;}}
  Ecran_Noir = false;}

void Tester_le_Clavier_pour_gerer_Increment() {
  Tester_le_Clavier();
  if (Num_BP !=0) {
      switch(Num_BP) {
        case 1 : {Increment_Puissance = 1; break;}
        case 2 : {if (LASER_permanent) Ecran_Noir = !Ecran_Noir; else Ecran_Noir = true; break;}   
        case 3 : {Increment_Puissance = 2; break;}
        case 4 : {Increment_Puissance = 10; break;}
        case 5 : Increment_Puissance = 5;}}}

void Traiter_CLAVIER_et_codeur_rotatif() {// Invoquée uniquement durant Faire un pas sur X ou sur Y.
  if (Mdf_Pmax_en_gravant) {
      if (Rotation) { Rotation = false; // Prise en compte du traitement.
                      if (Pyrogravure_en_cours) Traiter_la_rotation_durant_la_gravure();}}
  Tester_le_Clavier_pour_gerer_Increment();} // Testé dans tous les cas pour allumer ou éteindre l'écran.

void Pulse_moteur(byte Sortie) {
  Sortie_Multiplexeur_4095(Sortie); Delay_1mS(); Forcer_a_zero_une_sortie_multiplexeur(Sortie);}

void Allume_debordement() {Limites_franchies = true; Sortie_Multiplexeur_4095(12);}
void Eteindre_debordement() {Limites_franchies = false; Forcer_a_zero_une_sortie_multiplexeur(12);}

void Faire_un_pas_sur_X() {// Ancien X incrémenté ou décrémenté de 5 car chaque pas fait 0,5mm.
  boolean Limites_OK = true;       
  if ((DX_Positif) && (Ancien_X >= 2280)) Limites_OK = false; // Ne pas dépasser le MAXI.
  if ((!DX_Positif) && (Ancien_X == 0)) Limites_OK = false;      // Ne pas buter vers zéro.
  if (Limites_OK) {Eteindre_debordement();
     if (!STOP) {
        Moduler_le_LASER();
        if (DX_Positif) {Ancien_X = Ancien_X + 5; digitalWrite(Sens_sur_X, HIGH);}
        else {Ancien_X = Ancien_X - 5; digitalWrite(Sens_sur_X, LOW);}
        for (byte I=0; I<20; I++)
            {digitalWrite(Un_pas_sur_X, HIGH); Pulse_moteur(1); digitalWrite(Un_pas_sur_X, LOW);}
        if (Pourcentage > 0) delay(Delai_par_pas_elementaire);
        Couper_le_LASER();}}
  else Allume_debordement();
  Traiter_CLAVIER_et_codeur_rotatif();}

void Faire_un_pas_sur_Y() {// Ancien Y incrémenté ou décrémenté de 5 car chaque pas fait 0,5mm.
  boolean Limites_OK = true;       
  if ((DY_Positif) && (Ancien_Y >= 2020)) Limites_OK = false; // Ne pas dépasser le MAXI.
  if ((!DY_Positif) && (Ancien_Y == 0)) Limites_OK = false;      // Ne pas buter vers zéro.
  if (Limites_OK) {Eteindre_debordement();
     if (!STOP) {
       Moduler_le_LASER();
       if (DY_Positif) {Ancien_Y = Ancien_Y + 5; digitalWrite(Sens_sur_Y, HIGH);}
       else {Ancien_Y = Ancien_Y - 5; digitalWrite(Sens_sur_Y, LOW);}
       for (byte I=0; I<20; I++)
           {digitalWrite(Un_pas_sur_Y, HIGH); Pulse_moteur(2); digitalWrite(Un_pas_sur_Y, LOW);}
       if (Pourcentage > 0) delay(Delai_par_pas_elementaire);
        Couper_le_LASER();}}
  else Allume_debordement();
  Traiter_CLAVIER_et_codeur_rotatif();}

void Effectuer_un_mouvement_vectoriel(int DeltaX, int DeltaY, boolean DX_sup_a_DY) {
  boolean Fini; int NbPX, NbPY; float Cumul, Decimale;
  int Compteur; int Rapport; // Impérativement un int car peut aller jusqu'à 2250.
  // Attention ci-dessous cumul est utilisé pour un calcul préalable du rapport décimal.
  if (DX_sup_a_DY) Cumul = float(DeltaX) / float(DeltaY); else Cumul = float(DeltaY) / float(DeltaX);
  Decimale = Cumul - ((int) Cumul); Rapport = Cumul - Decimale ;
  Cumul = 0.0; NbPX=0; NbPY=0; Fini = false;
  if (DX_sup_a_DY) {
      while (!Fini) {
        Compteur = 0;
        while (Compteur < Rapport) {
           if (NbPX < DeltaX) {Faire_un_pas_sur_X(); NbPX++; Compteur++; Cumul = Cumul + Decimale;}
           else Compteur = Rapport;} // Forcer la sortie si NbPX = DeltaX.
        Faire_un_pas_sur_Y(); NbPY++; 
        if (int(Cumul + 0.01) > 0) {// Donc Cumul vaut au moins 1.
        if (NbPX < DeltaX) 
           {Faire_un_pas_sur_X(); NbPX++; Cumul = abs(Cumul - 1); if (Cumul > 1) Cumul = abs(Cumul - 1);}}
        Fini = NbPX == DeltaX;}
      while (NbPY < DeltaY) {Faire_un_pas_sur_Y(); NbPY++;}}
  else {// Si DeltaY > DeltaX il faut inverses les rôles joués par X et par Y.
      while (!Fini) {
        Compteur = 0;
        while (Compteur < Rapport) {// Si DeltaX inférieur à DeltaY il faux échanger les rôles de X et de Y.
           if (NbPY < DeltaY) {Faire_un_pas_sur_Y(); NbPY++; Compteur++; Cumul = Cumul + Decimale;}
           else Compteur = Rapport;} // Forcer la sortie si NbPY = DeltaY.
        Faire_un_pas_sur_X(); NbPX++; 
        if (int(Cumul + 0.01) > 0) {// Donc Cumul vaut au moins 1.
        if (NbPY < DeltaY) 
           {Faire_un_pas_sur_Y(); NbPY++; Cumul = abs(Cumul - 1); if (Cumul > 1) Cumul = abs(Cumul - 1);}}
        Fini = NbPY == DeltaY;}
      while (NbPX < DeltaX) {Faire_un_pas_sur_X(); NbPX++;}}}

void Traiter_les_mouvements_du_LASER(int DeltaX, int DeltaY, boolean DX_sup_a_DY) {
  int NbPX = 0;   int NbPY = 0;
  if ((DeltaX > 0) || (DeltaY > 0)) {// Ne rien faire si les deux déplacements sont nuls.
     if (DeltaX == 0) while (NbPY < DeltaY) {Faire_un_pas_sur_Y(); NbPY++;}
     if (DeltaY == 0) while (NbPX < DeltaX) {Faire_un_pas_sur_X(); NbPX++;}
     if ((DeltaX != 0) && (DeltaY != 0)) Effectuer_un_mouvement_vectoriel(DeltaX, DeltaY, DX_sup_a_DY);}}

void Effectuer_un_deplacement_gco() {
  // Ancien_X et Ancien_Y sont mis à jour par les routines qui font faire un pas aux moteurs.
  boolean DX_sup_a_DY;
  int DeltaX, DeltaY; // Nombre de pas à effectuer sur les moteurs.
  int X,Y; // Les coordonnées à atteindre exprimées en dixième de mm :
  X = int(Position_X * 10);
  Y = int(Position_Y * 10); 
  DX_Positif = (X >= Ancien_X); DY_Positif = (Y >= Ancien_Y); // Détermine les sens de déplacement.
  if (DX_Positif) DeltaX = X - Ancien_X; else DeltaX = Ancien_X - X;
  if (DY_Positif) DeltaY = Y - Ancien_Y; else DeltaY = Ancien_Y - Y;
  // Ci-dessous division par 5 car un PAS moteur engendre un déplacement de 0.5mm
  DeltaX = DeltaX / 5;  DeltaY = DeltaY / 5; DX_sup_a_DY = (DeltaX >= DeltaY);
  Traiter_les_mouvements_du_LASER(DeltaX, DeltaY, DX_sup_a_DY);}   

//=================================== Procédures du Menu FICHIER ===================================

void Affiche_en_ligne_12(int PTR, byte LGR) {u8g.setPrintPos(0, 12); Aff_TEXTE_EEPROM(PTR,LGR);}
void Affiche_en_ligne_24(int PTR, byte LGR) {u8g.setPrintPos(0, 24); Aff_TEXTE_EEPROM(PTR,LGR);}
void Affiche_en_ligne_36(int PTR, byte LGR) {u8g.setPrintPos(0, 36); Aff_TEXTE_EEPROM(PTR,LGR);}
void Affiche_en_ligne_48(int PTR, byte LGR) {u8g.setPrintPos(0, 48); Aff_TEXTE_EEPROM(PTR,LGR);}

void Liste_contenu_Fichier() {
 u8g.firstPage(); do {
   u8g.setPrintPos(0, 12); u8g.print(Titre);
   Affiche_en_ligne_24(415,7); u8g.print(Taille); Aff_TEXTE_EEPROM(421,7);
   Affiche_en_ligne_36(186,12); u8g.print(Origine_Image_X); 
   Affiche_en_ligne_48(200,12); u8g.print(Origine_Image_Y);  
   u8g.setPrintPos(0, 60); 
   if (Temps_moyen) {Aff_TEXTE_EEPROM(552,12); u8g.print(int((Taille / Tmp_moyen_pour_une_ligne) / 60));}
   else {Aff_TEXTE_EEPROM(428,12); u8g.print(Taille / 1800);}
   Aff_TEXTE_EEPROM(439,4);} // Affiche " min"
   while(u8g.nextPage());}

void BIP_puis_attendre_un_BP() {BIP(); ATTENDRE_un_BP();}

void Affiche_ERR_et_attente_BP(int PTR, byte LGR) {Couper_le_LASER(); // Simple précaution.
  u8g.firstPage(); do {u8g.setPrintPos(5, 35); Aff_TEXTE_EEPROM(PTR,LGR);} while(u8g.nextPage());
  BIP_puis_attendre_un_BP();}

void Coupe_le_LASER_et_Pourcentage_a_0() {Couper_le_LASER(); Pourcentage = 0;}

void Probleme_de_lecture() {// Suggère de relire la carte après intervention.
  Coupe_le_LASER_et_Pourcentage_a_0();
  switch(Type_Erreur) {
    case 4 : {Affiche_ERR_et_attente_BP(11,21); break;} // Affiche "Pb sur le lecteur SD."
    case 5 : {Affiche_ERR_et_attente_BP(267,19); break;} // Affiche "Pb Capteur ORIGINE !"
    case 6 : {Affiche_ERR_et_attente_BP(113,17);}} // Affiche "Erreur en lecture". 
  u8g.firstPage(); // Affiche "Resoudre le probleme & Relire la carte SD."
  do {u8g.setPrintPos(0, 22); Aff_TEXTE_EEPROM(212,20);
      u8g.setPrintPos(0, 42); Aff_TEXTE_EEPROM(705,21);}
       while(u8g.nextPage()); ATTENDRE_un_BP(); Fin_demandee = true;}

void Ouvrir_le_fichier() {
  Nom_Fichier[3] = Indice[Num_Img_indexee]; // Construire le Nom du Fichier indexé.
  if (pf_open(Nom_Fichier)) {Type_Erreur = 6; Probleme_de_lecture(); }}

void Traite_X() {
  Decale_la_chaine(); Position_X = atof(Tampon);
  Item_suivant('S'); Pourcentage =  atoi(Tampon); if (Pourcentage > 0) SAV_Pourcentage = Pourcentage;}

void Prepare_degagement() {Position_X = 10.0, Position_Y = 192.0;}

void Item_suivant(char Caractere) {while (Tampon[0] != Caractere) Decale_la_chaine(); Decale_la_chaine();}

boolean Teste_presence_caractere(char Caractere_attendu) {
  boolean Existe = false; boolean FIN_trouvee = false;
  byte Index; Index = 0; 
  while ((!Existe) && (!FIN_trouvee)) {Existe = (Tampon[Index] == Caractere_attendu);
    FIN_trouvee = (Tampon[Index] == '\0'); Index++;}
  return Existe;}

void Decale_la_chaine() {for (byte I = 0; I < Taille_tampon; I++) Tampon[I] = Tampon[I+1];}

void Demande_FIN_de_procedure() {
  boolean SAV_Ecran_noir = Ecran_Noir;
  // Touche OUI cliquée : Couper les moteurs, éteindre le LASER et sortir du fichier.
  Affiche_ERR_et_attente_BP(64, 18); // Affiche le texte "Fin de procedure ?".
  Fin_demandee = (Num_BP == 4);
  if (Fin_demandee) {STOP = true; Mode_pas_a_pas = false; Pourcentage = 0;}
  else {Ecran_Noir = false; AFFICHE_etat_machine();
        Ecran_Noir = SAV_Ecran_noir;}
  if (!Mode_Verification_Programme) Eteindre_Ecran();}

void Teste_demande_sortie() {
  Teste_activation_BP_codeur_incremental();
  if (BP_central_a_traiter) {BP_central_a_traiter = false; Demande_FIN_de_procedure();}}

void PAUSE() {
  u8g.firstPage(); do {u8g.setPrintPos(0, 40); Aff_TEXTE_EEPROM(763,15); u8g.print(Num_Ligne);}
  while(u8g.nextPage()); BIP_puis_attendre_un_BP(); Eteindre_Ecran();}

void Traite_G() {
  boolean S_Present;
  boolean G1; // Sur code G0 ou G1 saisit la valeur de X dans tous les cas :
  Decale_la_chaine(); G1 = Tampon[0] == '1'; 
  Item_suivant('X'); Position_X = atof(Tampon);
  //------------------------------------ Traiter G1 ------------------------------------
  // G1 est de la forme : G1 Xn Sn ou G1 Xn Yn ou G1 Xn Yn Sn.
  if (G1) {S_Present = Teste_presence_caractere('S');
      if (Teste_presence_caractere('Y')) {Item_suivant('Y'); Position_Y = atoi(Tampon);}
      if (S_Present) {Item_suivant('S'); Pourcentage = atoi(Tampon);
                      if (Pourcentage > 0) SAV_Pourcentage = Pourcentage;}
      else Pourcentage = SAV_Pourcentage;} // Sur S1 non présent restituer le dernier S > 0.
  //------------------------------------ Traiter G0 ------------------------------------
  // Ci-dessous, on traite G0 qui est de la forme : G0 Xn Yn S0 ou G0 Xn Yn. 
  else {// Traite G0 qui par définition du gcode est un déplacement rapide LASER à 0%.
        Pourcentage = 0; Item_suivant('Y'); Position_Y = atof(Tampon);
        //----------------------- Traiter la ligne Origine Image ----------------------
        if (Ligne_Premier_G0) {// Seul le tout premier G0 mémorisera les coordonnées.
            Origine_Image_X = Position_X; Origine_Image_Y = Position_Y; Ligne_Premier_G0 = false;}
        // Traite la ligne G0 X0 Y0 finale en adoptant la configuration machine dégagée : 
        if ((Longueur_ligne_de_code == 8) && (Ligne_M5_trouvee)) Prepare_degagement();}} 
            // Prepare_degagement() : Impose les coordonnées Position_X = 10.0 et Position_Y = 192.0
            
void Traite_M() {
  // RAPPEL : M3 > LASER ON,   M5 > LASER OFF.
  // Codes de compléments : M0 > Eteindre l'écran    M1 > Allumer l'écran
  // M2 > PAUSE    M4 > valeur de Puissance_MAX_sur_le_LASER    M6 > valeur de Delai_par_pas_elementaire
  Coupe_le_LASER_et_Pourcentage_a_0(); // M3 et M5 doivent forcer le LASER à puissance nulle.
  // Également pertinent pour M0, M1, M2 et M6.
  if (Tampon[1] == '0') {if (!Mode_Verification_Programme) Ecran_Noir = true; Ligne_M0_trouvee = true;}
  if (Tampon[1] == '1') {if (!Mode_Verification_Programme) Ecran_Noir = false; Ligne_M1_trouvee = true;}
  if (Tampon[1] == '2') {Ligne_M2_trouvee = true; if(Pyrogravure_en_cours) PAUSE();}
  // M3 n'aura aucun effet car les moteurs sont déjà actvés si pyrogravure en cours.
  // N'est pas une ligne ignorée dans Traite_ligne_a_ligne_le_fichier pour économiser un test "inutile".
  if (Tampon[1] == '4') {Item_suivant(' '); Puissance_MAX_sur_le_LASER =  atoi(Tampon);}
  if (Tampon[1] == '5'); Ligne_M5_trouvee = true;
  // Important : Ci-dessus ne pas utiliser l'instruction Ligne_M5_trouvee = (Tampon[1] == '5')
  //             car les lignes de type M2 après M5 feraient perdre la valeur true.
  if (Tampon[1] == '6') {Item_suivant(' '); Delai_par_pas_elementaire =  atoi(Tampon);}} 
  
void Analyseur_Syntaxique() {
  Ligne_M0_trouvee = false; // À chaque ligne il faut tester la présence du code ETEINDRE L'ECRAN.
  Ligne_M1_trouvee = false; // À chaque ligne il faut tester la présence du code ALLUMER L'ECRAN.
  Ligne_M2_trouvee = false; // À chaque ligne il faut tester la présence d'une PAUSE.
  //------ Teste le BP central du codeur rotatif avant de traiter la ligne. -------
  Teste_demande_sortie(); // Ne sera pas demander si annulé durant un PAS.
  //------------------------------------------------------------------------------- 
  switch (Tampon[0]) {
     case 'G' : {Traite_G(); break;}
     case 'M' : {Traite_M(); break;}
     case 'S' : {Item_suivant('S'); Pourcentage = atoi(Tampon);
                 if (Pourcentage == 0) Couper_le_LASER(); else {SAV_Pourcentage = Pourcentage;} break;}
     case 'X' : {Traite_X(); break;} // Le break est impératif pour sauter "defalt" si OK.
     default  : Type_Erreur = 3;}
  //----------------------- Gère une coordonnée négative. -------------------------
  if ((Position_X < 0) || (Position_Y < 0)) Type_Erreur = 1;
  //-------------------- Gère une coordonnée > MAX possible -----------------------
  if ((Position_X > Course_MAX_sur_X) || (Position_Y > Course_MAX_sur_Y)) Type_Erreur = 2;
  //-------------------------------------------------------------------------------
  if (Type_Erreur == 0) {
      if (!Pyrogravure_en_cours) AFFICHE_etat_machine();
      // En vérification de programme la machine est inerte :
      Moduler_le_LASER(); // Seul la LED rouge sera allumée si mode vérification defichier.
      // L'appel à Moduler_le_LASER() se fait systématiquement pour traiter la LED rouge du répétiteur,
      // y compris en mode vérification de programme pour visualiser l'état sur la LED.
      if (!Mode_Verification_Programme) Effectuer_un_deplacement_gco();}}

void Traite_ligne_a_ligne_le_fichier() {
  boolean Ligne_a_ignorer;
  unsigned long pos = 0; // Position courante dans le fichier.
  uint8_t buf[Taille_tampon]; // Tampon de travail.
  Type_Erreur = 0; Num_Ligne = 1; Ligne_Premier_G0 = true; Ligne_M5_trouvee = false;
  Fin_demandee = false;
  //++++++++++++++++++++++++++++++ // Lecture ligne à ligne du fichier. +++++++++++++++++++++++++++++++++ 
  while ((Type_Erreur == 0) && (!Fin_demandee)) {// Sera réalisé tant que "break" non rencontré.
    UINT PLFO; // Pointeur de Lecture dans le Fichier Ouvert.
    // On lit le fichier octet par octet en mettant 1 en deuxième paramètre :
    if (pf_read(buf, 1, &PLFO)) Probleme_de_lecture();
    if (PLFO == 0) break; // Sortie si fin de fichier.
    char CARtre = (char)buf[0]; //Pour la lecture, on stocke dans la variable CARtre.
    //**************************** Traiter la ligne construite ***************************
    if(CARtre=='\n' ){
       //Il faut rajouter le caractère de fin " \0 " pour marquer la fin de chaîne.
       Tampon[pos] = '\0'; Longueur_ligne_de_code = --pos;
       pos = 0; //On réinitialise la position à 0 pour une nouvelle lecture : Recommencer à zéro.
       //-------------------------- Traiter Contenu_ligne[I] -------------------------
       for (byte I = 0; I < Taille_tampon; I++) Contenu_ligne[I] = String(' ');
       for (byte I = 0; I < Longueur_ligne_de_code; I++) Contenu_ligne[I] = String(Tampon[I]);
       //------------------------- Traiter la première ligne -------------------------
       if (Num_Ligne == 1) {// Ci-dessous : Extraire la première ligne dans TITRE.
           for (byte I = 0; I < Taille_tampon; I++) Titre[I] = Tampon[I];}
       //------------------------- Traiter la deuxième ligne -------------------------
       if (Num_Ligne == 2) {// Ci-dessous : Extraire le nombre de lignes.
              Decale_la_chaine(); Taille = atol(Tampon); Tampon[0] = ';';}
       //-------------- Sortie anticipée si informations images saisies --------------
       if ((Saisie_Informations_Fichier) && (!Ligne_Premier_G0)) break;
       //------------------------ Repérer les lignes à ignorer -----------------------
       Ligne_a_ignorer = false;
       if (Longueur_ligne_de_code == 0) Ligne_a_ignorer = true;
       if (Tampon[0] == ';') Ligne_a_ignorer = true; // Ignorer les remarques.
       if (Tampon[0] == 'F') Ligne_a_ignorer = true; // Ignorer "F1000".
       if ((Tampon[0] == 'G') && (Tampon[1] == '9')) Ligne_a_ignorer = true;
       //----------------------- Traiter les lignes pertinentes ----------------------
       Type_Erreur = 0; // Avant d'analyser la ligne il n'y a pas d'erreur.
       // Si "le contenu de la ligne" est incorrect l'analyseur syntaxique force Fin_demandee à true.
       if (!Ligne_a_ignorer) Analyseur_Syntaxique();
       if (Type_Erreur == 0) Num_Ligne++; // Ne pas incrémenter si une erreur a été détectée.
       //---------------- Afficher les paramètres actuels en pyrogravure -------------
       if (Pyrogravure_en_cours) Afficher_Ecran_PYROGRAVURE();
       //------------------ Traiter le B.P. central du codeur rotatif ----------------
       Teste_demande_sortie();} // Gère la demande de l'opérateur.
    //************************************************************************************   
    else {//Si ce n'est pas la fin de ligne, on continue à remplir le Tampon à chaque position :
          Tampon[pos] = (char)buf[0];  pos++;}}
   //++++++++++++++++++++++++++++ // Lecture ligne à ligne du fichier. +++++++++++++++++++++++++++++++++
   Couper_le_LASER();} // Impératif si une erreur se produit en analyse de fichier ou en pyrogravure.

void Deplacement_gco_puis_VEILLE() {Effectuer_un_deplacement_gco(); Passer_en_VEILLE();}

void Degager_le_plateau() {
      Couper_le_LASER(); Prepare_degagement(); STOP = false; Deplacement_gco_puis_VEILLE();}

void Traiter_PB_Syntaxe() {
  Ecran_Noir = false; // Nécessaire pour afficher l'état machine.
  if (Type_Erreur > 0) {// Ci-dessous : Degager_le_plateau() coupe aussi le LASER.
     if (Pyrogravure_en_cours) Degager_le_plateau();
     switch (Type_Erreur) {
       case 1 : {Affiche_ERR_et_attente_BP(286, 20); break;}
       case 2 : {Affiche_ERR_et_attente_BP(306, 20); break;}
       case 3 : Affiche_ERR_et_attente_BP(32,13);}
     AFFICHE_etat_machine(); ATTENDRE_un_BP();} // Affiche l'instruction qui a généré le problème.
  else if (Mode_Verification_Programme) {u8g.firstPage(); do { // Ci-dessous : Affiche syntaxe correcte.
           u8g.setPrintPos(0, 25); Aff_TEXTE_EEPROM(336,8); Aff_TEXTE_EEPROM(346,9);}
           while(u8g.nextPage()); BIP_puis_attendre_un_BP();}}

void VERIFIER_LE_FICHIER() { // Si PB d'ouverture incite à corriger puis retour au menu de BASE.
  if (Existe[Num_Img_indexee]) {
      Ouvrir_le_fichier();
      if (Type_Erreur != 6) {// Ne traiter que si ouverture correcte du fichier.
         Mode_Verification_Programme = true; Ecran_Noir = false; Num_BP = 0; 
         Saisie_Informations_Fichier = false; Traite_ligne_a_ligne_le_fichier();
         if (!Saisie_Informations_Fichier) Traiter_PB_Syntaxe();}} // Rend compte du résultat de l'analyse.
  else BIP();} // Faire un BIP si fichier non valide dans la lise.

void Initialiser_les_caracteristiques_du_fichier() {
  Saisie_Informations_Fichier = true; Mode_Verification_Programme = true;
  Ouvrir_le_fichier(); // Ne fait qu'ouvrir le fichier indexé et traite une éventuelle erreur de lecture.
  Traite_ligne_a_ligne_le_fichier();}

void Lire_et_afficher_les_donnees_fichier() {
  Initialiser_les_caracteristiques_du_fichier(); Liste_contenu_Fichier(); ATTENDRE_un_BP();
  Liste_contenu_SD();} // Ne fait qu'afficher le contenu du tableau Existe[].

void Bouge_en_manuel_et_gestion_LASER() {// Ci-dessous multiplier par 40.95 pour que 100% donne 4095 en PWM.
    if (LASER_actif) pwm.setPWM(15, 0, int(Puissance_MAX_sur_le_LASER * 40.95)); 
    Effectuer_un_deplacement_gco(); Couper_le_LASER();}

void Attente_BP_avec_LASER_en_pointage() {
  pwm.setPWM(15, 0, 41); // La valeur de 41 correspond à 1% pour l'amplitude MAX de 4048. 
  ATTENDRE_un_BP(); Couper_le_LASER();}

void Menu_Mode_MANUEL() { // L'économiseur d'écran n'est pas effectif pour cette option car peu utile.
  // En mode manuel c'est la puissance maximale définie dans les OPTIONS qui est appliquée au LASER.
  // >>>>>>> Attention : En manuel il n'y a aucune vérification sur les coordonnées. Arbitrairement
  // >>>>>>> elles sont forcé à zéro. Elles peuvent devenir négatives et la mécanique aller en butée.
  Fin_demandee = false; STOP = false; LASER_actif = false; Avance_par_pas = 10.0;
  Num_BP = 0; Capturer_Origine_machine(); // Laisse X = 0 et Y = 221.
  u8g.firstPage(); do {u8g.setPrintPos(0, 40); Aff_TEXTE_EEPROM(726,19);} while(u8g.nextPage());
  Activer_le_Relais_LASER();    // Ci-dessus : Demande de focaliser le LASER.
  Attente_BP_avec_LASER_en_pointage();
  while (!Fin_demandee) {
    Ecran_mode_MANUEL();
    Teste_SORTIR_du_MENU(); // Vérifie si le BP central est activé et si oui force Fin_demandee à true.
    //------------------------ Traiter la rotation du codeur incrémental ------------------------
    if (Rotation) {
        Rotation = false; // Prise en compte du traitement.
        if (LASER_actif) {
           if (Sens_positif) {Delai_par_pas_elementaire = Delai_par_pas_elementaire + 10;
                              if (Delai_par_pas_elementaire > 9990) Delai_par_pas_elementaire = 10;}
           else {if (Delai_par_pas_elementaire >= 20)
                     Delai_par_pas_elementaire = Delai_par_pas_elementaire - 10;
                     else Delai_par_pas_elementaire = 9990;}}
        else {if (Sens_positif)Avance_par_pas = 10.0; else Avance_par_pas = 0.5;}} 
    //-------------------------------------------------------------------------------------------
    Tester_le_Clavier();
    if (Num_BP !=0) {
        switch(Num_BP) {
          case 1 : {Position_X = (Position_X) - Avance_par_pas; Bouge_en_manuel_et_gestion_LASER(); break;}
          case 2 : {LASER_actif = !LASER_actif;
                    if (LASER_actif) Pourcentage = 100; // Pourcentage utilisé ici en local au case 2.
                    else Pourcentage = 0; pwm.setPWM(4, 0, Pourcentage); break;}    
          case 3 : {Position_X = (Position_X) + Avance_par_pas; Bouge_en_manuel_et_gestion_LASER(); break;}
          case 4 : {Position_Y = (Position_Y) + Avance_par_pas; Bouge_en_manuel_et_gestion_LASER(); break;}
          case 5 : {Position_Y = (Position_Y) - Avance_par_pas; Bouge_en_manuel_et_gestion_LASER(); break;}}
        if (Limites_franchies) BIP();
        Num_BP = 0;}} // Action sur le B.P. prise en compte.
  Coupe_le_LASER_et_Pourcentage_a_0; Couper_le_Relais_LASER();
  Capture_Origine_puis_Degage_le_plateau(); Passer_en_VEILLE();}

void Bilan_contenu_SD() { // Teste la présence carte et message d'erreur si non OK :
  Lecteur_OK = true; if (pf_mount(&fs)) Lecteur_OK = false; 
  if (!Lecteur_OK) // Teste la présence carte et message d'erreur sur boucle infinie si non OK :
     {Type_Erreur = 4; Probleme_de_lecture();}
  else // Construire la table de présence des dix fichiers possibles :
     for (byte I=0; I<10; I++) {Nom_Fichier[3] = Indice[I]; Existe[I] = !pf_open(Nom_Fichier);}}

void Relecture_carte_SD() {
  u8g.firstPage(); do {u8g.setPrintPos(4, 36); Aff_TEXTE_EEPROM(707, 19);} while(u8g.nextPage());
  Num_Img_indexee = 0; Bilan_contenu_SD();}

void Traite_ecran_noir() {if (!Ecran_Noir) Liste_contenu_SD(); else Eteindre_Ecran();}

void Traite_BP_Ymoins() {Mode_pas_a_pas = false; Lire_et_afficher_les_donnees_fichier();}

void Menu_FICHIER() {
  Ecran_Noir = false; Fin_demandee = false; Menu_en_cours = 2;
  Num_Img_indexee = 0; Liste_contenu_SD();  // Le curseur est positionné sur le fichier Img0. 
  while (!Fin_demandee) {
    Traite_ecran_noir(); // Affiche ou éteint en fonction du booléen Ecran_Noir.
    Teste_SORTIR_du_MENU(); // Vérifie si le BP central est activé et si oui force Fin_demandee à true.
    if (Rotation) TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL(); // Déplace l'index dans la liste des fichiers.
    Tester_le_Clavier();
    if (Num_BP != 0) {
        switch(Num_BP) {// Case 1 : Relecture de la carte SD.
          case 1 : {Relecture_carte_SD(); break;}
          case 2 : {Inverser_booleen_Ecran_Noir(); break;}     // Ci-dessous : 73 choisi arbitrairement.
          case 3 : {Mode_pas_a_pas = !Mode_pas_a_pas; Aff_Item_une_seconde(398, 15); break;}
          case 4 : {Aff_Item_une_seconde(368, 18); VERIFIER_LE_FICHIER(); Menu_en_cours = 2;
                    Fin_demandee = false; break;} // Pour ne pas sortir du menu FICHIER si sortie anticipée.
          case 5 : {Traite_BP_Ymoins(); Menu_en_cours = 2;}}
        Num_BP = 0;}} // Action sur le B.P. prise en compte.
  Origine_machine_effective = false;} // Rétablir la "réalité" car les moteurs sont en veille.    

void PYROGRAVER() {
  if (Existe[Num_Img_indexee]) {// Il ne faut capturer l'origine que si le fichier s'ouvre.
      Ouvrir_le_fichier(); Increment_Puissance = 1; // Variations faibles si capteur tourné par mégarde.
      Saisie_Informations_Fichier = false;
      Mode_pas_a_pas = false; Menu_en_cours = 8; STOP = false; // Pas de sortie demandée.
      //------------ Capturer l'Origine Machine puis déplacer sur l'Origine Image ------------
      Capturer_Origine_machine();  // Commence par activer les moteurs passe (et laisse) en "Rapide".
      Initialiser_les_caracteristiques_du_fichier();
      Temps_moyen = true; Liste_contenu_Fichier(); Temps_moyen = false;
      Position_X = Origine_Image_X; Position_Y = Origine_Image_Y; Effectuer_un_deplacement_gco();
      //---------------- Allumer le LASER à 1% et attendre un clic au clavier ----------------
      // Après le positionnement, puissance à 1% pour focaliser le LASER :
      Activer_le_Relais_LASER(); Attente_BP_avec_LASER_en_pointage();
      //--------------- Si touche autre que le B.P. moir débuter la PYROGRAVURE --------------
      if (Num_BP == 4) {
          Pyrogravure_en_cours = true;  Ecran_Noir = true;
          Mode_Verification_Programme = false; Saisie_Informations_Fichier = false;
          Ouvrir_le_fichier(); Traite_ligne_a_ligne_le_fichier();
          if (Type_Erreur > 0) Traiter_PB_Syntaxe();}
      //--------------------------------------------------------------------------------------
       Couper_le_Relais_LASER();
       Degager_le_plateau(); // On dégage le plateau également si sortie anticipée en gravure.
      Pyrogravure_en_cours = false; Passer_en_VEILLE();}
  Ecran_Noir = false; BIP(); // Faire un BIP si fichier non valide ou FIN de GRAVURE.
  Menu_en_cours = 1; Fin_demandee = false;}

void Teste_Sortir() {
  u8g.firstPage(); do {u8g.setPrintPos(40, 40); Aff_TEXTE_EEPROM(813,7);} while(u8g.nextPage());
  ATTENDRE_un_BP();}

void Menu_PYROGRAVER() {
  Ecran_Noir = false; Fin_demandee = false; Menu_en_cours = 1; Mdf_Pmax_en_gravant = false;
  Num_Img_indexee = 0; Liste_contenu_SD();  // Le curseur est positionné sur le fichier Img0. 
  while (!Fin_demandee) {
    Traite_ecran_noir(); // Affiche ou éteint en fonction du booléen Ecran_Noir.
    Teste_SORTIR_du_MENU(); // Vérifie si le BP central est activé et si oui force Fin_demandee à true.
    if (Rotation) {TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();} // Déplace l'index dans la liste des fichiers.
    Tester_le_Clavier();
    if (Num_BP !=0) {
        switch(Num_BP) {// Déplacer sur Origine image impose d'armer l'option en validant "Afficher_en_gravant".
          case 1 : {Relecture_carte_SD(); break;}
          case 2 : {Inverser_booleen_Ecran_Noir(); break;}
          case 3 : {Mdf_Pmax_en_gravant = !Mdf_Pmax_en_gravant; Pyrogravure_en_cours = true;
                    Aff_Item_une_seconde(496,11); Pyrogravure_en_cours = false; break;}
                    // Ci-dessus, Pyrogravure_en_cours sert à afficher correctement l'option dans
                    // la procédure Aff_Item_une_seconde(). (Texte "Mfd en PYRO."
          case 4 : {PYROGRAVER(); break;}
          case 5 : {Temps_moyen = true; Traite_BP_Ymoins(); Temps_moyen = false; Menu_en_cours = 2;}}
        Num_BP = 0;}}} // Action sur le B.P. prise en compte.  

void Menu_OPTIONS() { // L'économiseur d'écran n'est pas effectif pour cette option car peu utile.
  byte Increment, Increment_Delai;
  Fin_demandee = false; Curseur = 2; Increment = 1; Increment_Delai = 10;
  while (!Fin_demandee) {
    Liste_les_OPTIONS();
    Teste_SORTIR_du_MENU(); // Vérifie si le BP central est activé et si oui force Fin_demandee à true.
    //------------------------ Traiter la rotation du codeur incrémental ------------------------
    if (Rotation) {
        Rotation = false; // Prise en compte du traitement.
        switch(Curseur) {
          case 2 : {if (Sens_positif) {Puissance_MAX_sur_le_LASER = Puissance_MAX_sur_le_LASER + Increment;
                                      if (Puissance_MAX_sur_le_LASER > 100) Puissance_MAX_sur_le_LASER = 1;}
                    else {Puissance_MAX_sur_le_LASER = Puissance_MAX_sur_le_LASER - Increment;
                          if (Puissance_MAX_sur_le_LASER > 200) Puissance_MAX_sur_le_LASER = 100;} break;}
                          // Ci-dessus 200 car devenant "négatif" la valeur devient égale à 254 "binaire".
          case 4 : {if (Sens_positif) {
                        Delai_par_pas_elementaire = Delai_par_pas_elementaire + Increment_Delai;
                        if (Delai_par_pas_elementaire > 9000) Delai_par_pas_elementaire = 10;}
                    else {Delai_par_pas_elementaire = Delai_par_pas_elementaire - Increment_Delai;
                          if (Delai_par_pas_elementaire < 10) Delai_par_pas_elementaire = 9990;}}}}  
    //-------------------------------------------------------------------------------------------
    Tester_le_Clavier();
    if (Num_BP !=0) {
        switch(Num_BP) {
          case 1 : {Increment = 1; Increment_Delai = 10; break;}
          case 2 : {Puissance_MAX_sur_le_LASER = 50; Delai_par_pas_elementaire = 5000; break;}
          case 3 : {Increment = 10; Increment_Delai = 100; break;}
          case 4 : {Curseur = 2; break;}
          case 5 : {Curseur = 4;}
        Num_BP = 0;}}} // Action sur le B.P. prise en compte. 
    Curseur = 4;}

void Capture_Origine_puis_Degage_le_plateau() {// Activer les moteurs passe (et laisse) en "Rapide".
  Capturer_Origine_machine(); Degager_le_plateau();}

void Affiche_Increment_Puissance() {Aff_TEXTE_EEPROM(801,12); u8g.print(Increment_Puissance);}

void Affiche_en_14_sur_X(byte Y, int PTR, byte LGR) {u8g.setPrintPos(14, Y); Aff_TEXTE_EEPROM(PTR,LGR);}   

void Decaler_puis_LASER_permanent() {
  // ATTENTION : Quand la machine est décalée les moteurs sont passés en VEILLE.
  Increment_Puissance = 1; Puissance_MAX_sur_le_LASER = 0; LASER_permanent = true;
  Fin_demandee = false; STOP = false; Mode_Verification_Programme = false;
  Capturer_Origine_machine(); Position_X = 60; Effectuer_un_deplacement_gco();
  Position_Y = 142; Deplacement_gco_puis_VEILLE(); Activer_le_Relais_LASER();
  Pourcentage = 100; // Placé après la capture du centre plateau, car cette procédure force Pourcentage à zéro.
  while (!Fin_demandee) {
    //---------------- Afficher % LASER graphiquement et en numérique -------------
    if (Ecran_Noir) Eteindre_Ecran();
    else u8g.firstPage();
         do {Affiche_le_pourcentage(); Affiche_en_14_sur_X(24,479,6);
         u8g.print(Puissance_MAX_sur_le_LASER); Aff_TEXTE_EEPROM(487,9); u8g.setPrintPos(24, 45);
         Affiche_Increment_Puissance();}
    while(u8g.nextPage()); Pourcentage = 100;
    //-----------------------------------------------------------------------------
    Teste_SORTIR_du_MENU(); // Vérifie si le BP central est activé et si oui force Fin_demandee à true.
    //------------------------------- Traiter le CLAVIER --------------------------
    Tester_le_Clavier_pour_gerer_Increment();
    //------------------------ Traiter la rotation du codeur incrémental ------------------------
    // Effet de butées logicielles à 0% et à 100% sur l'ajustement de la puissance LASER.
    if (Rotation) {Rotation = false; Traiter_la_rotation_durant_la_gravure(); Moduler_le_LASER();}}
    //-------------------------------------------------------------------------------------------
  Couper_le_LASER();  Couper_le_Relais_LASER(); Pourcentage = 0;
  Capture_Origine_puis_Degage_le_plateau(); LASER_permanent = false; Ecran_Noir = false;}

void Inverser_booleen_Ecran_Noir() {Ecran_Noir = !Ecran_Noir;}

void Activation_item_menu_SYSTEME() {
  switch(Curseur) {
        case 2 : {Capture_Origine_puis_Degage_le_plateau(); break;}
        case 3 : {Decaler_puis_LASER_permanent(); break;}
        case 4 : {Mesurer_sur_les_capteurs_origine(); break;}
        case 5 : {Tester_les_capteurs_Origine(); Eteindre_les_trois_couleurs(); Allume_LED_Bleue();
                  if (Test_Capteurs_OK) Allume_LED_Verte(); else Allume_LED_Rouge();}}}

void Menu_SYSTEME() { // L'économiseur d'écran n'est pas effectif pour cette option car peu utile.
  Fin_demandee = false; Curseur = 2;  Menu_en_cours = 5; 
  while (!Fin_demandee) {
    if (!Ecran_Noir) Affiche_les_OPTIONS_SYTEME();
    //------------------------ Traiter le B.P. central du codeur rotatif ------------------------
    Teste_activation_BP_codeur_incremental();
    if (BP_central_a_traiter) {
        BP_central_a_traiter = false; Activation_item_menu_SYSTEME();
        Fin_demandee = true;} // Après chaque option on retourne au menu SYSTEME.
    //------------------------ Traiter la rotation du codeur incrémental ------------------------
    if (Rotation) {
        Rotation = false; // Prise en compte du traitement.
        if (!Sens_positif) {Curseur++; if (Curseur == 6) Curseur = 2;}
        else {if (Curseur > 2) Curseur--; else Curseur = 5;}}
    //--------------- Traiter le CLAVIER dont seul B.P. noir et X+ sont valides -----------------
    Tester_le_Clavier();
    if (Num_BP !=0) {
        switch(Num_BP) {
          case 2 : {Inverser_bascule_Ecran_Noir(); if (!Ecran_Noir) Retablir_Ecran_OLED(); break;}
          case 3 : {Activation_item_menu_SYSTEME(); break;}
          default : BIP();} Num_BP =0;}} // B.P. pris en compte.    
  Curseur = 5;} // En sortie pointer à nouveau le menu SYSTEME.

void Prevenir_que_clavier_sans_action_sauf_le_BP_NOIR() {
  Tester_le_Clavier();
  if (Num_BP !=0) {
     if (Num_BP == 2) {Inverser_bascule_Ecran_Noir(); if (!Ecran_Noir) Retablir_Ecran_OLED();}
     else BIP();} Num_BP =0;} // B.P. pris en compte.

//============================== Routines d'affichage sur l'écran OLED ==============================
void Aff_TEXTE_EEPROM(int PTR, byte Longueur) {// Routines pour les affichages textes EEPROM.
    for (byte I=0; I<Longueur; I++) {u8g.print(char(EEPROM.read(PTR))); PTR++;}; }

void Affiche_Deux_points_espace_OUI_NON(boolean OUI) {
  if (OUI) Aff_TEXTE_EEPROM(173,5); else Aff_TEXTE_EEPROM(179,5);}

void Espace_deuxpoints_Espace() {Aff_TEXTE_EEPROM(412,3);}

void AFF_s() {u8g.print('s');}

void AFF_zero() {u8g.print('0');}

void Eteindre_Ecran(){u8g.firstPage(); do {} while(u8g.nextPage());}

void Affiche_OUI() {Aff_TEXTE_EEPROM(3,4);}
void Affiche_NON() {Aff_TEXTE_EEPROM(7,4);}

void Aff_Item_une_seconde(int PTR, byte LGR) {
  u8g.firstPage(); do {u8g.setPrintPos(0, 30); Aff_TEXTE_EEPROM(PTR, LGR);
  if ((Num_BP == 3) && (Menu_en_cours != 5)) {
     if (Pyrogravure_en_cours) {u8g.print(' '); Affiche_Deux_points_espace_OUI_NON(Mdf_Pmax_en_gravant);}
     else Affiche_Deux_points_espace_OUI_NON(Mode_pas_a_pas);}}
  while(u8g.nextPage()); delay (1000); Eteindre_Ecran();}

void Retablir_Ecran_OLED() {
    switch (Menu_en_cours) {
    case 1 : {MENU_de_base(); break;}
    case 2 : {Liste_contenu_SD(); break;}
    case 6 : {Liste_contenu_SD(); break;}
    case 7 : {AFFICHE_etat_machine(); break;}
    case 8 : Afficher_Ecran_PYROGRAVURE();}}

void Inverser_bascule_Ecran_Noir() {// Bascule pour éteindre ou activer l'écran OLED.
  if (Num_BP == 2) {Inverser_booleen_Ecran_Noir();
      if (Ecran_Noir) Eteindre_Ecran(); else Retablir_Ecran_OLED();}}

void Positionne_le_curseur(){
  byte Y;
  Y = 12;  // Remarque : La cascade de if coute 6 octets de moins qu'un switch / case.
  if(Curseur == 2) Y = 24;
  if(Curseur == 3) Y = 36;
  if(Curseur == 4) Y = 48;
  if(Curseur == 5) Y = 60;
  u8g.setPrintPos(0, Y); u8g.print('>');}
    
void Debut_Nom(byte X, byte Y, byte N) {u8g.setPrintPos(X, Y); Aff_TEXTE_EEPROM(0,3); u8g.print(N);}

void Liste_contenu_SD() {// Ne teste plus la présence carte et liste son "contenu images" en aveugle
  byte X,Y;              // en utilisant le tableau de booléens Existe[10] initialisé sur RESET.
  // Affiche la liste de l'état des fichiers sur OLED et positionne l'index sur le fichier pointé :
  u8g.firstPage(); do {
     Y = 10; X = 0;
     for (byte I=0; I<10; I++) {
          if (I > 4) X = 66; if (I == 5) Y = 10;
          Debut_Nom(X,Y,I); if (Existe[I]) Affiche_OUI(); else Affiche_NON();  Y = Y + 13;
          if (I == Num_Img_indexee) Aff_TEXTE_EEPROM(50,2);}} 
  while(u8g.nextPage());}

void MENU_de_base(){
  Menu_en_cours = 0;
  if (!Ecran_Noir) {
      u8g.firstPage(); do {
      Affiche_en_14_sur_X(12,149,11); Affiche_en_14_sur_X(24,379,7); Affiche_en_14_sur_X(36,386,11); 
      Affiche_en_14_sur_X(48,443,7); Affiche_en_14_sur_X(60,450,7); Positionne_le_curseur();}
      while(u8g.nextPage());}}

void Affiche_Num_Ligne_sur_total() {
  Aff_TEXTE_EEPROM(45,6); u8g.print(Num_Ligne); Aff_TEXTE_EEPROM(678,3); u8g.print(Taille);}      

void Affiche_le_contenu_du_tampon() {for(byte I=0; I<Taille_tampon; I++) u8g.print(Contenu_ligne[I]);}

void AFFICHE_etat_machine(){
  boolean Afficher_contenu_ligne = true;
  if (!Ecran_Noir) {
      u8g.firstPage(); do {
      u8g.setPrintPos(0, 12); u8g.print(Titre); u8g.setPrintPos(0, 24); Affiche_Num_Ligne_sur_total();
      u8g.setPrintPos(0, 37); if (Ligne_M0_trouvee) {Aff_TEXTE_EEPROM(839,17); Afficher_contenu_ligne = false;}
                              if (Ligne_M1_trouvee) {Aff_TEXTE_EEPROM(783,16); Afficher_contenu_ligne = false;}
                              if (Ligne_M2_trouvee) {Aff_TEXTE_EEPROM(763,5); Afficher_contenu_ligne = false;}
                              if (Afficher_contenu_ligne) Affiche_le_contenu_du_tampon();
      u8g.setPrintPos(0, 50); Aff_TEXTE_EEPROM(52,4); u8g.print(Pourcentage); Aff_TEXTE_EEPROM(326,10);
      u8g.setPrintPos(0, 63); Aff_TEXTE_EEPROM(56,4); u8g.print(Position_X,1);
      u8g.setPrintPos(65, 63); Aff_TEXTE_EEPROM(60,4); u8g.print(Position_Y,1);}
      while(u8g.nextPage());}
  Menu_en_cours = 7;
  if ((Mode_pas_a_pas) && (!Ecran_Noir)) {
       ATTENDRE_un_BP(); if (Num_BP == 2) Inverser_bascule_Ecran_Noir();}
  Tester_le_Clavier(); if (Num_BP == 2) Inverser_bascule_Ecran_Noir();}

void AFF_valeur_delai_par_pas() {
  Affiche_en_ligne_48(82,16);
  u8g.print(float(Delai_par_pas_elementaire) / 1000,2); AFF_s();}

void Ecran_mode_MANUEL(){
  Menu_en_cours = 3;
  u8g.firstPage(); do {
  Affiche_en_ligne_12(286,10); Espace_deuxpoints_Espace(); u8g.print(Position_X,1);
  Affiche_en_ligne_24(614,10); Espace_deuxpoints_Espace(); u8g.print(Position_Y,1);
  Affiche_en_ligne_36(667,11); Espace_deuxpoints_Espace(); u8g.print(Avance_par_pas,1);
  Aff_TEXTE_EEPROM(100,2); AFF_valeur_delai_par_pas();
  u8g.setPrintPos(0, 60); Aff_TEXTE_EEPROM(479,5); Espace_deuxpoints_Espace();
  if (LASER_actif) {u8g.print(Puissance_MAX_sur_le_LASER); Aff_TEXTE_EEPROM(487,9); pwm.setPWM(4, 0, 4050);}
  else {Aff_TEXTE_EEPROM(181,3); Forcer_a_zero_une_sortie_multiplexeur(4);}}
  while(u8g.nextPage());}

void Affiche_le_pourcentage() {
  byte LGR_ruban;
  // Echelle graduée du haut :
  u8g.setPrintPos(0, 10); AFF_zero(); u8g.setPrintPos(111, 10); Aff_TEXTE_EEPROM(332,3);
  u8g.drawFrame(7,8,101,5);
  for (byte X=7; X<110; X=X+20) u8g.drawVLine(X,0,8);  // Tracer les grandes graduations.
  for (byte X=17; X<100; X=X+20) u8g.drawVLine(X,4,4); // Tracer les petites graduations.
  u8g.drawBox(8,9,Puissance_MAX_sur_le_LASER,3);}

void Trace_les_rubans() {// Placé en subroutine pour faciliter la lisibilité de Liste_les_OPTIONS().
  byte LGR_ruban;
  boolean Plage_une_seconde = (Delai_par_pas_elementaire < 1001);
  Affiche_le_pourcentage(); // Trace les graduations du haut et son ruban.
  // Echelle graduée du bas :
  u8g.setPrintPos(0, 61); AFF_zero(); u8g.setPrintPos(111, 61);
  if (Plage_une_seconde) Aff_TEXTE_EEPROM(98,2); else Aff_TEXTE_EEPROM(684,3); u8g.drawFrame(7,59,101,5);
  for (byte X=7; X<120; X=X+20) u8g.drawVLine(X,51,8);  // Tracer les grandes graduations.
  for (byte X=17; X<110; X=X+20) u8g.drawVLine(X,55,4); // Tracer les petites graduations.
  if (Plage_une_seconde) LGR_ruban = byte(Delai_par_pas_elementaire / 10);
  else LGR_ruban = byte(Delai_par_pas_elementaire / 100);
  u8g.drawBox(8,60,LGR_ruban,3);} // Tracé du ruban.

void Liste_les_OPTIONS() { // L'économiseur d'écran n'est pas effectif pour cette option car peu utile.
  u8g.firstPage(); do {
  Affiche_en_14_sur_X(24,479,6); u8g.print(Puissance_MAX_sur_le_LASER);
  Aff_TEXTE_EEPROM(678,6); Affiche_en_14_sur_X(48,82,5);
  Espace_deuxpoints_Espace(); u8g.print(float(Delai_par_pas_elementaire) / 1000,2); AFF_s();
  Trace_les_rubans(); Positionne_le_curseur();}
  while(u8g.nextPage());}

void Affiche_les_OPTIONS_SYTEME() { // L'économiseur d'écran n'est pas utile pour cette option.
  u8g.firstPage(); do {
    Affiche_en_ligne_12(1004,20);    
    Affiche_en_14_sur_X(24,820, 18);
    Affiche_en_14_sur_X(36,745,18); Affiche_en_14_sur_X(48,587,17);
    Affiche_en_14_sur_X(60,570,17); Positionne_le_curseur();}
  while(u8g.nextPage());}

void Afficher_Ecran_PYROGRAVURE() {
  if (!Ecran_Noir) {
      u8g.firstPage(); do {
        u8g.setPrintPos(5, 10); Aff_TEXTE_EEPROM(149,20); u8g.drawFrame(0,0,128,12);
        u8g.setPrintPos(4, 24); Affiche_Num_Ligne_sur_total(); u8g.setPrintPos(4, 36);
        Aff_TEXTE_EEPROM(471,16); u8g.print(Puissance_MAX_sur_le_LASER); AFF_valeur_delai_par_pas();
        u8g.setPrintPos(4, 60); Affiche_Increment_Puissance();}
      while(u8g.nextPage()); ATTENDRE_un_BP(); if (Num_BP == 2) Ecran_Noir = true;}
  else Eteindre_Ecran();}

