/* Version dérivée de P20 avec possibilité de piloter la machine à partir du Moniteur de l'IDE.
   C'est le programme parfaitement adapté pour tester entièrement la machine en phase de
   développement et de mise au point. C'est également un programme très convivial pour
   "apprendrer" le gcode ou créer des programmes "image" personnels tels que ceux proposés
   dans le didacticiel.
   
   >>>>>>>>>> Version du 7 Juillet 2020 >>>>>>>>>> Donnée inscrite en EEPROM.
   
   UTILISATION DU PROGRAMME :   >>>>> Imposer 115200bds au Moniteur de l'IDE.
   ==========================   >>>>> Imposer "Retour chariot" au Moniteur pour générer les CR.
   En saisie des consignes USB valider engendre la validation de la ligne USB.
   Immédiatement après avoir reçu une consigne, le programme la retourne vers le moniteur pour avoir à l'écran
   de l'ordinateur l'historique des manipulations effectées.
   Puis le contenu de la consigne est soumis à l'ancien Traite_ligne_a_ligne_le_fichier() qui a été
   renommé Traite_la_ligne_USB et qui effectue le traitement du programme d'exploitation,
   sauf que la ligne étant traitée il y a retour de procédure.
   On peut ainsi frapper du code manuellement avec le Moniteur et observer ce qu'en fait le programme.
   Naturellement il faut respecter la syntaxe définie sur la fiche "Analyseur Syntaxique. 1/3"
   Pour revenir au menu de base donner M5 suivi de G0 X0 Y0 en  consignes comme le fait un fichier gco.
   Il sera commode de verrouiller [MAJ].
   >>> Le menu FICHIER non pertinent génère un BIP d'erreur. (Les trois derniers Items restent effectifs.)
   En particulier le menu SYSTEME permet d'ajuster l'optimisation des capteurs origine, et le menu
   Mode MANUEL facilite la détermination des limites mécanique machine.
                     
 Branchements à effectuer :
 --------------------------
 Multiplexeur  S0 : Branchement de la LED Verte du CLAVIER.
 Multiplexeur  S1 : Vers LED verte pulses moteur X.
 Multiplexeur  S2 : Vers LED bleue pulses moteur Y.
 Multiplexeur  S3 : Branchement de la LED "triple" Bleue du mode VEILLE sur les moteurs.
 Multiplexeur  S4 : Branchement de la LED "Répériteur LASER".
 Multiplexeur  S6 : LED "triple" Rouge Origine machine non initialisée.
 Multiplexeur  S9 : LED "triple" Verte : Origine machine capturée.
 Multiplexeur S12 : Vers LED rouge de Débordement limites Machine.
 Multiplexeur S13 : Pilotage du Relais d'alimentation du LASER.
 Multiplexeur S15 : Sortie de pilotage PWM LASER.
                                                                PGM                 RAM dynamique
P20 avant modifications :                                 29674 Octets (96%)       867 Octets (42%)
Ajout de Serial.begin(115200);                            30690 Octets (99%)       1040 Octets (50%)
Enlever du code inutile et ajouter le dialogue USB :      25740 Octets (83%)       983 Octets (47%)
Ajouté la gestion du relais du LASER :                    25668 Octets (83%)       983 Octets (47%) */

//--------------------- Déclarations pour le matériel -----------------------
#include <Wire.h> // Gère les protocole I2C sur A4 et A5 pour le Multiplexeur.
#include <EEPROM.h>
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)
#include "PetitFS.h"
FATFS fs;     // Zone de travail pour le volume SD : Objet système.
#include <Adafruit_PWMServoDriver.h> // déclaration du Multiplexeur.
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

//------------- Définition des broches d'entrées analogiques ----------------
#define Ligne_trois_touches 14 // A0 pour lire le clavier à 3 BP. 
#define Ligne_deux_touches 15 // A1 pour lire le clavier à 2 BP. 
#define BP_central 16 // A2 pour lire le bouton central.
#define Pilotage_capteurs_Origines 17 // A3 active les modules uniquement aux moments pertinents.
#define Capteur_Origine_X  20 // A6 pour lire le capteur sur butée X.
#define Capteur_Origine_Y  21 // A7 pour lire le capteur sur butée X.
//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_B 2
#define Codeur_A 3
#define BUZZER 4
#define Moteurs_actifs 5 // Broche qui à l'état haut fait passer le moteur en mode "non alimenté".
#define Sens_sur_X 6 // Pilotage du sens de rotation sur X.
#define Un_pas_sur_X 7 // Pilotage "Faire un pas" sur X.
#define Sens_sur_Y 8 // Pilotage du sens de rotation sur Y.
#define Un_pas_sur_Y 9 // Pilotage "Faire un pas" sur Y.
#define chipSelect 10
//------------------- Déclaration des constante du programme ----------------
#define Seuil_Origine_X 900 // Tension moyenne entre Actif et neutre sur X. @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Seuil_Origine_Y 553 // Tension moyenne entre Actif et neutre sur Y. @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Course_MAX_sur_X 228.0 // Déplacement maximal possible sur la machine @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Course_MAX_sur_Y 202.0 // Déplacement maximal possible sur la machine @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

// ATTENTION : Si Course_MAX_sur_Y modifiée reprendre les deux valeurs dans void Capturer_Origine_machine.
// ATTENTION : Si Course_MAX_sur_X ou sur Course_MAX_sur_Y modifiée reprendre les valeurs dans la
//             procédure de centrage void Positionner_LASER_sur_Origine_Image.

#define Puissance_MAX_LASER_sur_RESET 70 // Valeur à affiner par experience. @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define NB_tests_antirebonds 30 // Nombre de tests de stabilisation B.P.
#define Taille_tampon 28 // Taille des tampons pour les chaînes de caractères.

#define LGR_chaine 30 // Longueur maximale de la chaine de dialogue. (21 maximum en théorie.)
#define CR 13 // Code du Retour Chariot qui vaut 13 codé en ASCII sous forme décimale.

//----------------------- Déclaration des variables -------------------------
boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean Afficher_en_gravant;
boolean BP_central_a_traiter;
boolean Demande_FIN_active; // Bascule pour ne pas avoir un doublon si fin demandée lors d'un PAS moteur.
boolean DX_Positif, DY_Positif; //, DX_sup_a_DY;
boolean Ecran_Noir;
boolean Fin_demandee;
boolean FIN_du_dialogue_USB;
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean LASER_actif;
boolean Lecteur_OK;
boolean Ligne_a_ignorer;
boolean Ligne_M5_trouvee;
boolean Ligne_Premier_G0; // Permet de mémoriser les coordonnées Image.
boolean Limites_franchie;
boolean Mode_pas_a_pas;
boolean Mode_VEILLE;
boolean Mode_Verification_Programme;
boolean Origine_machine_effective;
boolean OrigineX_effective, OrigineY_effective;
boolean Recopie_Repetiteur_LASER;
boolean Saisie_Informations_Fichier;
boolean Sens_positif; // Sens de rotation.
boolean STOP; // Permet de sortir d'une gravure si fin demandée.
boolean Test_Capteurs_OK;

volatile boolean Rotation;  // Indique une activation du codeur rotatif.

boolean Existe[10];
char Caractere;
char Indice[10] = {'0','1','2','3','4','5','6','7','8','9'};
char Nom_Fichier[9] = "IMGN.GCO";
char Tampon[Taille_tampon];
char Titre[Taille_tampon];

byte Curseur; // Définit la ligne de texte écran où doit se trouver le curseur.
byte Longueur_ligne_de_code; 
byte Menu_en_cours; // Page-écran à rétablir suite à "Ecran NOIR".
byte Num_BP; // Désigne le bouton poussoir activé sur le clavier.
byte Num_Img_indexee;
byte Pourcentage; // Consigne de Pourcentage de puissance du LASER.
byte Puissance_MAX_sur_le_LASER; // Puissance qui sera appliquée quand Pourcentage = 100;
byte SAV_Num_BP; // Mémorisation de Num_BP sur la ligne 3BP pour le "test système".
byte Type_Erreur;

int Ancien_X, Ancien_Y; // Positions exprimées en dixièmes de mm.
int Delai_par_pas_elementaire;
int U_sur_X, U_sur_Y; // tensions sur les capteurs d'origine machine.

String Chaine_Memorisee; // Tampon mémoire pour la chaine entrante.

unsigned long Num_Ligne, Taille;

float Position_X, Position_Y, Origine_Image_X, Origine_Image_Y, Avance_par_pas;
//======================================================================================

void setup() {Serial.begin(115200);
  Chaine_Memorisee.reserve(LGR_chaine); // Reserve LGR_chaine octets pour la chaine.
  //---------------------- Définition des vecteurs d'interruption ----------------------
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  // Broche de l'interruption 1 (E/S D3) :
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pwm.begin(); // Initialise le module multiplexeur.
  pwm.setPWMFreq(1000); // Fréquence particulière 1000Hz pour piloter le module LASER.
  Couper_le_Relais_LASER();
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(chipSelect, OUTPUT); digitalWrite(chipSelect,HIGH);
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW);
  pinMode(Pilotage_capteurs_Origines,OUTPUT); digitalWrite(Pilotage_capteurs_Origines, LOW);
  pinMode(Moteurs_actifs, OUTPUT); Passer_en_VEILLE(); // Passe les moteurs en veille et allume la LED bleue.
  pinMode(Sens_sur_X, OUTPUT); pinMode(Un_pas_sur_X, OUTPUT); digitalWrite(Un_pas_sur_X, LOW);
  pinMode(Sens_sur_Y, OUTPUT); pinMode(Un_pas_sur_Y, OUTPUT); digitalWrite(Un_pas_sur_Y, LOW);
  pinMode(11,INPUT); pinMode(12,INPUT); pinMode(13,INPUT);
  digitalWrite(BP_central, HIGH); // Valide R PUL-UP sur A2.
  Front_sur_A(); Front_sur_B(); // Routines d'IRQ pour le codeur rotatif.
  //---------------------- Initialisation des diverses variables -----------------------
  Num_Img_indexee = 0; BP_central_a_traiter = false; Mode_pas_a_pas = false;
  Curseur = 2; // Valeur 2 pour pointer l'item Moniteur USB car sur RESET une rotation "négative" est détectée.
  Afficher_en_gravant = false; Saisie_Informations_Fichier = false; Origine_machine_effective = false;
  Delai_par_pas_elementaire = 10; // Mouvements manuels rapides par défaut.
  Puissance_MAX_sur_le_LASER = Puissance_MAX_LASER_sur_RESET;
  Recopie_Repetiteur_LASER = true; Limites_franchie = false;
  //--------------------------------- Actions initiales /-------------------------------
  Traiter_la_LED_triple(); Eteindre_LED_clavier();}

void loop() { // Dans la boucle de base le clavier ne gère que l'économiseur d'écran.
  MENU_de_base(); // L'écran est rafraichi en permanence mais ce n'est pas pénalisant.
  //--------------------------------- Traite le B.P. central ---------------------------
  Teste_activation_BP_codeur_incremental(); // Si B.P. activé saisir les caractéristiques de l'image.
  if (BP_central_a_traiter) { // Vectorisation par le MENU de BASE.
      BP_central_a_traiter = false;
      switch(Curseur) {Curseur = 2; 
        case 1 : {Menu_en_cours = 1; Dialogue_USB(); break;}
        case 2 : {Menu_en_cours = 2 ; Menu_FICHIER(); break;}
        case 3 : {Menu_Mode_MANUEL(); break;}
        case 4 : {Menu_OPTIONS(); break;}
        case 5 : Menu_SYSTEME();}}
  //--------------------- Traite la rotation du codeur incrémental ---------------------
  if (Rotation) TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();
  //---------------------- Traiter la requête de l'un des cinq B.P. --------------------
  Tester_le_Clavier();
  if (Num_BP !=0) {ATTENDRE_le_relacher();
                   if (Num_BP == 2) {Inverser_bascule_Ecran_Noir(); if (!Ecran_Noir) Retablir_Ecran_OLED();}
                   else BIP();}}

//====================================== Routines de servitude =====================================
void Front_sur_A(){ // Interruption quand A change d'état avant B.
  if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
  if (digitalRead(Codeur_A) != A_set) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();}}}
  
void Front_sur_B(){ // Interruption quand B change d'état avant A.
  if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
  if (digitalRead(Codeur_B) != B_set) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();}}}

void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 100 expérimentale.
void Delay_1mS() {delay(1);}
void Delay_30mS() {delay(30);}
void Delay_100mS() {delay(100);}

void BIP() {digitalWrite(BUZZER, HIGH); Delay_100mS(); digitalWrite(BUZZER, LOW);}

void Teste_activation_BP_codeur_incremental() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     //----------- Attendre le relacher du B.P. central. ----------
     Allume_LED_clavier();
     while (analogRead(BP_central) < 512); // Boucler sans rien faire pour attendre le relâché.
     Eteindre_LED_clavier();
     //------------------------------------------------------------
     BP_central_a_traiter = true;}}

void Tester_le_Clavier() {// Les valeurs tiennent compte de VCC = 4V car fourni par la carte NANO.
  Num_BP = 0; // Signale aucun B.P. utilisé.
  U_sur_X = analogRead(Ligne_trois_touches);
  if (U_sur_X < 758) {Num_BP = 1;
      if (U_sur_X < 508)  Num_BP = 2; if (U_sur_X < 230)  Num_BP = 3;}
  SAV_Num_BP = Num_BP; 
  U_sur_Y = analogRead(Ligne_deux_touches);
  if (U_sur_Y < 709) {Num_BP = 5; if (U_sur_Y < 300)  Num_BP = 4;}}

void Allume_LED_clavier() {pwm.setPWM(0, 0, 4095);}
void Eteindre_LED_clavier() {pwm.setPWM(0, 0, 0);}

void ATTENDRE_le_relacher() {
  byte SAV_Num_BP = Num_BP;
  byte COMPTEUR = 0; Delay_30mS(); // Attendre_la_stabilisation du travail.
  Allume_LED_clavier();
  while (COMPTEUR < NB_tests_antirebonds) {// Attendre_la_stabilisation du repos :
         COMPTEUR++; Tester_le_Clavier(); if (Num_BP > 0) COMPTEUR = 0;}
  Eteindre_LED_clavier(); Num_BP = SAV_Num_BP;}

void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  byte Mini, Maxi;
  INT_active = true; // Armer un nouveau comptage.
  Rotation = false; // Prise en compte du traitement.
  //-------------------- Ajuster la valeur du curseur -------------------
  Mini = 1; Maxi = 5;
  if (Sens_positif) {if (Curseur > 1) Curseur--; else Curseur = Maxi;}
  else {Curseur++; if (Curseur == (Maxi + 1)) Curseur = Mini;}}

void Teste_SORTIR_du_MENU() {
  Teste_activation_BP_codeur_incremental(); // Si B.P. activé saisir les caractéristiques de l'image.
  if (BP_central_a_traiter) {Fin_demandee = true; BP_central_a_traiter = false;}}
 
void ATTENDRE_un_BP() {
  boolean Allumer;
  Num_BP = 0;
  while (Num_BP == 0) {Tester_le_Clavier(); Delay_100mS();
         Allumer =!Allumer; if (Allumer) Allume_LED_clavier(); else Eteindre_LED_clavier();}
  ATTENDRE_le_relacher();}

//====================== Routines pour traiter les capteurs d'origines machine =====================
void Mesurer_U_Capteur(boolean Sur_X) {
  #define Nb_Iterations 20 // Mesure avec Nb itérations pour l'intégration "d'antiparasitage".
  // La valeur de 20 est choisie expérimentalement et semble un bon compromis entre des
  // mesures fiables et une rapidité suffisante de rotation des moteurs.
  long Valeurs_lues;
  byte Compteur = Nb_Iterations;
  Valeurs_lues = 0;
  for (int I = 0; I < Nb_Iterations; I++) // Cumuler N mesures. 
      {if (Sur_X) Valeurs_lues = Valeurs_lues + analogRead(Capteur_Origine_X);
       else Valeurs_lues = Valeurs_lues + analogRead(Capteur_Origine_Y);}
  if (Sur_X) U_sur_X = int(Valeurs_lues / Nb_Iterations); else U_sur_Y = int(Valeurs_lues / Nb_Iterations);}
  
void Traiter_Origines_sans_allumer() {
  Mesurer_U_Capteur(true); // Donc mesure avec anti-rebonds sur le capteur X.
  Mesurer_U_Capteur(false); // Donc mesure avec anti-rebonds sur le capteur Y.
  OrigineX_effective = (U_sur_X > Seuil_Origine_X); OrigineY_effective = (U_sur_Y > Seuil_Origine_Y); 
  digitalWrite(Pilotage_capteurs_Origines, LOW);}

void Mesurer_U_sur_capteurs_Origines() {
  digitalWrite(Pilotage_capteurs_Origines, HIGH); Delay_1mS(); Traiter_Origines_sans_allumer();}

void Mesurer_sur_les_capteurs_origine() {// Clavier et codeur rotatifs ignorés.
  while (!Fin_demandee) {
    Teste_SORTIR_du_MENU();
    Mesurer_U_sur_capteurs_Origines();
    u8g.firstPage(); do {
    u8g.setPrintPos(15, 15); Aff_TEXTE_EEPROM(184,14); u8g.print(U_sur_X);
    u8g.setPrintPos(15, 30); Aff_TEXTE_EEPROM(198,14); u8g.print(U_sur_Y);
    u8g.setPrintPos(15, 45); Aff_TEXTE_EEPROM(186,10);
    Affiche_Deux_points_espace_OUI_NON(OrigineX_effective);
    u8g.setPrintPos(15, 60); Aff_TEXTE_EEPROM(200,10); 
    Affiche_Deux_points_espace_OUI_NON(OrigineY_effective);}
  while(u8g.nextPage());}}

void Tester_les_capteurs_Origine() {
  Traiter_Origines_sans_allumer(); // Ci-dessous les deux doivent avoir "Origine_effective".
  Test_Capteurs_OK = ((OrigineX_effective) && (OrigineY_effective));
  Mesurer_U_sur_capteurs_Origines(); // Ci-dessous les deux doivent avoir NON "Origine_effective".
  if ((OrigineX_effective) || (OrigineY_effective)) Test_Capteurs_OK = false;
  digitalWrite(Pilotage_capteurs_Origines, LOW);}  

void Mesurer_sur_les_BP_CLAVIER() {// Clavier et codeur rotatifs ignorés.
  while (!Fin_demandee) {
    Teste_SORTIR_du_MENU();
    Tester_le_Clavier();
    u8g.firstPage(); do {
    u8g.setPrintPos(0, 15); Aff_TEXTE_EEPROM(624,17); u8g.print(U_sur_X);
    u8g.setPrintPos(0, 30); Aff_TEXTE_EEPROM(657,10); u8g.print(SAV_Num_BP);
    u8g.setPrintPos(0, 45); Aff_TEXTE_EEPROM(641,16); u8g.print(U_sur_Y);
    u8g.setPrintPos(0, 60); Aff_TEXTE_EEPROM(657,10); 
    if (SAV_Num_BP == 0) u8g.print(Num_BP); else AFF_zero();}
    while(u8g.nextPage());}}
    
//==================================== Procédures pour le LASER ====================================
void Sortie_Multiplexeur_4095(byte Sortie) {pwm.setPWM(Sortie, 0, 4095);}
void Forcer_a_zero_une_sortie_multiplexeur(byte Sortie) {pwm.setPWM(Sortie, 0, 0);}

void Activer_le_Relais_LASER() {Sortie_Multiplexeur_4095(13); Delay_100mS();}

void Couper_le_Relais_LASER() {Forcer_a_zero_une_sortie_multiplexeur(13);}

void Moduler_le_LASER() {// Place sur S15 la valeur correspondant à Pourcentage.
  int Pulse;
  if ((!Mode_Verification_Programme) && (LASER_actif)){
     // Moduler uniquement si GRAVURE et non en vérification de fichier. 
     Pulse = map(Pourcentage, 0, 100, 0, (Puissance_MAX_sur_le_LASER * 40.95)); pwm.setPWM(15, 0, Pulse);}
     // Ci-dessus : 40.95 pour que multiplié par 100 on obtienne 4095 les 100% de rapport cyclique.
  Pulse = map(Pourcentage, 0, 100, 0, 4095); pwm.setPWM(12, 0, Pulse); // Allume le répétiteur LED.
  if ((Recopie_Repetiteur_LASER) && (!Limites_franchie)) pwm.setPWM(4, 0, Pulse);}

void Couper_le_LASER() {pwm.setPWM(15, 0, 0); pwm.setPWM(12, 0, 0);
  if ((Recopie_Repetiteur_LASER) && (!Limites_franchie)) pwm.setPWM(4, 0, 0);}

//================================= Procédures pour la motorisation ================================

void Eteindre_les_trois_couleurs() {pwm.setPWM(3, 0, 0);  pwm.setPWM(6, 0, 0);  pwm.setPWM(9, 0, 0);}

void Allume_LED_Rouge() {pwm.setPWM(6, 0, 4095);}
void Allume_LED_verte() {pwm.setPWM(9, 0, 4095);}
void Allume_LED_Bleue() {pwm.setPWM(3, 0, 4095);}
  
void Traiter_la_LED_triple() {
   Eteindre_les_trois_couleurs();
   if (!Mode_Verification_Programme) {
       if (Mode_VEILLE) Allume_LED_Bleue(); // La LED bleue du mode Veille est prioritaire.
       else {if (Origine_machine_effective) Allume_LED_verte();
             else Allume_LED_Rouge();}}}
   
void Activer_les_moteurs() { // Ci-dessous delai 30mS pour stabilisation des moteurs.
  digitalWrite(Moteurs_actifs, LOW);  Mode_VEILLE = false; Traiter_la_LED_triple(); Delay_30mS();}

void Passer_en_VEILLE() { digitalWrite(Moteurs_actifs, HIGH);
  Mode_VEILLE = true; Origine_machine_effective = false; Traiter_la_LED_triple();}

void Capturer_Origine_machine() {// Déplacer à gauche et vers l'arrière de la machine.
   Ancien_Y = 0; Ancien_X = 2500; // Valeurs arbitraires mais > aux limites machine.
  // Tourner dans le sens "négatif" pour X et positif pour Y :
  STOP = false; Pourcentage = 0; Activer_les_moteurs(); DX_Positif = false; DY_Positif = true;
  Aff_Item_une_seconde(248, 19); // Affiche "> Capture ORIGINE <
  // Vérifier OrigineX_effective et OrigineY_effective avant d'effectuer les mouvements idoines :
  Mesurer_U_sur_capteurs_Origines(); // Positionne OrigineX_effective et OrigineY_effective actuelles.
  while (!OrigineX_effective) {Faire_un_pas_sur_X(); Mesurer_U_sur_capteurs_Origines();}
  while (!OrigineY_effective) {Faire_un_pas_sur_Y(); Mesurer_U_sur_capteurs_Origines();}
  Origine_machine_effective = true; // Car le programme arrive ici que si les deux sont capturées.
  Traiter_la_LED_triple(); Ancien_X = 0; Ancien_Y = 2020; // Attentions : Ancien_X,Y en dixièmes de mm.
  Position_X = 0; Position_Y = 202.0;}

void Positionner_LASER_sur_Origine_Image(boolean Centrage_sur_le_plateau) {
  if (Existe[Num_Img_indexee]) {STOP = false; Passer_en_VEILLE();
      if (Centrage_sur_le_plateau) {Origine_Image_X = 114.0; Origine_Image_Y = 101.0;}
      Capturer_Origine_machine();  // Commence par activer les moteurs passe (et laisse) en "Rapide".
      Position_X = Origine_Image_X; Position_Y = Origine_Image_Y;
      Effectuer_un_deplacement_gco();}} 

void Allumer_LED_debordement() {pwm.setPWM(4, 0, 4095);}

void Eteindre_LED_debordement() {pwm.setPWM(4, 0, 0);}

void Faire_un_pas_sur_X() {// Ancien X incrémenté ou décrémenté de 5 car chaque pas fait 0,5mm.
  boolean Limites_OK = true;       
  if ((DX_Positif) && (Ancien_X >= 2280)) Limites_OK = false; // Ne pas dépasser le MAXI.
  if ((!DX_Positif) && (Ancien_X == 0)) Limites_OK = false;      // Ne pas buter vers zéro.
  if (Limites_OK) {Eteindre_LED_debordement(); Limites_franchie = false;
     if (!STOP) {
        Moduler_le_LASER();
        if (DX_Positif) {Ancien_X = Ancien_X + 5; digitalWrite(Sens_sur_X, HIGH);}
        else {Ancien_X = Ancien_X - 5; digitalWrite(Sens_sur_X, LOW);}
        for (byte I=0; I<20; I++)
             {digitalWrite(Un_pas_sur_X, HIGH); Delay_1mS(); digitalWrite(Un_pas_sur_X, LOW);}
        if (Pourcentage > 0) delay(Delai_par_pas_elementaire);
        Couper_le_LASER();}}
  else {Allumer_LED_debordement(); Limites_franchie = true;}}

void Faire_un_pas_sur_Y() {// Ancien Y incrémenté ou décrémenté de 5 car chaque pas fait 0,5mm.
  boolean Limites_OK = true;       
  if ((DY_Positif) && (Ancien_Y >= 2020)) Limites_OK = false; // Ne pas dépasser le MAXI.
  if ((!DY_Positif) && (Ancien_Y == 0)) Limites_OK = false;      // Ne pas buter vers zéro.
  if (Limites_OK) {Eteindre_LED_debordement(); Limites_franchie = false;
     if (!STOP) {
       Moduler_le_LASER();
       if (DY_Positif) {Ancien_Y = Ancien_Y + 5; digitalWrite(Sens_sur_Y, HIGH);}
       else {Ancien_Y = Ancien_Y - 5; digitalWrite(Sens_sur_Y, LOW);}
       for (byte I=0; I<20; I++)
           {digitalWrite(Un_pas_sur_Y, HIGH); Delay_1mS(); digitalWrite(Un_pas_sur_Y, LOW);}
       if (Pourcentage > 0) delay(Delai_par_pas_elementaire);
        Couper_le_LASER();}}
  else {Allumer_LED_debordement(); Limites_franchie = true;}}

void Effectuer_un_mouvement_vectoriel(int DeltaX, int DeltaY, boolean DX_sup_a_DY) {
  boolean Fini; int NbPX, NbPY; float Cumul, Decimale;
  int Compteur; int Rapport; // Impérativement un int car peut aller jusqu'à 2250.
  // Attention ci-dessous cumul est utilisé pour un calcul préalable du rapport décimal.
  if (DX_sup_a_DY) Cumul = float(DeltaX) / float(DeltaY); else Cumul = float(DeltaY) / float(DeltaX);
  Decimale = Cumul - ((int) Cumul); Rapport = Cumul - Decimale ;
  Cumul = 0.0; NbPX=0; NbPY=0; Fini = false;
  if (DX_sup_a_DY) {
      while (!Fini) {
        Compteur = 0;
        while (Compteur < Rapport) {
           if (NbPX < DeltaX) {Faire_un_pas_sur_X(); NbPX++; Compteur++; Cumul = Cumul + Decimale;}
           else Compteur = Rapport;} // Forcer la sortie si NbPX = DeltaX.
        Faire_un_pas_sur_Y(); NbPY++; 
        if (int(Cumul + 0.01) > 0) {// Donc Cumul vaut au moins 1.
        if (NbPX < DeltaX) 
           {Faire_un_pas_sur_X(); NbPX++; Cumul = abs(Cumul - 1); if (Cumul > 1) Cumul = abs(Cumul - 1);}}
        Fini = NbPX == DeltaX;}
      while (NbPY < DeltaY) {Faire_un_pas_sur_Y(); NbPY++;}}
  else {// Si DeltaY > DeltaX il faut inverses les rôles joués par X et par Y.
      while (!Fini) {
        Compteur = 0;
        while (Compteur < Rapport) {// Si DeltaX inférieur à DeltaY il faux échanger les rôles de X et de Y.
           if (NbPY < DeltaY) {Faire_un_pas_sur_Y(); NbPY++; Compteur++; Cumul = Cumul + Decimale;}
           else Compteur = Rapport;} // Forcer la sortie si NbPY = DeltaY.
        Faire_un_pas_sur_X(); NbPX++; 
        if (int(Cumul + 0.01) > 0) {// Donc Cumul vaut au moins 1.
        if (NbPY < DeltaY) 
           {Faire_un_pas_sur_Y(); NbPY++; Cumul = abs(Cumul - 1); if (Cumul > 1) Cumul = abs(Cumul - 1);}}
        Fini = NbPY == DeltaY;}
      while (NbPX < DeltaX) {Faire_un_pas_sur_X(); NbPX++;}}}

void Traiter_les_mouvements_du_LASER(int DeltaX, int DeltaY, boolean DX_sup_a_DY) {
  int NbPX = 0;   int NbPY = 0;
  if ((DeltaX > 0) || (DeltaY > 0)) {// Ne rien faire si les deux déplacements sont nuls.
     if (DeltaX == 0) while (NbPY < DeltaY) {Faire_un_pas_sur_Y(); NbPY++;}
     if (DeltaY == 0) while (NbPX < DeltaX) {Faire_un_pas_sur_X(); NbPX++;}
     if ((DeltaX != 0) && (DeltaY != 0)) Effectuer_un_mouvement_vectoriel(DeltaX, DeltaY, DX_sup_a_DY);}}

void Effectuer_un_deplacement_gco() {
  // Ancien_X et Ancien_Y sont mis à jour par les routines qui font faire un pas aux moteurs.
  boolean DX_sup_a_DY;
  int DeltaX, DeltaY; // Nombre de pas à effectuer sur les moteurs.
  int X,Y; // Les coordonnées à atteindre exprimées en dixième de mm :
  X = int(Position_X * 10);
  Y = int(Position_Y * 10);
  DX_Positif = (X >= Ancien_X); DY_Positif = (Y >= Ancien_Y); // Détermine les sens de déplacement.
  if (DX_Positif) DeltaX = X - Ancien_X; else DeltaX = Ancien_X - X;
  if (DY_Positif) DeltaY = Y - Ancien_Y; else DeltaY = Ancien_Y - Y;
  // Ci-dessous division par 5 car un PAS moteur engendre un déplacement de 0.5mm
  DeltaX = DeltaX / 5;  DeltaY = DeltaY / 5; DX_sup_a_DY = (DeltaX >= DeltaY);
  Traiter_les_mouvements_du_LASER(DeltaX, DeltaY, DX_sup_a_DY);}   

//=================================== Procédures du Menu FICHIER ===================================
void Liste_contenu_Fichier() {
 u8g.firstPage(); do {
 u8g.setPrintPos(0, 12); u8g.print(Titre);
 u8g.setPrintPos(0, 24); Aff_TEXTE_EEPROM(415,7); u8g.print(Taille); Aff_TEXTE_EEPROM(421,7);
 u8g.setPrintPos(0, 36); Aff_TEXTE_EEPROM(186,12); u8g.print(Origine_Image_X); 
 u8g.setPrintPos(0, 48); Aff_TEXTE_EEPROM(200,12); u8g.print(Origine_Image_Y);  
 u8g.setPrintPos(0, 60); Aff_TEXTE_EEPROM(428,12); u8g.print(Taille / 1800); Aff_TEXTE_EEPROM(439,4);}
while(u8g.nextPage());}

void Affiche_ERR_et_attente_BP(int PTR, byte LGR) {Couper_le_LASER(); // Simple précaution.
  u8g.firstPage(); do {u8g.setPrintPos(5, 35); Aff_TEXTE_EEPROM(PTR,LGR);} while(u8g.nextPage());
  BIP(); ATTENDRE_un_BP();}

void Traite_X() {
  Decale_la_chaine(); Position_X = atof(Tampon); Item_suivant('S'); Pourcentage =  atoi(Tampon);}

void Prepare_degagement() {Position_X = 10.0, Position_Y = 192.0;}

void Item_suivant(char Caractere) {while (Tampon[0] != Caractere) Decale_la_chaine(); Decale_la_chaine();}

void Decale_la_chaine() {
  for (byte I = 0; I < Taille_tampon; I++) Tampon[I] = Tampon[I+1];}

void Position_degagee() {
  if (!Mode_Verification_Programme) {// Ne dégager que si mode pyrogravure.
      Couper_le_LASER(); Activer_les_moteurs(); Prepare_degagement();
      STOP = false; Effectuer_un_deplacement_gco(); Passer_en_VEILLE();}}

void Traiter_PB_Syntaxe() {
  Ecran_Noir = false; // Nécessaire pour afficher l'état machine.
  if (Type_Erreur > 0) {// Ci-dessous : Degager_le_plateau() coupe aussi le LASER.
     if (!Mode_Verification_Programme) Position_degagee(); // Ne dégage le plateau que si PYROGRAVURE.
     switch (Type_Erreur) {
       case 1 : {Affiche_ERR_et_attente_BP(286, 20); break;}
       case 2 : {Affiche_ERR_et_attente_BP(306, 20); break;}
       case 3 : Affiche_ERR_et_attente_BP(32,13);}
     AFFICHE_etat_machine(); ATTENDRE_un_BP();} // Affiche l'instruction qui a généré le problème.
  else if (Mode_Verification_Programme) {u8g.firstPage(); do { // Ci-dessous : Affiche syntaxe correcte.
           u8g.setPrintPos(0, 25); Aff_TEXTE_EEPROM(336,8); Aff_TEXTE_EEPROM(346,9);}
           while(u8g.nextPage()); ATTENDRE_un_BP();}}

void Bouge_en_manuel_et_gestion_LASER() {// Ci-dessous multiplier par 40.95 pour que 100% donne 4095 en PWM.
    if (LASER_actif) pwm.setPWM(15, 0, int(Puissance_MAX_sur_le_LASER * 40.95)); 
    Effectuer_un_deplacement_gco(); Couper_le_LASER();}

void Focaliser_le_LASER() {
  pwm.setPWM(15, 0, 41); // Allume le LASER à 1%.
  u8g.firstPage(); do {
  u8g.setPrintPos(0, 40); Aff_TEXTE_EEPROM(726,19);}
  while(u8g.nextPage());
  ATTENDRE_un_BP(); Couper_le_LASER();}

void Menu_Mode_MANUEL() { // L'économiseur d'écran n'est pas effectif pour cette option car peu utile.
  // En mode manuel c'est la puissance maximale définie dans les OPTIONS qui est appliquée au LASER.
  // >>>>>>> Attention : En manuel il n'y a aucune vérification sur les coordonnées. Arbitrairement
  // >>>>>>> elles sont forcé à zéro. Elles peuvent devenir négatives et la mécanique aller en butée.
  Fin_demandee = false; LASER_actif = false; Avance_par_pas = 0.5;
  Capturer_Origine_machine(); // Laisse X = 0 et Y = 221.
  Focaliser_le_LASER();
  while (!Fin_demandee) {
    Ecran_mode_MANUEL();
    Teste_SORTIR_du_MENU(); // Vérifie si le BP central est activé et si oui force Fin_demandee à true.
    //------------------------ Traiter la rotation du codeur incrémental ------------------------
    if (Rotation) {
        Rotation = false; // Prise en compte du traitement.
        if (Sens_positif) {Delai_par_pas_elementaire = Delai_par_pas_elementaire + 10;
                           if (Delai_par_pas_elementaire == 10010) Delai_par_pas_elementaire = 10;}
        else {if (Delai_par_pas_elementaire >= 20)
                  Delai_par_pas_elementaire = Delai_par_pas_elementaire - 10;
                  else Delai_par_pas_elementaire = 10000;}}    
    //-------------------------------------------------------------------------------------------
    Tester_le_Clavier();
    if (Num_BP !=0) {
        switch(Num_BP) {
          case 1 : {Position_X = (Position_X) - Avance_par_pas; Bouge_en_manuel_et_gestion_LASER(); break;}
          case 2 : {ATTENDRE_le_relacher();
                    LASER_actif = !LASER_actif;
                    if (LASER_actif) Pourcentage = 100; // Pourcentage utilisé ici en local au case 2.
                    else Pourcentage = 0; pwm.setPWM(12, 0, Pourcentage); break;}    
          case 3 : {Position_X = (Position_X) + Avance_par_pas; Bouge_en_manuel_et_gestion_LASER(); break;}
          case 4 : {Position_Y = (Position_Y) + Avance_par_pas; Bouge_en_manuel_et_gestion_LASER(); break;}
          case 5 : {Position_Y = (Position_Y) - Avance_par_pas; Bouge_en_manuel_et_gestion_LASER(); break;}}
        Num_BP = 0;}} // Action sur le B.P. prise en compte.
  Couper_le_LASER(); Pourcentage = 0; Prepare_degagement(); Effectuer_un_deplacement_gco(); Passer_en_VEILLE();}

void Menu_FICHIER() {BIP();} 

void Menu_OPTIONS() { // L'économiseur d'écran n'est pas effectif pour cette option car peu utile.
  byte Increment, Increment_Delai;
  Fin_demandee = false; Curseur = 2; Increment = 1; Increment_Delai = 10;
  while (!Fin_demandee) {
    Liste_les_OPTIONS();
    Teste_SORTIR_du_MENU(); // Vérifie si le BP central est activé et si oui force Fin_demandee à true.
    //------------------------ Traiter la rotation du codeur incrémental ------------------------
    if (Rotation) {
        Rotation = false; // Prise en compte du traitement.
        switch(Curseur) {
          case 2 : {if (Sens_positif) {Puissance_MAX_sur_le_LASER = Puissance_MAX_sur_le_LASER + Increment;
                                      if (Puissance_MAX_sur_le_LASER > 100) Puissance_MAX_sur_le_LASER = 1;}
                    else {Puissance_MAX_sur_le_LASER = Puissance_MAX_sur_le_LASER - Increment;
                          if (Puissance_MAX_sur_le_LASER > 200) Puissance_MAX_sur_le_LASER = 100;} break;}
                          // Ci-dessus 200 car devenant "négatif" la valeur devient égale à 254 "binaire".
          case 3 : {Afficher_en_gravant = !Afficher_en_gravant; break;}
          case 4 : {if (Sens_positif) {
                        Delai_par_pas_elementaire = Delai_par_pas_elementaire + Increment_Delai;
                        if (Delai_par_pas_elementaire > 9000) Delai_par_pas_elementaire = 10;}
                    else {Delai_par_pas_elementaire = Delai_par_pas_elementaire - Increment_Delai;
                          if (Delai_par_pas_elementaire < 10) Delai_par_pas_elementaire = 9990;}}}}  
    //-------------------------------------------------------------------------------------------
    Tester_le_Clavier();
    if (Num_BP !=0) {
        ATTENDRE_le_relacher();
        switch(Num_BP) {// Déplacer sur Origine image impose d'armer l'option en validant "Afficher_en_gravant".
          case 1 : {Increment = 1; Increment_Delai = 10; break;}
          case 2 : {Puissance_MAX_sur_le_LASER = 50; Delai_par_pas_elementaire = 5000; break;}
          case 3 : {Increment = 10; Increment_Delai = 100; break;}
          case 4 : {if (Curseur > 2) Curseur--; else Curseur = 4; break;}
          case 5 : {Curseur++; if (Curseur == 5) Curseur = 2;}
        Num_BP = 0;}}} // Action sur le B.P. prise en compte. 
    Curseur = 4;}

void Menu_SYSTEME() { // L'économiseur d'écran n'est pas effectif pour cette option car peu utile.
  Fin_demandee = false; Curseur = 2;
  while (!Fin_demandee) {
    Affiche_les_OPTIONS_SYTEME();
    //------------------------ Traiter le B.P. central du codeur rotatif ------------------------
    Teste_activation_BP_codeur_incremental();
    if (BP_central_a_traiter) {
        BP_central_a_traiter = false;
        switch(Curseur) {
          case 2 : {Tester_les_capteurs_Origine(); Eteindre_les_trois_couleurs(); Allume_LED_Bleue();
                    if (Test_Capteurs_OK) Allume_LED_verte(); else Allume_LED_Rouge(); break;}
          case 3 : {Mesurer_sur_les_capteurs_origine(); break;}
          case 4 : {Mesurer_sur_les_BP_CLAVIER(); break;}
          case 5 : Recopie_Repetiteur_LASER = !Recopie_Repetiteur_LASER;}}
    //------------------------ Traiter la rotation du codeur incrémental ------------------------
    if (Rotation) {
        Rotation = false; // Prise en compte du traitement.
       if (!Sens_positif) {Curseur++; if (Curseur == 6) Curseur = 2;}
       else {if (Curseur > 2) Curseur--; else Curseur = 5;}}}
    //-------------------------------------------------------------------------------------------
  Curseur = 5; Traiter_la_LED_triple();} // Rétablir la couleur qui était en entrée de procédure.

//============================== Routines d'affichage sur l'écran OLED ==============================
void Aff_TEXTE_EEPROM(int PTR, byte Longueur) {// Routines pour les affichages textes EEPROM.
    for (byte I=0; I<Longueur; I++) {u8g.print(char(EEPROM.read(PTR))); PTR++;}; }

void Affiche_Deux_points_espace_OUI_NON(boolean OUI) {
  if (OUI) Aff_TEXTE_EEPROM(173,5); else Aff_TEXTE_EEPROM(179,5);}

void Espace_deuxpoints_Espace() {Aff_TEXTE_EEPROM(412,3);}

void AFF_s() {u8g.print('s');}

void AFF_zero() {u8g.print('0');}

void Eteindre_Ecran(){u8g.firstPage(); do {} while(u8g.nextPage());}

void Affiche_OUI() {Aff_TEXTE_EEPROM(3,4);}
void Affiche_NON() {Aff_TEXTE_EEPROM(7,4);}

void Aff_Item_une_seconde(int PTR, byte LGR) {
  u8g.firstPage(); do {u8g.setPrintPos(0, 30); Aff_TEXTE_EEPROM(PTR, LGR);
  if (Num_BP == 3) Affiche_Deux_points_espace_OUI_NON(Afficher_en_gravant);
  if (Num_BP == 73) Affiche_Deux_points_espace_OUI_NON(Mode_pas_a_pas);} // La valeur 73 pour le B.P. est
                    // choisie arbitrarement pour différencier "Afficher_en_gravant" et "Mode_pas_a_pas".
  while(u8g.nextPage()); delay (1000); Eteindre_Ecran();}

void Retablir_Ecran_OLED() {
    switch (Menu_en_cours) {
    case 1 : {MENU_de_base(); break;}
    case 2 : {MENU_de_base(); break;}
    case 6 : {MENU_de_base(); break;}
    case 7 : {AFFICHE_etat_machine(); break;}}}

void Inverser_bascule_Ecran_Noir() {// Bascule pour éteindre ou activer l'écran OLED.
  if (Num_BP == 2) {Ecran_Noir = !Ecran_Noir; ATTENDRE_le_relacher();
      if (Ecran_Noir) Eteindre_Ecran(); else Retablir_Ecran_OLED();}}

void Positionne_le_curseur(){
  byte Y;
  switch (Curseur) {
    case 1 : {Y = 12; break;}
    case 2 : {Y = 24; break;}
    case 3 : {Y = 36; break;}
    case 4 : {Y = 48; break;}
    case 5 : Y = 60;}
    u8g.setPrintPos(0, Y); u8g.print('>');}
    
void Debut_Nom(byte X, byte Y, byte N) {u8g.setPrintPos(X, Y); Aff_TEXTE_EEPROM(0,3); u8g.print(N);}
    
void MENU_de_base(){
  Menu_en_cours = 0;
  if (!Ecran_Noir) {
      u8g.firstPage(); do {
      u8g.setPrintPos(14, 12); u8g.print("Moniteur USB");
      u8g.setPrintPos(14, 24); u8g.print("--- /// ---");
      u8g.setPrintPos(14, 36); Aff_TEXTE_EEPROM(386,11); 
      u8g.setPrintPos(14, 48); Aff_TEXTE_EEPROM(443,7);  
      u8g.setPrintPos(14, 60); Aff_TEXTE_EEPROM(450,7);
      Positionne_le_curseur();}
      while(u8g.nextPage());}}

void Ecran_mode_MANUEL(){
  Menu_en_cours = 3;
  u8g.firstPage(); do {
  u8g.setPrintPos(0, 12); Aff_TEXTE_EEPROM(286,10); Espace_deuxpoints_Espace(); u8g.print(Position_X,1);
  u8g.setPrintPos(0, 24); Aff_TEXTE_EEPROM(614,10); Espace_deuxpoints_Espace(); u8g.print(Position_Y,1);
  u8g.setPrintPos(0, 36); Aff_TEXTE_EEPROM(667,11); Espace_deuxpoints_Espace(); u8g.print(Avance_par_pas,1);
  Aff_TEXTE_EEPROM(100,2);
  u8g.setPrintPos(0, 48); Aff_TEXTE_EEPROM(471,11); Espace_deuxpoints_Espace();
  u8g.print(float(Delai_par_pas_elementaire) / 1000,2); AFF_s();
  u8g.setPrintPos(0, 60); Aff_TEXTE_EEPROM(482,5); Espace_deuxpoints_Espace();
  if (LASER_actif) {u8g.print(Puissance_MAX_sur_le_LASER); Aff_TEXTE_EEPROM(487,9); pwm.setPWM(12, 0, 4050);}
  else {Aff_TEXTE_EEPROM(181,3); pwm.setPWM(12, 0, 0);}}
  while(u8g.nextPage());}

void Trace_les_rubans() {// Placé en subroutine pour faciliter la lisibilité de Liste_les_OPTIONS().
  byte LGR_ruban;
  boolean Plage_une_seconde = (Delai_par_pas_elementaire < 1001);
  // Echelle graduée du haut :
  u8g.setPrintPos(0, 10); AFF_zero(); u8g.setPrintPos(111, 10); Aff_TEXTE_EEPROM(332,3);
  u8g.drawFrame(7,8,101,5);
  for (byte X=7; X<110; X=X+20) u8g.drawVLine(X,0,8);  // Tracer les grandes graduations.
  for (byte X=17; X<100; X=X+20) u8g.drawVLine(X,4,4); // Tracer les petites graduations.
  u8g.drawBox(8,9,Puissance_MAX_sur_le_LASER,3);
  // Echelle graduée du bas :
  u8g.setPrintPos(0, 61); AFF_zero(); u8g.setPrintPos(111, 61);
  if (Plage_une_seconde) Aff_TEXTE_EEPROM(98,2); else Aff_TEXTE_EEPROM(684,3); u8g.drawFrame(7,59,101,5);
  for (byte X=7; X<120; X=X+20) u8g.drawVLine(X,51,8);  // Tracer les grandes graduations.
  for (byte X=17; X<110; X=X+20) u8g.drawVLine(X,55,4); // Tracer les petites graduations.
  if (Plage_une_seconde) LGR_ruban = byte(Delai_par_pas_elementaire / 10);
  else LGR_ruban = byte(Delai_par_pas_elementaire / 100);
  u8g.drawBox(8,60,LGR_ruban,3);} // Tracé du ruban.

void Liste_les_OPTIONS() { // L'économiseur d'écran n'est pas effectif pour cette option car peu utile.
  u8g.firstPage(); do {
  u8g.setPrintPos(14, 24); Aff_TEXTE_EEPROM(482,6); u8g.print(Puissance_MAX_sur_le_LASER);
  Aff_TEXTE_EEPROM(678,6); u8g.setPrintPos(14, 36); Aff_TEXTE_EEPROM(496,13);
  Affiche_Deux_points_espace_OUI_NON(Afficher_en_gravant); u8g.setPrintPos(14, 48); Aff_TEXTE_EEPROM(471,5);
  Espace_deuxpoints_Espace(); u8g.print(float(Delai_par_pas_elementaire) / 1000,2); AFF_s();
  Trace_les_rubans(); Positionne_le_curseur();}
  while(u8g.nextPage());}

void Affiche_les_OPTIONS_SYTEME() { // L'économiseur d'écran n'est pas utile pour cette option.
  u8g.firstPage(); do {
  u8g.setPrintPos(0, 12); Aff_TEXTE_EEPROM(1004,20); u8g.setPrintPos(14, 24); Aff_TEXTE_EEPROM(570,17);
  u8g.setPrintPos(14, 36); Aff_TEXTE_EEPROM(587,17); u8g.setPrintPos(14, 48); Aff_TEXTE_EEPROM(687,18);
  u8g.setPrintPos(14, 60); Aff_TEXTE_EEPROM(745,8);
  if (Recopie_Repetiteur_LASER) Aff_TEXTE_EEPROM(753,10); else Aff_TEXTE_EEPROM(179,5);
  Positionne_le_curseur();} while(u8g.nextPage());}

void Attendre_une_chaine_sur_RX() {// Cette procédure attend une chaine sur la ligne USB.
  Longueur_ligne_de_code = 0; Chaine_Memorisee = ""; // Vider le tampon USB pour une nouvelle capture.
  Caractere = 'A'; // Caractère arbitraire pour "effacer" l'ancien CR.
  Tampon[0] = '\0'; // Présise un tampon vide.
  //----------------------------------------------------------------------------------------
  while (Caractere != CR) {// CR : Retour Chariot qui sera généré par le Moniteur de l'IDE.
    if (Serial.available()) {Caractere = Serial.read(); // Puiser un caractère dans le FIFO.
    // Puis concaténer ce caractère sauf le CR :
    if ((Longueur_ligne_de_code < LGR_chaine) && (Caractere != CR))
       {Chaine_Memorisee = Chaine_Memorisee + Caractere; Tampon[Longueur_ligne_de_code] = Caractere;
        if (Caractere != CR) Longueur_ligne_de_code++;}}}
  //---------------------------------------------------------------------------------------- 
  Tampon[Longueur_ligne_de_code] = '\0'; //" \0 " pour marquer la fin de chaîne.
  FIN_du_dialogue_USB = ((Chaine_Memorisee == "G0 X0 Y0") && (Ligne_M5_trouvee));}

void AFFICHE_etat_machine(){
  u8g.firstPage(); do {
      u8g.setPrintPos(0, 12); u8g.print("Pilotage USB");
      u8g.setPrintPos(0, 24); Aff_TEXTE_EEPROM(56,4); u8g.print(Position_X,1);
      u8g.setPrintPos(0, 37); Aff_TEXTE_EEPROM(60,4); u8g.print(Position_Y,1);
      u8g.setPrintPos(0, 50); Aff_TEXTE_EEPROM(52,4); u8g.print(Pourcentage); Aff_TEXTE_EEPROM(326,10);}
      while(u8g.nextPage());
  Menu_en_cours = 7;}

boolean Teste_presence_caractere(char Caractere_attendu) {
  boolean Existe = false; boolean FIN_trouvee = false;
  byte Index; Index = 0; 
  while ((!Existe) && (!FIN_trouvee)) {Existe = (Tampon[Index] == Caractere_attendu);
    FIN_trouvee = (Tampon[Index] == '\0'); Index++;}
  Serial.println(); Serial.print("Caractere '"); Serial.print(Caractere_attendu);
  if (Existe) Serial.print("' PRESENT."); else Serial.print("' NON present.");
  return Existe;}
 
void Traite_G() {
  boolean S_Present;
  boolean G1; // Sur code G0 ou G1 saisit la valeur de X dans tous les cas :
  Decale_la_chaine(); G1 = Tampon[0] == '1'; 
  Item_suivant('X'); Position_X = atof(Tampon);
  //------------------------------------ Traiter G1 ------------------------------------
  // G1 est de la forme : G1 Xn Sn ou G1 Xn Yn ou G1 Xn Yn Sn.
  if (G1) {S_Present = Teste_presence_caractere('S');
      if (Teste_presence_caractere('Y')) {Item_suivant('Y'); Position_Y = atoi(Tampon);}
      if (S_Present) {Item_suivant('S'); Pourcentage = atoi(Tampon);}}
  //------------------------------------ Traiter G0 ------------------------------------
  // Ci-dessous, on traite G0 qui est de la forme : G0 Xn Yn Sn ou G0 Xn Yn. 
  else {// Traite G0 qui par définition du gcode est un déplacement rapide LASER à 0%.
        // Traite la ligne G0 X0 Y0 finale en adoptant la configuration machine dégagée.
        Pourcentage = 0; Item_suivant('Y'); Position_Y = atof(Tampon);
        //----------------------- Traiter la ligne Origine Image ----------------------
        if (Ligne_Premier_G0) {// Seul le tout premier G0 mémorisera les coordonnées.
             Origine_Image_X = Position_X; Origine_Image_Y = Position_Y; Ligne_Premier_G0 = false;} 
        if ((Longueur_ligne_de_code == 8) && (Ligne_M5_trouvee)) Prepare_degagement();}} 
            // Prepare_degagement() : Impose les coordonnées Position_X = 10.0 et Position_Y = 192.0

void Traite_S() {Item_suivant('S'); Pourcentage = atoi(Tampon); if (Pourcentage == 0) Couper_le_LASER();}

void Analyseur_Syntaxique() {
  Type_Erreur = 0; // Avant d'analyser la ligne il n'y a pas d'erreur.
  //------------------------------------------------------------------------------- 
  switch (Tampon[0]) {  // NOTE : M3 : LASER ON,   M5 : LASER OFF.
     case 'G' : {Traite_G(); break;}
     case 'M' : {Couper_le_LASER(); // M3 et M5 forcent le LASER à puissance nulle.
                 if (Tampon[1] == '3') Activer_les_moteurs();
                 Ligne_M5_trouvee = (Tampon[1] == '5');
                 if (Ligne_M5_trouvee) {Serial.println(); Serial.print("Ligne M5 trouv");
                                        Serial.print(char(233)); Serial.print("e.");} break;}
     case 'S' : {Traite_S(); break;}
     case 'X' : {Traite_X(); break;} // Le break est impératif pour sauter "defalt" si OK.
     default  : Type_Erreur = 3;}
  //----------------------- Gère une coordonnée négative. -------------------------
  if ((Position_X < 0) || (Position_Y < 0)) Type_Erreur = 1;
  //-------------------- Gère une coordonnée > MAX possible -----------------------
  if ((Position_X > Course_MAX_sur_X) || (Position_Y > Course_MAX_sur_X)) Type_Erreur = 2;
  //-------------------------------------------------------------------------------
  if (Type_Erreur == 0) {AFFICHE_etat_machine(); Effectuer_un_deplacement_gco();}}

void Traite_la_ligne_USB() { // Traite la ligne reçue sur USB        
  //------------------------ Repérer les lignes à ignorer -----------------------
  Ligne_a_ignorer = false;
  if (Longueur_ligne_de_code == 0) Ligne_a_ignorer = true;
  if (Tampon[0] == ';') Ligne_a_ignorer = true; // Ignorer les remarques.
  if (Tampon[0] == 'F') Ligne_a_ignorer = true; // Ignorer "F1000".
  if ((Tampon[0] == 'G') && (Tampon[1] == '9')) Ligne_a_ignorer = true;
  //----------------------- Traiter les lignes pertinentes ----------------------
  if (!Ligne_a_ignorer) Analyseur_Syntaxique();
  else {Serial.print(" >>> Ligne ignor"); Serial.print(char(233)); Serial.print("e.");}} 

void Dialogue_USB() {
  FIN_du_dialogue_USB = false;
  Ecran_Noir = false; Mode_pas_a_pas = false;
  Mode_Verification_Programme = false; LASER_actif = true; Puissance_MAX_sur_le_LASER = 100;
  Serial.println(); Serial.print("Capture Origines.");
  Capturer_Origine_machine(); Serial.println(); Serial.print("Proposer des consignes USB valides :");
  AFFICHE_etat_machine(); while (Serial.available()) Caractere = Serial.read(); // Vider le tampon USB.
  // Initialisations qui se trouvaient au début de Traite_ligne_a_ligne_le_fichier();
  Ligne_Premier_G0 = false; Ligne_M5_trouvee = false; Activer_le_Relais_LASER(); 
  Type_Erreur = 0; Fin_demandee = false; Demande_FIN_active = true;
  //--------------------------------------------------------------------------------------
  while (!FIN_du_dialogue_USB) {
    Attendre_une_chaine_sur_RX();
    Serial.println(); Serial.print("Traite [");
    for (byte I = 0; I<Longueur_ligne_de_code; I++) Serial.print(Tampon[I]); Serial.print(']');
    if (FIN_du_dialogue_USB) {
        Serial.println(); Serial.print("FIN du pilotage USB"); // Prévenir avant Position_degagee().
        Serial.println(); Serial.print("Mouvement pour Machine d"); Serial.print(char(233));
        Serial.print("gag"); Serial.print(char(233)); Serial.print("e."); BIP();}
    Traite_la_ligne_USB(); AFFICHE_etat_machine();}
  //--------------------------------------------------------------------------------------
  Couper_le_Relais_LASER(); Position_degagee(); Passer_en_VEILLE();}
 

